@@ -1585,8 +1585,9 @@ njs_engine_njs_complete(njs_engine_t *engine, njs_str_t *expression)
             return NULL;
         }
 
-        ret = njs_value_property(vm, njs_value_arg(&value), njs_value_arg(&key),
-                                 njs_value_arg(&retval));
+        ret = njs_value_property_val(vm, njs_value_arg(&value),
+                                     njs_value_arg(&key),
+                                     njs_value_arg(&retval));
         if (njs_slow_path(ret != NJS_OK)) {
             if (ret == NJS_DECLINED && !global) {
                 goto done;
@@ -1901,13 +1902,6 @@ njs_qjs_clear_timeout(JSContext *ctx, JSValueConst this_val, int argc,
 }
 
 
-static JSValue
-njs_qjs_console_to_string_tag(JSContext *ctx, JSValueConst this_val)
-{
-    return JS_NewString(ctx, "Console");
-}
-
-
 static JSValue
 njs_qjs_process_getter(JSContext *ctx, JSValueConst this_val)
 {
@@ -2487,7 +2481,8 @@ static const JSCFunctionListEntry njs_qjs_global_proto[] = {
 
 
 static const JSCFunctionListEntry njs_qjs_console_proto[] = {
-    JS_CGETSET_DEF("[Symbol.toStringTag]", njs_qjs_console_to_string_tag, NULL),
+    JS_PROP_STRING_DEF("[Symbol.toStringTag]", "Console",
+                       JS_PROP_CONFIGURABLE),
     JS_CFUNC_MAGIC_DEF("error", 0, njs_qjs_console_log, NJS_LOG_ERROR),
     JS_CFUNC_MAGIC_DEF("info", 0, njs_qjs_console_log, NJS_LOG_INFO),
     JS_CFUNC_MAGIC_DEF("log", 0, njs_qjs_console_log, NJS_LOG_INFO),
@@ -2590,7 +2585,7 @@ njs_qjs_module_loader(JSContext *ctx, const char *module_name, void *opaque)
     (void) close(info.fd);
 
     if (njs_slow_path(ret != NJS_OK)) {
-        JS_ThrowInternalError(ctx, "while reading \"%*s\" module",
+        JS_ThrowInternalError(ctx, "while reading \"%.*s\" module",
                               (int) info.file.length, info.file.start);
         return NULL;
     }
@@ -2599,7 +2594,7 @@ njs_qjs_module_loader(JSContext *ctx, const char *module_name, void *opaque)
 
     ret = njs_console_set_cwd(console, &info.file);
     if (njs_slow_path(ret != NJS_OK)) {
-        JS_ThrowInternalError(ctx, "while setting cwd for \"%*s\" module",
+        JS_ThrowInternalError(ctx, "while setting cwd for \"%.*s\" module",
                               (int) info.file.length, info.file.start);
         return NULL;
     }
@@ -2827,7 +2822,7 @@ njs_engine_qjs_unhandled_rejection(njs_engine_t *engine)
         return -1;
     }
 
-    JS_ThrowTypeError(ctx, "unhandled promise rejection: %*s", (int) len, str);
+    JS_ThrowTypeError(ctx, "unhandled promise rejection: %.*s", (int) len, str);
     JS_FreeCString(ctx, str);
 
     for (i = 0; i < console->rejected_promises->items; i++) {
@@ -2956,7 +2951,6 @@ njs_qjs_object_completions(njs_engine_t *engine, JSContext *ctx,
 
         for (n = 0; n < length; n++) {
             key.start = (u_char *) JS_AtomToCString(ctx, ptab[n].atom);
-            JS_FreeAtom(ctx, ptab[n].atom);
             if (njs_slow_path(key.start == NULL)) {
                 goto fail;
             }
@@ -2999,7 +2993,7 @@ next:
             JS_FreeCString(ctx, (const char *) key.start);
         }
 
-        js_free_rt(JS_GetRuntime(ctx), ptab);
+        qjs_free_prop_enum(ctx, ptab, length);
 
         prototype = JS_GetPrototype(ctx, object);
         if (JS_IsException(prototype)) {
@@ -3023,7 +3017,7 @@ fail:
     }
 
     if (ptab != NULL) {
-        js_free_rt(JS_GetRuntime(ctx), ptab);
+        qjs_free_prop_enum(ctx, ptab, length);
     }
 
     JS_FreeValue(ctx, object);
@@ -3671,11 +3665,11 @@ njs_ext_console_time(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
                 return ret;
             }
 
-            njs_value_string_get(value, &name);
+            njs_value_string_get(vm, value, &name);
         }
 
     } else {
-        njs_value_string_get(value, &name);
+        njs_value_string_get(vm, value, &name);
     }
 
     if (njs_console_time(console, &name) != NJS_OK) {
@@ -3720,11 +3714,11 @@ njs_ext_console_time_end(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
                 return ret;
             }
 
-            njs_value_string_get(value, &name);
+            njs_value_string_get(vm, value, &name);
         }
 
     } else {
-        njs_value_string_get(value, &name);
+        njs_value_string_get(vm, value, &name);
     }
 
     njs_console_time_end(console, &name, ns);
