@@ -120,8 +120,8 @@ static ngx_stream_js_ev_t *ngx_stream_js_event(ngx_stream_session_t *s,
     njs_str_t *event);
 
 static njs_int_t ngx_stream_js_ext_get_remote_address(njs_vm_t *vm,
-    njs_object_prop_t *prop, njs_value_t *value, njs_value_t *setval,
-    njs_value_t *retval);
+    njs_object_prop_t *prop, uint32_t unused, njs_value_t *value,
+    njs_value_t *setval, njs_value_t *retval);
 
 static njs_int_t ngx_stream_js_ext_done(njs_vm_t *vm, njs_value_t *args,
      njs_uint_t nargs, njs_index_t unused, njs_value_t *retval);
@@ -137,16 +137,14 @@ static njs_int_t ngx_stream_js_ext_set_return_value(njs_vm_t *vm,
     njs_value_t *retval);
 
 static njs_int_t ngx_stream_js_ext_variables(njs_vm_t *vm,
-    njs_object_prop_t *prop, njs_value_t *value, njs_value_t *setval,
-    njs_value_t *retval);
+    njs_object_prop_t *prop, uint32_t atom_id, njs_value_t *value,
+    njs_value_t *setval, njs_value_t *retval);
 static njs_int_t ngx_stream_js_periodic_variables(njs_vm_t *vm,
-    njs_object_prop_t *prop, njs_value_t *value, njs_value_t *setval,
-    njs_value_t *retval);
+    njs_object_prop_t *prop, uint32_t atom_id, njs_value_t *value,
+    njs_value_t *setval, njs_value_t *retval);
 
 #if (NJS_HAVE_QUICKJS)
 
-static JSValue ngx_stream_qjs_ext_to_string_tag(JSContext *cx,
-    JSValueConst this_val);
 static JSValue ngx_stream_qjs_ext_done(JSContext *cx, JSValueConst this_val,
     int argc, JSValueConst *argv, int magic);
 static JSValue ngx_stream_qjs_ext_log(JSContext *cx, JSValueConst this_val,
@@ -155,8 +153,6 @@ static JSValue ngx_stream_qjs_ext_on(JSContext *cx, JSValueConst this_val,
     int argc, JSValueConst *argv);
 static JSValue ngx_stream_qjs_ext_off(JSContext *cx, JSValueConst this_val,
     int argc, JSValueConst *argv);
-static JSValue ngx_stream_qjs_ext_periodic_to_string_tag(JSContext *cx,
-    JSValueConst this_val);
 static JSValue ngx_stream_qjs_ext_periodic_variables(JSContext *cx,
     JSValueConst this_val, int type);
 static JSValue ngx_stream_qjs_ext_remote_address(JSContext *cx,
@@ -191,6 +187,7 @@ static ngx_stream_session_t *ngx_stream_qjs_session(JSValueConst val);
 static JSValue ngx_stream_qjs_session_make(JSContext *cx, ngx_int_t proto_id,
     ngx_stream_session_t *s);
 static void ngx_stream_qjs_session_finalizer(JSRuntime *rt, JSValue val);
+static void ngx_stream_qjs_periodic_finalizer(JSRuntime *rt, JSValue val);
 
 #endif
 
@@ -267,6 +264,13 @@ static ngx_command_t  ngx_stream_js_commands[] = {
       offsetof(ngx_stream_js_srv_conf_t, reuse),
       NULL },
 
+    { ngx_string("js_context_reuse_max_size"),
+      NGX_STREAM_MAIN_CONF|NGX_STREAM_SRV_CONF|NGX_CONF_TAKE1,
+      ngx_conf_set_size_slot,
+      NGX_STREAM_SRV_CONF_OFFSET,
+      offsetof(ngx_stream_js_srv_conf_t, reuse_max_size),
+      NULL },
+
     { ngx_string("js_import"),
       NGX_STREAM_MAIN_CONF|NGX_STREAM_SRV_CONF|NGX_CONF_TAKE13,
       ngx_js_import,
@@ -761,8 +765,8 @@ njs_module_t *njs_stream_js_addon_modules[] = {
 #if (NJS_HAVE_QUICKJS)
 
 static const JSCFunctionListEntry ngx_stream_qjs_ext_session[] = {
-    JS_CGETSET_DEF("[Symbol.toStringTag]", ngx_stream_qjs_ext_to_string_tag,
-                   NULL),
+    JS_PROP_STRING_DEF("[Symbol.toStringTag]", "Stream Session",
+                       JS_PROP_CONFIGURABLE),
     JS_CFUNC_MAGIC_DEF("allow", 1, ngx_stream_qjs_ext_done, NGX_OK),
     JS_CFUNC_MAGIC_DEF("decline", 1, ngx_stream_qjs_ext_done, -NGX_DECLINED),
     JS_CFUNC_MAGIC_DEF("deny", 1, ngx_stream_qjs_ext_done, -NGX_DONE),
@@ -789,8 +793,8 @@ static const JSCFunctionListEntry ngx_stream_qjs_ext_session[] = {
 
 
 static const JSCFunctionListEntry ngx_stream_qjs_ext_periodic[] = {
-    JS_CGETSET_DEF("[Symbol.toStringTag]",
-                   ngx_stream_qjs_ext_periodic_to_string_tag, NULL),
+    JS_PROP_STRING_DEF("[Symbol.toStringTag]", "PeriodicSession",
+                       JS_PROP_CONFIGURABLE),
     JS_CGETSET_MAGIC_DEF("rawVariables", ngx_stream_qjs_ext_periodic_variables,
                    NULL, NGX_JS_BUFFER),
     JS_CGETSET_MAGIC_DEF("variables", ngx_stream_qjs_ext_periodic_variables,
@@ -813,7 +817,7 @@ static JSClassDef ngx_stream_qjs_session_class = {
 
 static JSClassDef ngx_stream_qjs_periodic_class = {
     "Periodic",
-    .finalizer = NULL,
+    .finalizer = ngx_stream_qjs_periodic_finalizer,
 };
 
 
@@ -836,10 +840,18 @@ static JSClassDef ngx_stream_qjs_variables_class = {
 
 qjs_module_t *njs_stream_qjs_addon_modules[] = {
     &ngx_qjs_ngx_module,
+    &ngx_qjs_ngx_shared_dict_module,
+    &ngx_qjs_ngx_fetch_module,
     /*
      * Shared addons should be in the same order and the same positions
      * in all nginx modules.
      */
+#ifdef NJS_HAVE_OPENSSL
+    &qjs_webcrypto_module,
+#endif
+#ifdef NJS_HAVE_XML
+    &qjs_xml_module,
+#endif
 #ifdef NJS_HAVE_ZLIB
     &qjs_zlib_module,
 #endif
@@ -1401,8 +1413,8 @@ ngx_stream_js_event(ngx_stream_session_t *s, njs_str_t *event)
 
 static njs_int_t
 ngx_stream_js_ext_get_remote_address(njs_vm_t *vm,
-    njs_object_prop_t *prop, njs_value_t *value, njs_value_t *setval,
-    njs_value_t *retval)
+    njs_object_prop_t *prop, uint32_t unused, njs_value_t *value,
+    njs_value_t *setval, njs_value_t *retval)
 {
     ngx_connection_t      *c;
     ngx_stream_session_t  *s;
@@ -1704,7 +1716,8 @@ ngx_stream_js_ext_set_return_value(njs_vm_t *vm, njs_value_t *args,
 
 static njs_int_t
 ngx_stream_js_session_variables(njs_vm_t *vm, njs_object_prop_t *prop,
-    ngx_stream_session_t *s, njs_value_t *setval, njs_value_t *retval)
+    uint32_t atom_id, ngx_stream_session_t *s, njs_value_t *setval,
+    njs_value_t *retval)
 {
     njs_int_t                     rc;
     njs_str_t                     val;
@@ -1713,18 +1726,29 @@ ngx_stream_js_session_variables(njs_vm_t *vm, njs_object_prop_t *prop,
     ngx_stream_variable_t        *v;
     ngx_stream_core_main_conf_t  *cmcf;
     ngx_stream_variable_value_t  *vv;
+    u_char                        storage[64];
 
-    rc = njs_vm_prop_name(vm, prop, &val);
+    rc = njs_vm_prop_name(vm, atom_id, &val);
     if (rc != NJS_OK) {
         njs_value_undefined_set(retval);
         return NJS_DECLINED;
     }
 
-    name.data = val.start;
-    name.len = val.length;
-
     if (setval == NULL) {
-        key = ngx_hash_strlow(name.data, name.data, name.len);
+        if (val.length < sizeof(storage)) {
+            name.data = storage;
+
+        } else {
+            name.data = ngx_pnalloc(s->connection->pool, val.length);
+            if (name.data == NULL) {
+                njs_vm_error(vm, "internal error");
+                return NJS_ERROR;
+            }
+        }
+
+        name.len = val.length;
+
+        key = ngx_hash_strlow(name.data, val.start, val.length);
 
         vv = ngx_stream_get_variable(s, &name, key);
         if (vv == NULL || vv->not_found) {
@@ -1738,9 +1762,20 @@ ngx_stream_js_session_variables(njs_vm_t *vm, njs_object_prop_t *prop,
 
     cmcf = ngx_stream_get_module_main_conf(s, ngx_stream_core_module);
 
-    key = ngx_hash_strlow(name.data, name.data, name.len);
+    if (val.length < sizeof(storage)) {
+        name.data = storage;
 
-    v = ngx_hash_find(&cmcf->variables_hash, key, name.data, name.len);
+    } else {
+        name.data = ngx_pnalloc(s->connection->pool, val.length);
+        if (name.data == NULL) {
+            njs_vm_error(vm, "internal error");
+            return NJS_ERROR;
+        }
+    }
+
+    key = ngx_hash_strlow(name.data, val.start, val.length);
+
+    v = ngx_hash_find(&cmcf->variables_hash, key, name.data, val.length);
 
     if (v == NULL) {
         njs_vm_error(vm, "variable not found");
@@ -1792,7 +1827,8 @@ ngx_stream_js_session_variables(njs_vm_t *vm, njs_object_prop_t *prop,
 
 static njs_int_t
 ngx_stream_js_ext_variables(njs_vm_t *vm, njs_object_prop_t *prop,
-    njs_value_t *value, njs_value_t *setval, njs_value_t *retval)
+    uint32_t atom_id, njs_value_t *value, njs_value_t *setval,
+    njs_value_t *retval)
 {
     ngx_stream_session_t  *s;
 
@@ -1802,13 +1838,14 @@ ngx_stream_js_ext_variables(njs_vm_t *vm, njs_object_prop_t *prop,
         return NJS_DECLINED;
     }
 
-    return ngx_stream_js_session_variables(vm, prop, s, setval, retval);
+    return ngx_stream_js_session_variables(vm, prop, atom_id, s, setval, retval);
 }
 
 
 static njs_int_t
 ngx_stream_js_periodic_variables(njs_vm_t *vm, njs_object_prop_t *prop,
-    njs_value_t *value, njs_value_t *setval, njs_value_t *retval)
+    uint32_t atom_id, njs_value_t *value, njs_value_t *setval,
+    njs_value_t *retval)
 {
     ngx_stream_session_t  *s;
 
@@ -1818,7 +1855,7 @@ ngx_stream_js_periodic_variables(njs_vm_t *vm, njs_object_prop_t *prop,
         return NJS_DECLINED;
     }
 
-    return ngx_stream_js_session_variables(vm, prop, s, setval, retval);
+    return ngx_stream_js_session_variables(vm, prop, atom_id, s, setval, retval);
 }
 
 
@@ -1888,7 +1925,7 @@ static void
 ngx_stream_js_event_finalize(ngx_stream_session_t *s, ngx_int_t rc)
 {
     ngx_log_debug1(NGX_LOG_DEBUG_STREAM, s->connection->log, 0,
-                   "http js event finalize rc: %i", rc);
+                   "stream js event finalize rc: %i", rc);
 
     if (rc == NGX_ERROR) {
         if (s->health_check) {
@@ -1971,13 +2008,6 @@ ngx_engine_njs_clone(ngx_js_ctx_t *ctx, ngx_js_loc_conf_t *cf,
 
 #if (NJS_HAVE_QUICKJS)
 
-static JSValue
-ngx_stream_qjs_ext_to_string_tag(JSContext *cx, JSValueConst this_val)
-{
-    return JS_NewString(cx, "Stream Session");
-}
-
-
 static JSValue
 ngx_stream_qjs_ext_done(JSContext *cx, JSValueConst this_val, int argc,
     JSValueConst *argv, int magic)
@@ -2100,7 +2130,7 @@ ngx_stream_qjs_event(ngx_stream_session_t *s, JSContext *cx, ngx_str_t *event)
     }
 
     if (i == n) {
-        (void) JS_ThrowInternalError(cx, "unknown event \"%*s\"",
+        (void) JS_ThrowInternalError(cx, "unknown event \"%.*s\"",
                                      (int) event->len, event->data);
         return NULL;
     }
@@ -2136,7 +2166,7 @@ ngx_stream_qjs_ext_on(JSContext *cx, JSValueConst this_val, int argc,
 
     ctx = ngx_stream_get_module_ctx(ses->session, ngx_stream_js_module);
 
-    if (ngx_qjs_string(ctx->engine, argv[0], &name) != NGX_OK) {
+    if (ngx_qjs_string(cx, argv[0], &name) != NGX_OK) {
         return JS_EXCEPTION;
     }
 
@@ -2179,7 +2209,7 @@ ngx_stream_qjs_ext_off(JSContext *cx, JSValueConst this_val, int argc,
 
     ctx = ngx_stream_get_module_ctx(s, ngx_stream_js_module);
 
-    if (ngx_qjs_string(ctx->engine, argv[0], &name) != NGX_OK) {
+    if (ngx_qjs_string(cx, argv[0], &name) != NGX_OK) {
         return JS_EXCEPTION;
     }
 
@@ -2195,14 +2225,6 @@ ngx_stream_qjs_ext_off(JSContext *cx, JSValueConst this_val, int argc,
 }
 
 
-static JSValue
-ngx_stream_qjs_ext_periodic_to_string_tag(JSContext *cx,
-    JSValueConst this_val)
-{
-    return JS_NewString(cx, "PeriodicSession");
-}
-
-
 static JSValue
 ngx_stream_qjs_ext_periodic_variables(JSContext *cx,
     JSValueConst this_val, int type)
@@ -2270,7 +2292,7 @@ ngx_stream_qjs_ext_send(JSContext *cx, JSValueConst this_val, int argc,
         return JS_ThrowInternalError(cx, "cannot send buffer in this handler");
     }
 
-    if (ngx_qjs_string(ctx->engine, argv[0], &buffer) != NGX_OK) {
+    if (ngx_qjs_string(cx, argv[0], &buffer) != NGX_OK) {
         return JS_EXCEPTION;
     }
 
@@ -2441,10 +2463,11 @@ ngx_stream_qjs_variables_own_property(JSContext *cx,
     JSPropertyDescriptor *pdesc, JSValueConst obj, JSAtom prop)
 {
     uint32_t                      buffer_type;
-    ngx_str_t                     name;
+    ngx_str_t                     name, name_lc;
     ngx_uint_t                    key;
     ngx_stream_session_t         *s;
     ngx_stream_variable_value_t  *vv;
+    u_char                        storage[64];
 
     s = JS_GetOpaque(obj, NGX_QJS_CLASS_ID_STREAM_VARS);
 
@@ -2463,9 +2486,22 @@ ngx_stream_qjs_variables_own_property(JSContext *cx,
 
     name.len = ngx_strlen(name.data);
 
-    key = ngx_hash_strlow(name.data, name.data, name.len);
+    if (name.len < sizeof(storage)) {
+        name_lc.data = storage;
 
-    vv = ngx_stream_get_variable(s, &name, key);
+    } else {
+        name_lc.data = ngx_pnalloc(s->connection->pool, name.len);
+        if (name_lc.data == NULL) {
+            (void) JS_ThrowOutOfMemory(cx);
+            return -1;
+        }
+    }
+
+    name_lc.len = name.len;
+
+    key = ngx_hash_strlow(name_lc.data, name.data, name.len);
+
+    vv = ngx_stream_get_variable(s, &name_lc, key);
     JS_FreeCString(cx, (char *) name.data);
     if (vv == NULL || vv->not_found) {
         return 0;
@@ -2486,13 +2522,13 @@ static int
 ngx_stream_qjs_variables_set_property(JSContext *cx, JSValueConst obj,
     JSAtom prop, JSValueConst value, JSValueConst receiver, int flags)
 {
-    ngx_str_t                     name, val;
+    ngx_str_t                     name, name_lc, val;
     ngx_uint_t                    key;
-    ngx_js_ctx_t                 *ctx;
     ngx_stream_session_t         *s;
     ngx_stream_variable_t        *v;
     ngx_stream_variable_value_t  *vv;
     ngx_stream_core_main_conf_t  *cmcf;
+    u_char                        storage[64];
 
     s = JS_GetOpaque(obj, NGX_QJS_CLASS_ID_STREAM_VARS);
 
@@ -2510,11 +2546,22 @@ ngx_stream_qjs_variables_set_property(JSContext *cx, JSValueConst obj,
 
     name.len = ngx_strlen(name.data);
 
-    key = ngx_hash_strlow(name.data, name.data, name.len);
+    if (name.len < sizeof(storage)) {
+        name_lc.data = storage;
+
+    } else {
+        name_lc.data = ngx_pnalloc(s->connection->pool, name.len);
+        if (name_lc.data == NULL) {
+            (void) JS_ThrowOutOfMemory(cx);
+            return -1;
+        }
+    }
+
+    key = ngx_hash_strlow(name_lc.data, name.data, name.len);
 
     cmcf = ngx_stream_get_module_main_conf(s, ngx_stream_core_module);
 
-    v = ngx_hash_find(&cmcf->variables_hash, key, name.data, name.len);
+    v = ngx_hash_find(&cmcf->variables_hash, key, name_lc.data, name.len);
     JS_FreeCString(cx, (char *) name.data);
 
     if (v == NULL) {
@@ -2522,9 +2569,7 @@ ngx_stream_qjs_variables_set_property(JSContext *cx, JSValueConst obj,
         return -1;
     }
 
-    ctx = ngx_stream_get_module_ctx(s, ngx_stream_js_module);
-
-    if (ngx_qjs_string(ctx->engine, value, &val) != NGX_OK) {
+    if (ngx_qjs_string(cx, value, &val) != NGX_OK) {
         return -1;
     }
 
@@ -2635,8 +2680,7 @@ ngx_stream_qjs_run_event(ngx_stream_session_t *s, ngx_stream_js_ctx_t *ctx,
 
     JS_SetOpaque(argv[1], (void *) flags);
 
-    rc = ngx_qjs_call((ngx_js_ctx_t *) ctx, ngx_qjs_arg(event->function),
-                      &argv[0], 2);
+    rc = ngx_qjs_call(cx, ngx_qjs_arg(event->function), &argv[0], 2);
     JS_FreeValue(cx, argv[0]);
     JS_FreeValue(cx, argv[1]);
 
@@ -2760,6 +2804,20 @@ ngx_stream_qjs_session_finalizer(JSRuntime *rt, JSValue val)
 }
 
 
+static void
+ngx_stream_qjs_periodic_finalizer(JSRuntime *rt, JSValue val)
+{
+    ngx_stream_qjs_session_t  *ses;
+
+    ses = JS_GetOpaque(val, NGX_QJS_CLASS_ID_STREAM_PERIODIC);
+    if (ses == NULL) {
+        return;
+    }
+
+    js_free_rt(rt, ses);
+}
+
+
 static ngx_engine_t *
 ngx_engine_qjs_clone(ngx_js_ctx_t *ctx, ngx_js_loc_conf_t *cf,
     njs_int_t proto_id, void *external)
@@ -3141,21 +3199,12 @@ ngx_stream_js_periodic_init(ngx_js_periodic_t *periodic)
 
 
 static ngx_int_t
-ngx_stream_js_init_worker(ngx_cycle_t *cycle)
+ngx_stream_js_init_worker_periodics(ngx_js_main_conf_t *jmcf)
 {
-    ngx_uint_t           i;
-    ngx_js_periodic_t   *periodics;
-    ngx_js_main_conf_t  *jmcf;
+    ngx_uint_t          i;
+    ngx_js_periodic_t  *periodics;
 
-    if ((ngx_process != NGX_PROCESS_WORKER)
-        && ngx_process != NGX_PROCESS_SINGLE)
-    {
-        return NGX_OK;
-    }
-
-    jmcf = ngx_stream_cycle_get_module_main_conf(cycle, ngx_stream_js_module);
-
-    if (jmcf == NULL || jmcf->periodics == NULL) {
+    if (jmcf->periodics == NULL) {
         return NGX_OK;
     }
 
@@ -3183,6 +3232,35 @@ ngx_stream_js_init_worker(ngx_cycle_t *cycle)
 }
 
 
+static ngx_int_t
+ngx_stream_js_init_worker(ngx_cycle_t *cycle)
+{
+    ngx_js_main_conf_t  *jmcf;
+
+    if ((ngx_process != NGX_PROCESS_WORKER)
+        && ngx_process != NGX_PROCESS_SINGLE)
+    {
+        return NGX_OK;
+    }
+
+    jmcf = ngx_stream_cycle_get_module_main_conf(cycle, ngx_stream_js_module);
+
+    if (jmcf == NULL) {
+        return NGX_OK;
+    }
+
+    if (ngx_stream_js_init_worker_periodics(jmcf) != NGX_OK) {
+        return NGX_ERROR;
+    }
+
+    if (ngx_js_dict_init_worker(jmcf) != NGX_OK) {
+        return NGX_ERROR;
+    }
+
+    return NGX_OK;
+}
+
+
 static char *
 ngx_stream_js_periodic(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
 {
