@@ -11,8 +11,8 @@
 
 #include <njs_auto_config.h>
 
-#define NJS_VERSION                 "0.8.9"
-#define NJS_VERSION_NUMBER          0x000809
+#define NJS_VERSION                 "0.9.1"
+#define NJS_VERSION_NUMBER          0x000901
 
 
 #include <string.h>
@@ -34,6 +34,7 @@ typedef struct njs_function_s         njs_function_t;
 typedef struct njs_vm_shared_s        njs_vm_shared_t;
 typedef struct njs_object_init_s      njs_object_init_t;
 typedef struct njs_object_prop_s      njs_object_prop_t;
+typedef struct njs_object_prop_init_s njs_object_prop_init_t;
 typedef struct njs_object_type_init_s njs_object_type_init_t;
 typedef struct njs_external_s         njs_external_t;
 
@@ -43,7 +44,7 @@ typedef struct njs_external_s         njs_external_t;
  */
 
 typedef struct {
-    uint64_t                        filler[2];
+    uint32_t                        filler[4];
 } njs_opaque_value_t;
 
 /* sizeof(njs_value_t) is 16 bytes. */
@@ -61,6 +62,11 @@ extern const njs_value_t            njs_value_undefined;
     memcpy(dst, src, sizeof(njs_opaque_value_t))
 
 #define njs_value_arg(val) ((njs_value_t *) val)
+#define njs_value_atom(val) (((njs_opaque_value_t *) (val))->filler[0])
+
+#define njs_atom_is_number(atom_id) ((atom_id) & 0x80000000)
+#define njs_atom_number(atom_id) ((atom_id) & 0x7FFFFFFF)
+#define njs_number_atom(n) ((n) | 0x80000000)
 
 #define njs_lvalue_arg(lvalue, args, nargs, n)                                \
     ((n < nargs) ? njs_argument(args, n)                                      \
@@ -104,7 +110,8 @@ extern const njs_value_t            njs_value_undefined;
  *   the exception value.
  */
 typedef njs_int_t (*njs_prop_handler_t) (njs_vm_t *vm, njs_object_prop_t *prop,
-    njs_value_t *value, njs_value_t *setval, njs_value_t *retval);
+    uint32_t atom_id, njs_value_t *value, njs_value_t *setval,
+    njs_value_t *retval);
 typedef njs_int_t (*njs_exotic_keys_t)(njs_vm_t *vm, njs_value_t *value,
     njs_value_t *retval);
 typedef njs_int_t (*njs_function_native_t) (njs_vm_t *vm, njs_value_t *args,
@@ -180,7 +187,7 @@ struct njs_external_s {
 
     union {
         struct {
-            const char              value[15]; /* NJS_STRING_SHORT + 1. */
+            const char              *value;
             njs_prop_handler_t      handler;
             uint16_t                magic16;
             uint32_t                magic32;
@@ -354,23 +361,24 @@ NJS_EXPORT njs_int_t njs_vm_external_create(njs_vm_t *vm, njs_value_t *value,
 NJS_EXPORT njs_external_ptr_t njs_vm_external(njs_vm_t *vm,
     njs_int_t proto_id, const njs_value_t *value);
 NJS_EXPORT njs_int_t njs_external_property(njs_vm_t *vm,
-    njs_object_prop_t *prop, njs_value_t *value, njs_value_t *setval,
-    njs_value_t *retval);
+    njs_object_prop_t *prop, uint32_t unused, njs_value_t *value,
+    njs_value_t *setval, njs_value_t *retval);
+NJS_EXPORT njs_int_t njs_atom_atomize_key(njs_vm_t *vm, njs_value_t *value);
 NJS_EXPORT njs_int_t njs_value_property(njs_vm_t *vm, njs_value_t *value,
-    njs_value_t *key, njs_value_t *retval);
+    uint32_t atom_id, njs_value_t *retval);
 NJS_EXPORT njs_int_t njs_value_property_set(njs_vm_t *vm, njs_value_t *value,
-    njs_value_t *key, njs_value_t *setval);
+    uint32_t atom_id, njs_value_t *setval);
 NJS_EXPORT uintptr_t njs_vm_meta(njs_vm_t *vm, njs_uint_t index);
 NJS_EXPORT njs_vm_opt_t *njs_vm_options(njs_vm_t *vm);
 
 NJS_EXPORT njs_int_t njs_error_constructor(njs_vm_t *vm, njs_value_t *args,
     njs_uint_t nargs, njs_index_t type, njs_value_t *retval);
 NJS_EXPORT njs_int_t njs_object_prototype_create_constructor(njs_vm_t *vm,
-    njs_object_prop_t *prop, njs_value_t *value, njs_value_t *setval,
-    njs_value_t *retval);
+    njs_object_prop_t *prop, uint32_t unused, njs_value_t *value,
+    njs_value_t *setval, njs_value_t *retval);
 NJS_EXPORT njs_int_t njs_object_prototype_create(njs_vm_t *vm,
-    njs_object_prop_t *prop, njs_value_t *value, njs_value_t *setval,
-    njs_value_t *retval);
+    njs_object_prop_t *prop, uint32_t unused, njs_value_t *value,
+    njs_value_t *setval, njs_value_t *retval);
 
 NJS_EXPORT njs_function_t *njs_vm_function_alloc(njs_vm_t *vm,
     njs_function_native_t native, njs_bool_t shared, njs_bool_t ctor);
@@ -403,12 +411,13 @@ NJS_EXPORT njs_int_t njs_value_to_integer(njs_vm_t *vm, njs_value_t *value,
     int64_t *dst);
 
 /*  Gets string value, no copy. */
-NJS_EXPORT void njs_value_string_get(njs_value_t *value, njs_str_t *dst);
+NJS_EXPORT void njs_value_string_get(njs_vm_t *vm, njs_value_t *value,
+    njs_str_t *dst);
 NJS_EXPORT njs_int_t njs_vm_value_string_create(njs_vm_t *vm,
     njs_value_t *value, const u_char *start, uint32_t size);
 NJS_EXPORT njs_int_t njs_vm_value_string_create_chb(njs_vm_t *vm,
     njs_value_t *value, njs_chb_t *chain);
-NJS_EXPORT njs_int_t njs_vm_string_compare(const njs_value_t *v1,
+NJS_EXPORT njs_int_t njs_vm_string_compare(njs_vm_t *vm, const njs_value_t *v1,
     const njs_value_t *v2);
 
 NJS_EXPORT njs_int_t njs_vm_value_array_buffer_set(njs_vm_t *vm,
@@ -477,7 +486,7 @@ NJS_EXPORT njs_int_t njs_value_external_tag(const njs_value_t *value);
 
 NJS_EXPORT uint16_t njs_vm_prop_magic16(njs_object_prop_t *prop);
 NJS_EXPORT uint32_t njs_vm_prop_magic32(njs_object_prop_t *prop);
-NJS_EXPORT njs_int_t njs_vm_prop_name(njs_vm_t *vm, njs_object_prop_t *prop,
+NJS_EXPORT njs_int_t njs_vm_prop_name(njs_vm_t *vm, uint32_t atom_id,
     njs_str_t *dst);
 
 NJS_EXPORT njs_int_t njs_value_is_null(const njs_value_t *value);
@@ -536,4 +545,49 @@ NJS_EXPORT njs_int_t njs_vm_promise_create(njs_vm_t *vm, njs_value_t *retval,
     njs_value_t *callbacks);
 
 
+njs_inline njs_int_t
+njs_value_property_val(njs_vm_t *vm, njs_value_t *value, njs_value_t *key,
+    njs_value_t *retval)
+{
+    njs_int_t  ret;
+
+    if (njs_value_atom(key) == 0 /* NJS_ATOM_STRING_unknown */) {
+        ret = njs_atom_atomize_key(vm, key);
+        if (ret != NJS_OK) {
+            return ret;
+        }
+    }
+
+    return njs_value_property(vm, value, njs_value_atom(key), retval);
+}
+
+
+njs_inline njs_int_t
+njs_value_property_val_set(njs_vm_t *vm, njs_value_t *value, njs_value_t *key,
+    njs_value_t *setval)
+{
+    njs_int_t  ret;
+
+    if (njs_value_atom(key) == 0 /* NJS_ATOM_STRING_unknown */) {
+        ret = njs_atom_atomize_key(vm, key);
+        if (ret != NJS_OK) {
+            return ret;
+        }
+    }
+
+    return njs_value_property_set(vm, value, njs_value_atom(key), setval);
+}
+
+
+njs_inline size_t
+njs_value_string_length(njs_vm_t *vm, njs_value_t *value)
+{
+    njs_str_t  str;
+
+    njs_value_string_get(vm, value, &str);
+
+    return str.length;
+}
+
+
 #endif /* _NJS_H_INCLUDED_ */
