@@ -100,7 +100,7 @@ njs_text_encoder_encode(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
             }
         }
 
-        (void) njs_string_prop(&prop, input);
+        (void) njs_string_prop(vm, &prop, input);
 
         if (prop.length != 0) {
             return njs_text_encoder_encode_utf8(vm, &prop, retval);
@@ -164,13 +164,11 @@ njs_text_encoder_encode_into(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     njs_int_t             ret;
     njs_str_t             str;
     njs_value_t           *this, *input, *dest, value, read, written;
+    njs_object_t          *object;
     const u_char          *start, *end;
     njs_typed_array_t     *array;
     njs_unicode_decode_t  ctx;
 
-    static const njs_value_t  read_str = njs_string("read");
-    static const njs_value_t  written_str = njs_string("written");
-
     this = njs_argument(args, 0);
     input = njs_arg(args, nargs, 1);
     dest = njs_arg(args, nargs, 2);
@@ -195,7 +193,7 @@ njs_text_encoder_encode_into(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         return NJS_ERROR;
     }
 
-    njs_string_get(input, &str);
+    njs_string_get(vm, input, &str);
 
     start = str.start;
     end = start + str.length;
@@ -204,7 +202,6 @@ njs_text_encoder_encode_into(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     to = njs_typed_array_start(array);
     to_end = to + array->byte_length;
 
-    cp = 0;
     njs_set_number(&read, 0);
     njs_set_number(&written, 0);
 
@@ -229,22 +226,41 @@ njs_text_encoder_encode_into(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         to = njs_utf8_encode(to, cp);
     }
 
-    return njs_vm_object_alloc(vm, retval, &read_str, &read,
-                               &written_str, &written, NULL);
+    object = njs_object_alloc(vm);
+    if (njs_slow_path(object == NULL)) {
+        return NJS_ERROR;
+    }
+
+    njs_set_object(retval, object);
+
+    ret = njs_object_prop_define(vm, retval, NJS_ATOM_STRING_read, &read,
+                                 NJS_OBJECT_PROP_VALUE_CW);
+    if (njs_slow_path(ret != NJS_OK)) {
+        return ret;
+    }
+
+    ret = njs_object_prop_define(vm, retval, NJS_ATOM_STRING_written, &written,
+                                 NJS_OBJECT_PROP_VALUE_CW);
+    if (njs_slow_path(ret != NJS_OK)) {
+        return ret;
+    }
+
+    return NJS_OK;
 }
 
 
-static const njs_object_prop_t  njs_text_encoder_properties[] =
+static const njs_object_prop_init_t  njs_text_encoder_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_VALUE("encoding", njs_string("utf-8"), 0),
+    NJS_DECLARE_PROP_VALUE(STRING_encoding, njs_ascii_strval("utf-8"), 0),
 
-    NJS_DECLARE_PROP_NATIVE("encode", njs_text_encoder_encode, 0, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_encode, njs_text_encoder_encode, 0, 0),
 
-    NJS_DECLARE_PROP_NATIVE("encodeInto", njs_text_encoder_encode_into, 2, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_encodeInto, njs_text_encoder_encode_into,
+                            2, 0),
 };
 
 
@@ -254,13 +270,14 @@ const njs_object_init_t  njs_text_encoder_init = {
 };
 
 
-static const njs_object_prop_t  njs_text_encoder_constructor_properties[] =
+static const njs_object_prop_init_t  njs_text_encoder_constructor_properties[] =
 {
     NJS_DECLARE_PROP_LENGTH(0),
 
     NJS_DECLARE_PROP_NAME("TextEncoder"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 };
 
 
@@ -342,7 +359,7 @@ njs_text_decoder_arg_encoding(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         }
     }
 
-    njs_string_get(value, &str);
+    njs_string_get(vm, value, &str);
 
     for (label = &njs_encoding_labels[0]; label->name.length != 0; label++) {
         if (njs_strstr_eq(&str, &label->name)) {
@@ -364,9 +381,6 @@ njs_text_decoder_arg_options(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     njs_int_t    ret;
     njs_value_t  retval, *value;
 
-    static const njs_value_t  fatal_str = njs_string("fatal");
-    static const njs_value_t  ignore_bom_str = njs_string("ignoreBOM");
-
     if (nargs < 3) {
         data->fatal = 0;
         data->ignore_bom = 0;
@@ -381,15 +395,14 @@ njs_text_decoder_arg_options(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         return NJS_ERROR;
     }
 
-    ret = njs_value_property(vm, value, njs_value_arg(&fatal_str), &retval);
+    ret = njs_value_property(vm, value, NJS_ATOM_STRING_fatal, &retval);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return ret;
     }
 
     data->fatal = njs_bool(&retval);
 
-    ret = njs_value_property(vm, value, njs_value_arg(&ignore_bom_str),
-                             &retval);
+    ret = njs_value_property(vm, value, NJS_ATOM_STRING_ignoreBOM, &retval);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return ret;
     }
@@ -401,13 +414,11 @@ njs_text_decoder_arg_options(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
 
 static njs_int_t
-njs_text_decoder_encoding(njs_vm_t *vm, njs_object_prop_t *prop,
+njs_text_decoder_encoding(njs_vm_t *vm, njs_object_prop_t *prop, uint32_t unused,
     njs_value_t *value, njs_value_t *setval, njs_value_t *retval)
 {
     njs_encoding_decode_t  *data;
 
-    static const njs_value_t  utf8_str = njs_string("utf-8");
-
     if (njs_slow_path(!njs_is_object_data(value, NJS_DATA_TAG_TEXT_DECODER))) {
         njs_set_undefined(retval);
         return NJS_DECLINED;
@@ -417,7 +428,7 @@ njs_text_decoder_encoding(njs_vm_t *vm, njs_object_prop_t *prop,
 
     switch (data->encoding) {
     case NJS_ENCODING_UTF8:
-        *retval = utf8_str;
+        njs_atom_to_value(vm, retval, NJS_ATOM_STRING_utf_8);
         break;
 
     default:
@@ -430,7 +441,7 @@ njs_text_decoder_encoding(njs_vm_t *vm, njs_object_prop_t *prop,
 
 
 static njs_int_t
-njs_text_decoder_fatal(njs_vm_t *vm, njs_object_prop_t *prop,
+njs_text_decoder_fatal(njs_vm_t *vm, njs_object_prop_t *prop, uint32_t unused,
     njs_value_t *value, njs_value_t *setval, njs_value_t *retval)
 {
     njs_encoding_decode_t  *data;
@@ -450,7 +461,8 @@ njs_text_decoder_fatal(njs_vm_t *vm, njs_object_prop_t *prop,
 
 static njs_int_t
 njs_text_decoder_ignore_bom(njs_vm_t *vm, njs_object_prop_t *prop,
-    njs_value_t *value, njs_value_t *setval, njs_value_t *retval)
+    uint32_t unused, njs_value_t *value, njs_value_t *setval,
+    njs_value_t *retval)
 {
     njs_encoding_decode_t  *data;
 
@@ -483,8 +495,6 @@ njs_text_decoder_decode(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     const njs_typed_array_t   *array;
     const njs_array_buffer_t  *buffer;
 
-    static const njs_value_t  stream_str = njs_string("stream");
-
     start = NULL;
     end = NULL;
 
@@ -528,8 +538,7 @@ njs_text_decoder_decode(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
             return NJS_ERROR;
         }
 
-        ret = njs_value_property(vm, options, njs_value_arg(&stream_str),
-                                 retval);
+        ret = njs_value_property(vm, options, NJS_ATOM_STRING_stream, retval);
         if (njs_slow_path(ret == NJS_ERROR)) {
             return ret;
         }
@@ -543,7 +552,7 @@ njs_text_decoder_decode(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
     /* Looking for BOM. */
 
-    if (!data->ignore_bom) {
+    if (start != NULL && !data->ignore_bom) {
         start += njs_utf8_bom(start, end);
     }
 
@@ -569,19 +578,21 @@ njs_text_decoder_decode(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 }
 
 
-static const njs_object_prop_t  njs_text_decoder_properties[] =
+static const njs_object_prop_init_t  njs_text_decoder_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_HANDLER("encoding", njs_text_decoder_encoding, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_encoding, njs_text_decoder_encoding, 0,
+                             0),
 
-    NJS_DECLARE_PROP_HANDLER("fatal", njs_text_decoder_fatal, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_fatal, njs_text_decoder_fatal, 0, 0),
 
-    NJS_DECLARE_PROP_HANDLER("ignoreBOM", njs_text_decoder_ignore_bom, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_ignoreBOM, njs_text_decoder_ignore_bom,
+                             0, 0),
 
-    NJS_DECLARE_PROP_NATIVE("decode", njs_text_decoder_decode, 0, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_decode, njs_text_decoder_decode, 0, 0),
 };
 
 
@@ -591,13 +602,14 @@ const njs_object_init_t  njs_text_decoder_init = {
 };
 
 
-static const njs_object_prop_t  njs_text_decoder_constructor_properties[] =
+static const njs_object_prop_init_t  njs_text_decoder_constructor_properties[] =
 {
     NJS_DECLARE_PROP_LENGTH(0),
 
     NJS_DECLARE_PROP_NAME("TextDecoder"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 };
 
 
