@@ -9,8 +9,8 @@
 
 
 static njs_int_t njs_external_prop_handler(njs_vm_t *vm,
-    njs_object_prop_t *self, njs_value_t *value, njs_value_t *setval,
-    njs_value_t *retval);
+    njs_object_prop_t *self, uint32_t atom_id, njs_value_t *value,
+    njs_value_t *setval, njs_value_t *retval);
 
 
 static njs_int_t
@@ -19,12 +19,13 @@ njs_external_add(njs_vm_t *vm, njs_arr_t *protos,
 {
     size_t                size;
     njs_int_t             ret;
-    njs_lvlhsh_t          *hash;
+    njs_value_t           prop_name;
     const u_char          *start;
+    njs_flathsh_t         *hash;
     njs_function_t        *function;
     njs_object_prop_t     *prop;
-    njs_lvlhsh_query_t    lhq;
     njs_exotic_slots_t    *slot, *next;
+    njs_flathsh_query_t   lhq;
     const njs_external_t  *end;
 
     slot = njs_arr_add(protos);
@@ -57,34 +58,34 @@ njs_external_add(njs_vm_t *vm, njs_arr_t *protos,
             continue;
         }
 
-        prop = njs_object_prop_alloc(vm, &njs_string_empty,
-                                     &njs_value_invalid, 1);
-        if (njs_slow_path(prop == NULL)) {
-            goto memory_error;
-        }
-
-        prop->writable = external->writable;
-        prop->configurable = external->configurable;
-        prop->enumerable = external->enumerable;
-
         if (external->flags & NJS_EXTERN_SYMBOL) {
-            njs_set_symbol(&prop->name, external->name.symbol, NULL);
-
             lhq.key_hash = external->name.symbol;
 
         } else {
-            ret = njs_string_create(vm, &prop->name,
-                                    external->name.string.start,
-                                    external->name.string.length);
+            ret = njs_atom_string_create(vm, &prop_name,
+                                         external->name.string.start,
+                                         external->name.string.length);
             if (njs_slow_path(ret != NJS_OK)) {
                 return NJS_ERROR;
             }
 
-            lhq.key = external->name.string;
-            lhq.key_hash = njs_djb_hash(lhq.key.start, lhq.key.length);
+            lhq.key_hash = prop_name.atom_id;
         }
 
-        lhq.value = prop;
+        ret = njs_flathsh_unique_insert(hash, &lhq);
+        if (njs_slow_path(ret != NJS_OK)) {
+            njs_internal_error(vm, "lvlhsh insert failed");
+            return NJS_ERROR;
+        }
+
+        prop = lhq.value;
+
+        prop->type = NJS_PROPERTY;
+        prop->enumerable = external->enumerable;
+        prop->configurable = external->configurable;
+        prop->writable = external->writable;
+        prop->u.value = njs_value_invalid;
+
 
         switch (external->flags & NJS_EXTERN_TYPE_MASK) {
         case NJS_EXTERN_METHOD:
@@ -170,12 +171,6 @@ njs_external_add(njs_vm_t *vm, njs_arr_t *protos,
             break;
         }
 
-        ret = njs_lvlhsh_insert(hash, &lhq);
-        if (njs_slow_path(ret != NJS_OK)) {
-            njs_internal_error(vm, "lvlhsh insert failed");
-            return NJS_ERROR;
-        }
-
         external++;
     }
 
@@ -191,14 +186,15 @@ memory_error:
 
 static njs_int_t
 njs_external_prop_handler(njs_vm_t *vm, njs_object_prop_t *self,
-    njs_value_t *value, njs_value_t *setval, njs_value_t *retval)
+    uint32_t atom_id, njs_value_t *value, njs_value_t *setval,
+    njs_value_t *retval)
 {
-    njs_int_t           ret;
-    njs_object_prop_t   *prop;
-    njs_external_ptr_t  external;
-    njs_object_value_t  *ov;
-    njs_lvlhsh_query_t  lhq;
-    njs_exotic_slots_t  *slots;
+    njs_int_t            ret;
+    njs_object_prop_t    *prop;
+    njs_external_ptr_t   external;
+    njs_object_value_t   *ov;
+    njs_exotic_slots_t   *slots;
+    njs_flathsh_query_t  lhq;
 
     if (njs_slow_path(retval == NULL)) {
         return NJS_DECLINED;
@@ -226,28 +222,26 @@ njs_external_prop_handler(njs_vm_t *vm, njs_object_prop_t *self,
         njs_set_object_value(retval, ov);
     }
 
-    prop = njs_object_prop_alloc(vm, &self->name, retval, 1);
-    if (njs_slow_path(prop == NULL)) {
-        return NJS_ERROR;
-    }
-
-    prop->writable = self->writable;
-    prop->configurable = self->configurable;
-    prop->enumerable = self->enumerable;
-
-    lhq.value = prop;
-    njs_string_get(&self->name, &lhq.key);
-    lhq.key_hash = njs_prop_magic32(self);
+    lhq.key_hash = atom_id;
     lhq.replace = 1;
     lhq.pool = vm->mem_pool;
     lhq.proto = &njs_object_hash_proto;
 
-    ret = njs_lvlhsh_insert(njs_object_hash(value), &lhq);
+    ret = njs_flathsh_unique_insert(njs_object_hash(value), &lhq);
     if (njs_slow_path(ret != NJS_OK)) {
         njs_internal_error(vm, "lvlhsh insert/replace failed");
         return NJS_ERROR;
     }
 
+    prop = lhq.value;
+
+    prop->type = NJS_PROPERTY;
+    prop->enumerable = self->enumerable;
+    prop->configurable = self->configurable;
+    prop->writable = self->writable;
+
+    prop->u.value = *retval;
+
     return NJS_OK;
 }
 
@@ -315,7 +309,8 @@ njs_vm_external_prototype(njs_vm_t *vm, const njs_external_t *definition,
 
 static njs_int_t
 njs_vm_external_constructor_handler(njs_vm_t *vm, njs_object_prop_t *prop,
-    njs_value_t *value, njs_value_t *setval, njs_value_t *retval)
+    uint32_t atom_id, njs_value_t *value, njs_value_t *setval,
+    njs_value_t *retval)
 {
     njs_set_function(retval, &njs_vm_ctor(vm, njs_prop_magic32(prop)));
 
@@ -325,7 +320,7 @@ njs_vm_external_constructor_handler(njs_vm_t *vm, njs_object_prop_t *prop,
 
 njs_int_t
 njs_vm_external_constructor(njs_vm_t *vm, const njs_str_t *name,
-    njs_function_native_t native, const njs_external_t *ctor_props,
+    const njs_function_native_t native, const njs_external_t *ctor_props,
     njs_uint_t ctor_nprops, const njs_external_t *proto_props,
     njs_uint_t proto_nprops)
 {
@@ -445,8 +440,8 @@ njs_value_external_tag(const njs_value_t *value)
 
 
 njs_int_t
-njs_external_property(njs_vm_t *vm, njs_object_prop_t *prop, njs_value_t *value,
-    njs_value_t *setval, njs_value_t *retval)
+njs_external_property(njs_vm_t *vm, njs_object_prop_t *prop, uint32_t unused,
+    njs_value_t *value, njs_value_t *setval, njs_value_t *retval)
 {
     char        *p;
     njs_int_t   i;
