@@ -76,14 +76,6 @@
 #define NJS_FLATHSH_ELTS_MINIMUM_TO_SHRINK    8
 
 
-struct njs_flathsh_descr_s {
-    uint32_t     hash_mask;
-    uint32_t     elts_size;          /* allocated properties */
-    uint32_t     elts_count;         /* include deleted properties */
-    uint32_t     elts_deleted_count;
-};
-
-
 static njs_flathsh_descr_t *njs_flathsh_alloc(njs_flathsh_query_t *fhq,
     size_t hash_size, size_t elts_size);
 static njs_flathsh_descr_t *njs_expand_elts(njs_flathsh_query_t *fhq,
@@ -144,13 +136,6 @@ njs_flathsh_chunk(njs_flathsh_descr_t *h)
 }
 
 
-njs_inline njs_flathsh_elt_t *
-njs_hash_elts(njs_flathsh_descr_t *h)
-{
-    return (njs_flathsh_elt_t *) ((char *) h + sizeof(njs_flathsh_descr_t));
-}
-
-
 /*
  * Create a new empty flat hash.
  */
@@ -225,13 +210,14 @@ njs_flathsh_add_elt(njs_flathsh_t *fh, njs_flathsh_query_t *fhq)
     elts = njs_hash_elts(h);
     elt = &elts[h->elts_count++];
 
-    elt->value = fhq->value;
     elt->key_hash = fhq->key_hash;
 
     cell_num = fhq->key_hash & h->hash_mask;
     elt->next_elt = njs_hash_cells_end(h)[-cell_num - 1];
     njs_hash_cells_end(h)[-cell_num - 1] = h->elts_count;
 
+    elt->type = NJS_PROPERTY;
+
     return elt;
 }
 
@@ -284,7 +270,7 @@ njs_expand_elts(njs_flathsh_query_t *fhq, njs_flathsh_descr_t *h)
         njs_memzero(chunk, sizeof(uint32_t) * new_hash_size);
 
         for (i = 0, elt = njs_hash_elts(h); i < h->elts_count; i++, elt++) {
-            if (elt->value != NULL) {
+            if (elt->type != NJS_FREE_FLATHSH_ELEMENT) {
                 cell_num = elt->key_hash & new_hash_mask;
                 elt->next_elt = njs_hash_cells_end(h)[-cell_num - 1];
                 njs_hash_cells_end(h)[-cell_num - 1] = i + 1;
@@ -343,12 +329,41 @@ njs_flathsh_find(const njs_flathsh_t *fh, njs_flathsh_query_t *fhq)
     while (elt_num != 0) {
         e = &elts[elt_num - 1];
 
-        /* TODO: need to be replaced by atomic test. */
-
         if (e->key_hash == fhq->key_hash &&
             fhq->proto->test(fhq, e->value) == NJS_OK)
         {
-            fhq->value = e->value;
+            fhq->value = e;
+            return NJS_OK;
+        }
+
+        elt_num = e->next_elt;
+    }
+
+    return NJS_DECLINED;
+}
+
+
+njs_int_t
+njs_flathsh_unique_find(const njs_flathsh_t *fh, njs_flathsh_query_t *fhq)
+{
+    njs_int_t            cell_num, elt_num;
+    njs_flathsh_elt_t    *e, *elts;
+    njs_flathsh_descr_t  *h;
+
+    h = fh->slot;
+    if (njs_slow_path(h == NULL)) {
+        return NJS_DECLINED;
+    }
+
+    cell_num = fhq->key_hash & h->hash_mask;
+    elt_num = njs_hash_cells_end(h)[-cell_num - 1];
+    elts = njs_hash_elts(h);
+
+    while (elt_num != 0) {
+        e = &elts[elt_num - 1];
+
+        if (e->key_hash == fhq->key_hash) {
+            fhq->value = e;
             return NJS_OK;
         }
 
@@ -362,7 +377,6 @@ njs_flathsh_find(const njs_flathsh_t *fh, njs_flathsh_query_t *fhq)
 njs_int_t
 njs_flathsh_insert(njs_flathsh_t *fh, njs_flathsh_query_t *fhq)
 {
-    void                 *tmp;
     njs_int_t            cell_num, elt_num;
     njs_flathsh_elt_t    *elt, *elts;
     njs_flathsh_descr_t  *h;
@@ -385,21 +399,14 @@ njs_flathsh_insert(njs_flathsh_t *fh, njs_flathsh_query_t *fhq)
     while (elt_num != 0) {
         elt = &elts[elt_num - 1];
 
-        /* TODO: need to be replaced by atomic test. */
-
         if (elt->key_hash == fhq->key_hash &&
             fhq->proto->test(fhq, elt->value) == NJS_OK)
         {
             if (fhq->replace) {
-                tmp = fhq->value;
-                fhq->value = elt->value;
-                elt->value = tmp;
-
+                fhq->value = elt;
                 return NJS_OK;
 
             } else {
-                fhq->value = elt->value;
-
                 return NJS_DECLINED;
             }
         }
@@ -412,7 +419,56 @@ njs_flathsh_insert(njs_flathsh_t *fh, njs_flathsh_query_t *fhq)
         return NJS_ERROR;
     }
 
-    elt->value = fhq->value;
+    fhq->value = elt;
+
+    return NJS_OK;
+}
+
+
+njs_int_t
+njs_flathsh_unique_insert(njs_flathsh_t *fh, njs_flathsh_query_t *fhq)
+{
+    njs_int_t            cell_num, elt_num;
+    njs_flathsh_elt_t    *elt, *elts;
+    njs_flathsh_descr_t  *h;
+
+    h = fh->slot;
+
+    if (h == NULL) {
+        h = njs_flathsh_new(fhq);
+        if (h == NULL) {
+            return NJS_ERROR;
+        }
+
+        fh->slot = h;
+    }
+
+    cell_num = fhq->key_hash & h->hash_mask;
+    elt_num = njs_hash_cells_end(h)[-cell_num - 1];
+    elts = njs_hash_elts(h);
+
+    while (elt_num != 0) {
+        elt = &elts[elt_num - 1];
+
+        if (elt->key_hash == fhq->key_hash) {
+            if (fhq->replace) {
+                fhq->value = elt;
+                return NJS_OK;
+
+            } else {
+                return NJS_DECLINED;
+            }
+        }
+
+        elt_num = elt->next_elt;
+    }
+
+    elt = njs_flathsh_add_elt(fh, fhq);
+    if (elt == NULL) {
+        return NJS_ERROR;
+    }
+
+    fhq->value = elt;
 
     return NJS_OK;
 }
@@ -454,8 +510,8 @@ njs_shrink_elts(njs_flathsh_query_t *fhq, njs_flathsh_descr_t *h)
 
     elt_src = njs_hash_elts(h_src);
     for (i = 0, j = 0, elt = njs_hash_elts(h); i < h->elts_count; i++) {
-        if (elt_src->value != NULL) {
-            elt->value = elt_src->value;
+        if (elt_src->type != NJS_FREE_FLATHSH_ELEMENT) {
+            *elt = *elt_src;
             elt->key_hash = elt_src->key_hash;
 
             cell_num = elt_src->key_hash & new_hash_mask;
@@ -502,12 +558,10 @@ njs_flathsh_delete(njs_flathsh_t *fh, njs_flathsh_query_t *fhq)
     while (elt_num != 0) {
         elt = &elts[elt_num - 1];
 
-        /* TODO: use atomic comparision. */
-
         if (elt->key_hash == fhq->key_hash &&
             fhq->proto->test(fhq, elt->value) == NJS_OK)
         {
-            fhq->value = elt->value;
+            fhq->value = elt;
 
             if (elt_prev != NULL) {
                 elt_prev->next_elt = elt->next_elt;
@@ -518,7 +572,7 @@ njs_flathsh_delete(njs_flathsh_t *fh, njs_flathsh_query_t *fhq)
 
             h->elts_deleted_count++;
 
-            elt->value = NULL;
+            elt->type = NJS_FREE_FLATHSH_ELEMENT;
 
             /* Shrink elts if elts_deleted_count is eligible. */
 
@@ -550,11 +604,75 @@ njs_flathsh_delete(njs_flathsh_t *fh, njs_flathsh_query_t *fhq)
 }
 
 
-void *
+njs_int_t
+njs_flathsh_unique_delete(njs_flathsh_t *fh, njs_flathsh_query_t *fhq)
+{
+    njs_int_t            cell_num, elt_num;
+    njs_flathsh_elt_t    *elt, *elt_prev, *elts;
+    njs_flathsh_descr_t  *h;
+
+    h = fh->slot;
+
+    if (njs_slow_path(h == NULL)) {
+        return NJS_DECLINED;
+    }
+
+    cell_num = fhq->key_hash & h->hash_mask;
+    elt_num = njs_hash_cells_end(h)[-cell_num - 1];
+    elts = njs_hash_elts(h);
+    elt_prev = NULL;
+
+    while (elt_num != 0) {
+        elt = &elts[elt_num - 1];
+
+        if (elt->key_hash == fhq->key_hash) {
+            fhq->value = elt;
+
+            if (elt_prev != NULL) {
+                elt_prev->next_elt = elt->next_elt;
+
+            } else {
+                njs_hash_cells_end(h)[-cell_num - 1] = elt->next_elt;
+            }
+
+            h->elts_deleted_count++;
+
+            elt->type = NJS_FREE_FLATHSH_ELEMENT;
+
+            /* Shrink elts if elts_deleted_count is eligible. */
+
+            if (h->elts_deleted_count >= NJS_FLATHSH_ELTS_MINIMUM_TO_SHRINK
+                && h->elts_deleted_count
+                   >= (h->elts_count / NJS_FLATHSH_ELTS_FRACTION_TO_SHRINK))
+            {
+                h = njs_shrink_elts(fhq, h);
+                if (njs_slow_path(h == NULL)) {
+                    return NJS_ERROR;
+                }
+
+                fh->slot = h;
+            }
+
+            if (h->elts_deleted_count == h->elts_count) {
+                njs_flathsh_free(fhq, njs_flathsh_chunk(h));
+                fh->slot = NULL;
+            }
+
+            return NJS_OK;
+        }
+
+        elt_prev = elt;
+        elt_num = elt->next_elt;
+    }
+
+    return NJS_DECLINED;
+}
+
+
+njs_flathsh_elt_t *
 njs_flathsh_each(const njs_flathsh_t *fh, njs_flathsh_each_t *fhe)
 {
-    void                 *v;
-    njs_flathsh_elt_t    *elt;
+    njs_flathsh_elt_t    *e, *elt;
     njs_flathsh_descr_t  *h;
 
     h = fh->slot;
@@ -565,9 +683,9 @@ njs_flathsh_each(const njs_flathsh_t *fh, njs_flathsh_each_t *fhe)
     elt = njs_hash_elts(h);
 
     while (fhe->cp < h->elts_count) {
-        v = elt[fhe->cp++].value;
-        if (v != NULL) {
-            return v;
+        e = &elt[fhe->cp++];
+        if (e->type != NJS_FREE_FLATHSH_ELEMENT) {
+            return e;
         }
     }
 
