@@ -118,19 +118,33 @@ njs_int_t
 njs_function_name_set(njs_vm_t *vm, njs_function_t *function,
     njs_value_t *name, const char *prefix)
 {
-    u_char              *p;
-    size_t              len, symbol;
-    njs_int_t           ret;
-    njs_value_t         value;
-    njs_string_prop_t   string;
-    njs_object_prop_t   *prop;
-    njs_lvlhsh_query_t  lhq;
+    u_char               *p;
+    size_t               len, symbol;
+    njs_int_t            ret;
+    njs_value_t          value;
+    njs_string_prop_t    string;
+    njs_object_prop_t    *prop;
+    njs_flathsh_query_t  lhq;
 
-    prop = njs_object_prop_alloc(vm, &njs_string_name, name, 0);
-    if (njs_slow_path(prop == NULL)) {
+    lhq.key_hash = NJS_ATOM_STRING_name;
+    lhq.replace = 0;
+    lhq.pool = vm->mem_pool;
+    lhq.proto = &njs_object_hash_proto;
+
+    ret = njs_flathsh_unique_insert(&function->object.hash, &lhq);
+    if (njs_slow_path(ret != NJS_OK)) {
+        njs_internal_error(vm, "lvlhsh insert failed");
         return NJS_ERROR;
     }
 
+    prop = lhq.value;
+
+    prop->type = NJS_PROPERTY;
+    prop->enumerable = 0;
+    prop->configurable = 1;
+    prop->writable = 0;
+    prop->u.value = *name;
+
     symbol = 0;
 
     if (njs_is_symbol(njs_prop_value(prop))) {
@@ -142,7 +156,7 @@ njs_function_name_set(njs_vm_t *vm, njs_function_t *function,
     if (prefix != NULL || symbol != 0) {
         if (njs_is_defined(njs_prop_value(prop))) {
             njs_value_assign(&value, njs_prop_value(prop));
-            (void) njs_string_prop(&string, &value);
+            (void) njs_string_prop(vm, &string, &value);
 
             len = (prefix != NULL) ? njs_strlen(prefix) + 1: 0;
             p = njs_string_alloc(vm, njs_prop_value(prop),
@@ -168,25 +182,10 @@ njs_function_name_set(njs_vm_t *vm, njs_function_t *function,
             }
 
         } else {
-            njs_value_assign(njs_prop_value(prop), &njs_string_empty);
+            njs_set_empty_string(vm, njs_prop_value(prop));
         }
     }
 
-    prop->configurable = 1;
-
-    lhq.key_hash = NJS_NAME_HASH;
-    lhq.key = njs_str_value("name");
-    lhq.replace = 0;
-    lhq.value = prop;
-    lhq.proto = &njs_object_hash_proto;
-    lhq.pool = vm->mem_pool;
-
-    ret = njs_lvlhsh_insert(&function->object.hash, &lhq);
-    if (njs_slow_path(ret != NJS_OK)) {
-        njs_internal_error(vm, "lvlhsh insert failed");
-        return NJS_ERROR;
-    }
-
     return NJS_OK;
 }
 
@@ -251,8 +250,6 @@ njs_function_arguments_object_init(njs_vm_t *vm, njs_native_frame_t *frame)
     njs_value_t   value, length;
     njs_object_t  *arguments;
 
-    static const njs_value_t  string_length = njs_string("length");
-
     arguments = njs_object_alloc(vm);
     if (njs_slow_path(arguments == NULL)) {
         return NJS_ERROR;
@@ -263,9 +260,8 @@ njs_function_arguments_object_init(njs_vm_t *vm, njs_native_frame_t *frame)
     njs_set_object(&value, arguments);
     njs_set_number(&length, frame->nargs);
 
-    ret = njs_object_prop_define(vm, &value, njs_value_arg(&string_length),
-                                 &length, NJS_OBJECT_PROP_VALUE_CW,
-                                 NJS_LENGTH_HASH);
+    ret = njs_object_prop_define(vm, &value, NJS_ATOM_STRING_length, &length,
+                                 NJS_OBJECT_PROP_VALUE_CW);
     if (njs_slow_path(ret != NJS_OK)) {
         return NJS_ERROR;
     }
@@ -328,14 +324,16 @@ njs_function_prototype_thrower(njs_vm_t *vm, njs_value_t *args,
 }
 
 
-const njs_object_prop_t  njs_arguments_object_instance_properties[] =
+const njs_object_prop_init_t  njs_arguments_object_instance_properties[] =
 {
     {
-        .type = NJS_ACCESSOR,
-        .name = njs_string("callee"),
-        .u.accessor = njs_accessor(njs_function_prototype_thrower, 0,
-                                   njs_function_prototype_thrower, 0),
-        .writable = NJS_ATTRIBUTE_UNSET,
+        .desc = {
+        .atom_id = NJS_ATOM_STRING_callee,
+            .type = NJS_ACCESSOR,
+            .u.accessor = njs_accessor(njs_function_prototype_thrower, 0,
+                                       njs_function_prototype_thrower, 0),
+            .writable = 0,
+        },
     },
 };
 
@@ -396,6 +394,19 @@ njs_function_lambda_frame(njs_vm_t *vm, njs_function_t *function,
 
     lambda = function->u.lambda;
 
+    /*
+     * Lambda frame has the following layout:
+     *  njs_frame_t | p0 , p2, ..., pn | v0, v1, ..., vn
+     *  where:
+     *  p0, p1, ..., pn - pointers to arguments and locals,
+     *  v0, v1, ..., vn - values of arguments and locals.
+     *  n - number of arguments + locals.
+     *
+     *  Normally, the pointers point to the values directly after them,
+     *  but if a value was captured as a closure by an inner function,
+     *  pn points to a value allocated from the heap.
+     */
+
     args_count = njs_max(nargs, lambda->nargs);
     value_count = args_count + lambda->nlocal;
 
@@ -525,7 +536,6 @@ njs_function_lambda_call(njs_vm_t *vm, njs_value_t *retval, void *promise_cap)
     njs_value_t            *args, **local, *value;
     njs_value_t            **cur_local, **cur_closures;
     njs_function_t         *function;
-    njs_declaration_t      *declr;
     njs_function_lambda_t  *lambda;
 
     frame = (njs_frame_t *) vm->top_frame;
@@ -584,29 +594,6 @@ njs_function_lambda_call(njs_vm_t *vm, njs_value_t *retval, void *promise_cap)
 
     vm->active_frame = frame;
 
-    /* Closures */
-
-    n = lambda->ndeclarations;
-
-    while (n != 0) {
-        n--;
-
-        declr = &lambda->declarations[n];
-        value = njs_scope_value(vm, declr->index);
-
-        *value = *declr->value;
-
-        function = njs_function_value_copy(vm, value);
-        if (njs_slow_path(function == NULL)) {
-            return NJS_ERROR;
-        }
-
-        ret = njs_function_capture_closure(vm, function, function->u.lambda);
-        if (njs_slow_path(ret != NJS_OK)) {
-            return ret;
-        }
-    }
-
     ret = njs_vmcode_interpreter(vm, lambda->start, retval, promise_cap, NULL);
 
     /* Restore current level. */
@@ -701,11 +688,11 @@ njs_function_frame_free(njs_vm_t *vm, njs_native_frame_t *native)
 njs_int_t
 njs_function_frame_save(njs_vm_t *vm, njs_frame_t *frame, u_char *pc)
 {
-    size_t              args_count, value_count, n;
-    njs_value_t         *start, *end, *p, **new, *value, **local;
-    njs_function_t      *function;
+    size_t                 args_count, value_count, n;
+    njs_value_t            **map, *value, **current_map;
+    njs_function_t         *function;
+    njs_native_frame_t     *active, *native;
     njs_function_lambda_t  *lambda;
-    njs_native_frame_t  *active, *native;
 
     *frame = *vm->active_frame;
 
@@ -723,34 +710,37 @@ njs_function_frame_save(njs_vm_t *vm, njs_frame_t *frame, u_char *pc)
     args_count = njs_max(native->nargs, lambda->nargs);
     value_count = args_count + lambda->nlocal;
 
-    new = (njs_value_t **) ((u_char *) native + NJS_FRAME_SIZE);
-    value = (njs_value_t *) (new + value_count);
+    /*
+     * We need to save the current frame state because it will be freed
+     * when the function returns.
+     *
+     *  To detect whether a value is captured as a closure,
+     *  we check whether the pointer is within the frame. In this case
+     *  the pointer is copied as is because the value it points to
+     *  is already allocated in the heap and will not be freed.
+     *  See njs_function_capture_closure() and njs_function_lambda_frame()
+     *  for details.
+     */
+
+    map = (njs_value_t **) ((u_char *) native + NJS_FRAME_SIZE);
+    value = (njs_value_t *) (map + value_count);
+
+    current_map = (njs_value_t **) ((u_char *) active + NJS_FRAME_SIZE);
+
+    for (n = 0; n < value_count; n++) {
+        if (njs_is_value_allocated_on_frame(active, current_map[n])) {
+            map[n] = &value[n];
+            njs_value_assign(&value[n], current_map[n]);
+
+        } else {
+            map[n] = current_map[n];
+        }
+    }
 
     native->arguments = value;
-    native->local = new + njs_function_frame_args_count(active);
+    native->local = map + args_count;
     native->pc = pc;
 
-    start = njs_function_frame_values(active, &end);
-    p = native->arguments;
-
-    while (start < end) {
-        njs_value_assign(p, start++);
-        *new++ = p++;
-    }
-
-    /* Move all arguments. */
-
-    p = native->arguments;
-    local = native->local + 1 /* this */;
-
-    for (n = 0; n < function->args_count; n++) {
-        if (!njs_is_valid(p)) {
-            njs_set_undefined(p);
-        }
-
-        *local++ = p++;
-    }
-
     return NJS_OK;
 }
 
@@ -772,7 +762,6 @@ njs_int_t
 njs_function_capture_closure(njs_vm_t *vm, njs_function_t *function,
     njs_function_lambda_t *lambda)
 {
-    void                *start, *end;
     uint32_t            n;
     njs_value_t         *value, **closure;
     njs_native_frame_t  *frame;
@@ -787,9 +776,6 @@ njs_function_capture_closure(njs_vm_t *vm, njs_function_t *function,
         frame = frame->previous;
     }
 
-    start = frame;
-    end = frame->free;
-
     closure = njs_function_closures(function);
     n = lambda->nclosures;
 
@@ -798,7 +784,7 @@ njs_function_capture_closure(njs_vm_t *vm, njs_function_t *function,
 
         value = njs_scope_value(vm, lambda->closures[n]);
 
-        if (start <= (void *) value && (void *) value < end) {
+        if (njs_is_value_allocated_on_frame(frame, value)) {
             value = njs_scope_value_clone(vm, lambda->closures[n], value);
             if (njs_slow_path(value == NULL)) {
                 return NJS_ERROR;
@@ -814,14 +800,14 @@ njs_function_capture_closure(njs_vm_t *vm, njs_function_t *function,
 
 
 njs_inline njs_value_t *
-njs_function_closure_value(njs_vm_t *vm, njs_value_t **scope, njs_index_t index,
-    void *start, void *end)
+njs_function_closure_value(njs_vm_t *vm, njs_native_frame_t *frame,
+    njs_value_t **scope, njs_index_t index)
 {
     njs_value_t  *value, *newval;
 
     value = scope[njs_scope_index_value(index)];
 
-    if (start <= (void *) value && end > (void *) value) {
+    if (njs_is_value_allocated_on_frame(frame, value)) {
         newval = njs_mp_alloc(vm->mem_pool, sizeof(njs_value_t));
         if (njs_slow_path(newval == NULL)) {
             njs_memory_error(vm);
@@ -841,7 +827,6 @@ njs_function_closure_value(njs_vm_t *vm, njs_value_t **scope, njs_index_t index,
 njs_int_t
 njs_function_capture_global_closures(njs_vm_t *vm, njs_function_t *function)
 {
-    void                   *start, *end;
     uint32_t               n;
     njs_value_t            *value, **refs, **global;
     njs_index_t            *indexes, index;
@@ -860,9 +845,6 @@ njs_function_capture_global_closures(njs_vm_t *vm, njs_function_t *function)
         native = native->previous;
     }
 
-    start = native;
-    end = native->free;
-
     indexes = lambda->closures;
     refs = njs_function_closures(function);
 
@@ -877,12 +859,12 @@ njs_function_capture_global_closures(njs_vm_t *vm, njs_function_t *function)
 
         switch (njs_scope_index_type(index)) {
         case NJS_LEVEL_LOCAL:
-            value = njs_function_closure_value(vm, native->local, index,
-                                               start, end);
+            value = njs_function_closure_value(vm, native, native->local,
+                                               index);
             break;
 
         case NJS_LEVEL_GLOBAL:
-            value = njs_function_closure_value(vm, global, index, start, end);
+            value = njs_function_closure_value(vm, native, global, index);
             break;
 
         default:
@@ -905,38 +887,35 @@ njs_function_capture_global_closures(njs_vm_t *vm, njs_function_t *function)
 
 
 static njs_value_t *
-njs_function_property_prototype_set(njs_vm_t *vm, njs_lvlhsh_t *hash,
+njs_function_property_prototype_set(njs_vm_t *vm, njs_flathsh_t *hash,
     njs_value_t *prototype)
 {
-    njs_int_t           ret;
-    njs_object_prop_t   *prop;
-    njs_lvlhsh_query_t  lhq;
+    njs_int_t            ret;
+    njs_object_prop_t    *prop;
+    njs_flathsh_query_t  lhq;
 
-    const njs_value_t  proto_string = njs_string("prototype");
-
-    prop = njs_object_prop_alloc(vm, &proto_string, prototype, 0);
-    if (njs_slow_path(prop == NULL)) {
-        return NULL;
-    }
-
-    prop->writable = 1;
-
-    lhq.value = prop;
-    lhq.key_hash = NJS_PROTOTYPE_HASH;
-    lhq.key = njs_str_value("prototype");
+    lhq.key_hash = NJS_ATOM_STRING_prototype;
     lhq.replace = 1;
     lhq.pool = vm->mem_pool;
     lhq.proto = &njs_object_hash_proto;
 
-    ret = njs_lvlhsh_insert(hash, &lhq);
+    ret = njs_flathsh_unique_insert(hash, &lhq);
 
-    if (njs_fast_path(ret == NJS_OK)) {
-        return njs_prop_value(prop);
+    if (njs_slow_path(ret != NJS_OK)) {
+        njs_internal_error(vm, "lvlhsh insert failed");
+        return NULL;
     }
 
-    njs_internal_error(vm, "lvlhsh insert failed");
+    prop = lhq.value;
+
+    prop->type = NJS_PROPERTY;
+    prop->enumerable = 0;
+    prop->configurable = 0;
+    prop->writable = 1;
+    prop->u.value = *prototype;
+
+    return njs_prop_value(prop);
 
-    return NULL;
 }
 
 
@@ -952,11 +931,11 @@ njs_function_property_prototype_set(njs_vm_t *vm, njs_lvlhsh_t *hash,
 
 njs_int_t
 njs_function_prototype_create(njs_vm_t *vm, njs_object_prop_t *prop,
-    njs_value_t *value, njs_value_t *setval, njs_value_t *retval)
+    uint32_t unused, njs_value_t *value, njs_value_t *setval,
+    njs_value_t *retval)
 {
-    njs_value_t     *proto, proto_value, *cons;
-    njs_object_t    *prototype;
-    njs_function_t  *function;
+    njs_value_t   *proto, proto_value, *cons;
+    njs_object_t  *prototype;
 
     if (setval == NULL) {
         prototype = njs_object_alloc(vm);
@@ -969,11 +948,6 @@ njs_function_prototype_create(njs_vm_t *vm, njs_object_prop_t *prop,
         setval = &proto_value;
     }
 
-    function = njs_function_value_copy(vm, value);
-    if (njs_slow_path(function == NULL)) {
-        return NJS_ERROR;
-    }
-
     proto = njs_function_property_prototype_set(vm, njs_object_hash(value),
                                                 setval);
     if (njs_slow_path(proto == NULL)) {
@@ -1002,6 +976,7 @@ njs_function_constructor(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     njs_int_t               ret;
     njs_str_t               str, file;
     njs_uint_t              i;
+    njs_value_t             name;
     njs_parser_t            parser;
     njs_vm_code_t           *code;
     njs_function_t          *function;
@@ -1032,7 +1007,7 @@ njs_function_constructor(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         goto fail;
     }
 
-    NJS_CHB_MP_INIT(&chain, vm);
+    NJS_CHB_MP_INIT(&chain, njs_vm_memory_pool(vm));
 
     if (async) {
         njs_chb_append_literal(&chain, "(async function(");
@@ -1052,7 +1027,7 @@ njs_function_constructor(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         }
     }
 
-    njs_chb_append_literal(&chain, "){");
+    njs_chb_append_literal(&chain, "\n){\n");
 
     if (nargs > 1) {
         ret = njs_value_to_chain(vm, &chain, njs_argument(args, nargs - 1));
@@ -1061,7 +1036,7 @@ njs_function_constructor(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         }
     }
 
-    njs_chb_append_literal(&chain, "})");
+    njs_chb_append_literal(&chain, "\n})");
 
     ret = njs_chb_join(&chain, &str);
     if (njs_slow_path(ret != NJS_OK)) {
@@ -1072,7 +1047,7 @@ njs_function_constructor(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     file = njs_str_value("runtime");
 
     ret = njs_parser_init(vm, &parser, NULL, &file, str.start,
-                          str.start + str.length, 1);
+                          str.start + str.length);
     if (njs_slow_path(ret != NJS_OK)) {
         return ret;
     }
@@ -1128,7 +1103,15 @@ njs_function_constructor(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
     njs_chb_destroy(&chain);
 
-    lambda = ((njs_vmcode_function_t *) generator.code_start)->lambda;
+    if ((code->end - code->start)
+        != (sizeof(njs_vmcode_function_t) + sizeof(njs_vmcode_return_t))
+        || ((njs_vmcode_generic_t *) code->start)->code != NJS_VMCODE_FUNCTION)
+    {
+        njs_syntax_error(vm, "single function literal required");
+        return NJS_ERROR;
+    }
+
+    lambda = ((njs_vmcode_function_t *) code->start)->lambda;
 
     function = njs_function_alloc(vm, lambda, (njs_bool_t) async);
     if (njs_slow_path(function == NULL)) {
@@ -1139,8 +1122,9 @@ njs_function_constructor(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     function->global_this = 1;
     function->args_count = lambda->nargs - lambda->rest_parameters;
 
-    ret = njs_function_name_set(vm, function,
-                                njs_value_arg(&njs_string_anonymous), NULL);
+    njs_atom_to_value(vm, &name, NJS_ATOM_STRING_anonymous);
+
+    ret = njs_function_name_set(vm, function, &name, NULL);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return ret;
     }
@@ -1156,13 +1140,14 @@ fail:
 }
 
 
-static const njs_object_prop_t  njs_function_constructor_properties[] =
+static const njs_object_prop_init_t  njs_function_constructor_properties[] =
 {
     NJS_DECLARE_PROP_LENGTH(1),
 
     NJS_DECLARE_PROP_NAME("Function"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 };
 
 
@@ -1174,7 +1159,8 @@ const njs_object_init_t  njs_function_constructor_init = {
 
 njs_int_t
 njs_function_instance_length(njs_vm_t *vm, njs_object_prop_t *prop,
-    njs_value_t *value, njs_value_t *setval, njs_value_t *retval)
+    uint32_t unused, njs_value_t *value, njs_value_t *setval,
+    njs_value_t *retval)
 {
     njs_function_t  *function;
 
@@ -1193,7 +1179,8 @@ njs_function_instance_length(njs_vm_t *vm, njs_object_prop_t *prop,
 
 njs_int_t
 njs_function_instance_name(njs_vm_t *vm, njs_object_prop_t *prop,
-    njs_value_t *value, njs_value_t *setval, njs_value_t *retval)
+    uint32_t unused, njs_value_t *value, njs_value_t *setval,
+    njs_value_t *retval)
 {
     njs_function_t  *function;
 
@@ -1209,7 +1196,7 @@ njs_function_instance_name(njs_vm_t *vm, njs_object_prop_t *prop,
         return NJS_OK;
     }
 
-    njs_value_assign(retval, &njs_string_empty);
+    njs_set_empty_string(vm, retval);
 
     return NJS_OK;
 }
@@ -1376,14 +1363,13 @@ njs_function_prototype_bind(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
     function->context = njs_function(&args[0]);
 
-    ret = njs_value_property(vm, &args[0], njs_value_arg(&njs_string_name),
-                             &name);
+    ret = njs_value_property(vm, &args[0], NJS_ATOM_STRING_name, &name);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return ret;
     }
 
     if (!njs_is_string(&name)) {
-        name = njs_string_empty;
+        njs_set_empty_string(vm, &name);
     }
 
     ret = njs_function_name_set(vm, function, &name, "bound");
@@ -1428,38 +1414,43 @@ njs_function_prototype_bind(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 }
 
 
-static const njs_object_prop_t  njs_function_prototype_properties[] =
+static const njs_object_prop_init_t  njs_function_prototype_properties[] =
 {
     NJS_DECLARE_PROP_LENGTH(0),
 
     NJS_DECLARE_PROP_NAME(""),
 
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_NATIVE("call", njs_function_prototype_call, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_call, njs_function_prototype_call, 1, 0),
 
-    NJS_DECLARE_PROP_NATIVE("apply", njs_function_prototype_apply, 2, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_apply, njs_function_prototype_apply, 2,
+                            0),
 
-    NJS_DECLARE_PROP_NATIVE("bind", njs_function_prototype_bind, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_bind, njs_function_prototype_bind, 1, 0),
 
     {
-        .type = NJS_ACCESSOR,
-        .name = njs_string("caller"),
-        .u.accessor = njs_accessor(njs_function_prototype_thrower, 0,
-                                   njs_function_prototype_thrower, 0),
-        .writable = NJS_ATTRIBUTE_UNSET,
-        .configurable = 1,
+        .desc = {
+            .atom_id = NJS_ATOM_STRING_caller,
+            .type = NJS_ACCESSOR,
+            .u.accessor = njs_accessor(njs_function_prototype_thrower, 0,
+                                       njs_function_prototype_thrower, 0),
+            .writable = 0,
+            .configurable = 1,
+        },
     },
 
     {
-        .type = NJS_ACCESSOR,
-        .name = njs_string("arguments"),
-        .u.accessor = njs_accessor(njs_function_prototype_thrower, 0,
-                                   njs_function_prototype_thrower, 0),
-        .writable = NJS_ATTRIBUTE_UNSET,
-        .configurable = 1,
+        .desc = {
+            .atom_id = NJS_ATOM_STRING_arguments,
+            .type = NJS_ACCESSOR,
+            .u.accessor = njs_accessor(njs_function_prototype_thrower, 0,
+                                       njs_function_prototype_thrower, 0),
+            .writable = 0,
+            .configurable = 1,
+        },
     },
 };
 
@@ -1470,15 +1461,16 @@ const njs_object_init_t  njs_function_prototype_init = {
 };
 
 
-const njs_object_prop_t  njs_function_instance_properties[] =
+const njs_object_prop_init_t  njs_function_instance_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("length", njs_function_instance_length, 0, 0,
+    NJS_DECLARE_PROP_HANDLER(STRING_length, njs_function_instance_length,
+                             0, NJS_OBJECT_PROP_VALUE_C),
+
+    NJS_DECLARE_PROP_HANDLER(STRING_name, njs_function_instance_name, 0,
                              NJS_OBJECT_PROP_VALUE_C),
 
-    NJS_DECLARE_PROP_HANDLER("name", njs_function_instance_name, 0, 0,
-                             NJS_OBJECT_PROP_VALUE_C),
-
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_function_prototype_create, 0, 0,
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype,
+                             njs_function_prototype_create, 0,
                              NJS_OBJECT_PROP_VALUE_W),
 };
 
@@ -1489,12 +1481,12 @@ const njs_object_init_t  njs_function_instance_init = {
 };
 
 
-const njs_object_prop_t  njs_arrow_instance_properties[] =
+const njs_object_prop_init_t  njs_arrow_instance_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("length", njs_function_instance_length, 0, 0,
-                             NJS_OBJECT_PROP_VALUE_C),
+    NJS_DECLARE_PROP_HANDLER(STRING_length, njs_function_instance_length,
+                             0, NJS_OBJECT_PROP_VALUE_C),
 
-    NJS_DECLARE_PROP_HANDLER("name", njs_function_instance_name, 0, 0,
+    NJS_DECLARE_PROP_HANDLER(STRING_name, njs_function_instance_name, 0,
                              NJS_OBJECT_PROP_VALUE_C),
 };
 
