@@ -63,7 +63,7 @@ static const u_char *njs_json_skip_space(const u_char *start,
     const u_char *end);
 
 static njs_int_t njs_json_internalize_property(njs_vm_t *vm,
-    njs_function_t *reviver, njs_value_t *holder, njs_value_t *name,
+    njs_function_t *reviver, njs_value_t *holder, uint32_t atom_id,
     njs_int_t depth, njs_value_t *retval);
 static void njs_json_parse_exception(njs_json_parse_ctx_t *ctx,
     const char *msg, const u_char *pos);
@@ -80,15 +80,15 @@ static njs_int_t njs_json_stringify_array(njs_json_stringify_t *stringify);
 
 static njs_int_t njs_json_append_value(njs_vm_t *vm, njs_chb_t *chain,
     njs_value_t *value);
-static void njs_json_append_string(njs_chb_t *chain, const njs_value_t *value,
-    char quote);
+static void njs_json_append_string(njs_vm_t *vm, njs_chb_t *chain,
+    const njs_value_t *value, char quote);
 static void njs_json_append_number(njs_chb_t *chain, const njs_value_t *value);
 
 static njs_object_t *njs_json_wrap_value(njs_vm_t *vm, njs_value_t *wrapper,
     const njs_value_t *value);
 
 
-static const njs_object_prop_t  njs_json_object_properties[];
+static const njs_object_prop_init_t  njs_json_object_properties[];
 
 
 static njs_int_t
@@ -112,7 +112,7 @@ njs_json_parse(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         }
     }
 
-    (void) njs_string_prop(&string, text);
+    (void) njs_string_prop(vm, &string, text);
 
     p = string.start;
     end = p + string.size;
@@ -149,9 +149,8 @@ njs_json_parse(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         }
 
         return njs_json_internalize_property(vm, njs_function(reviver),
-                                             &wrapper,
-                                             njs_value_arg(&njs_string_empty),
-                                             0, retval);
+                                             &wrapper, NJS_ATOM_STRING_empty, 0,
+                                             retval);
     }
 
     njs_value_assign(retval, &value);
@@ -166,7 +165,7 @@ njs_vm_json_parse(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 {
     njs_function_t  *parse;
 
-    parse = njs_function(&njs_json_object_properties[1].u.value);
+    parse = njs_function(&njs_json_object_properties[1].desc.u.value);
 
     return njs_vm_invoke(vm, parse, args, nargs, retval);
 }
@@ -225,7 +224,7 @@ njs_json_stringify(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
     switch (space->type) {
     case NJS_STRING:
-        length = njs_string_prop(&prop, space);
+        length = njs_string_prop(vm, &prop, space);
         p = njs_string_offset(&prop, njs_min(length, 10));
 
         stringify->space.start = prop.start;
@@ -272,7 +271,7 @@ njs_vm_json_stringify(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 {
     njs_function_t  *stringify;
 
-    stringify = njs_function(&njs_json_object_properties[2].u.value);
+    stringify = njs_function(&njs_json_object_properties[2].desc.u.value);
 
     return njs_vm_invoke(vm, stringify, args, nargs, retval);
 }
@@ -336,11 +335,11 @@ static const u_char *
 njs_json_parse_object(njs_json_parse_ctx_t *ctx, njs_value_t *value,
     const u_char *p)
 {
-    njs_int_t           ret;
-    njs_object_t        *object;
-    njs_value_t         prop_name, prop_value;
-    njs_object_prop_t   *prop;
-    njs_lvlhsh_query_t  lhq;
+    njs_int_t            ret;
+    njs_object_t         *object;
+    njs_value_t          prop_name, prop_value;
+    njs_object_prop_t    *prop;
+    njs_flathsh_query_t  lhq;
 
     if (njs_slow_path(--ctx->depth == 0)) {
         njs_json_parse_exception(ctx, "Nested too deep", p);
@@ -395,24 +394,25 @@ njs_json_parse_object(njs_json_parse_ctx_t *ctx, njs_value_t *value,
             return NULL;
         }
 
-        prop = njs_object_prop_alloc(ctx->vm, &prop_name, &prop_value, 1);
-        if (njs_slow_path(prop == NULL)) {
-            goto memory_error;
-        }
-
-        njs_string_get(&prop_name, &lhq.key);
-        lhq.key_hash = njs_djb_hash(lhq.key.start, lhq.key.length);
-        lhq.value = prop;
+        lhq.key_hash = prop_name.atom_id;
         lhq.replace = 1;
         lhq.pool = ctx->pool;
         lhq.proto = &njs_object_hash_proto;
 
-        ret = njs_lvlhsh_insert(&object->hash, &lhq);
+        ret = njs_flathsh_unique_insert(&object->hash, &lhq);
         if (njs_slow_path(ret != NJS_OK)) {
             njs_internal_error(ctx->vm, "lvlhsh insert/replace failed");
             return NULL;
         }
 
+        prop = lhq.value;
+
+        prop->type = NJS_PROPERTY;
+        prop->enumerable = 1;
+        prop->configurable = 1;
+        prop->writable = 1;
+        prop->u.value = prop_value;
+
         p = njs_json_skip_space(p, ctx->end);
         if (njs_slow_path(p == ctx->end)) {
             goto error_end;
@@ -734,7 +734,7 @@ njs_json_parse_string(njs_json_parse_ctx_t *ctx, njs_value_t *value,
         start = dst;
     }
 
-    ret = njs_string_create(ctx->vm, value, (u_char *) start, size);
+    ret = njs_atom_string_create(ctx->vm, value, (u_char *) start, size);
     if (njs_slow_path(ret != NJS_OK)) {
         return NULL;
     }
@@ -829,12 +829,12 @@ njs_json_skip_space(const u_char *start, const u_char *end)
 
 static njs_int_t
 njs_json_internalize_property(njs_vm_t *vm, njs_function_t *reviver,
-    njs_value_t *holder, njs_value_t *name, njs_int_t depth,
+    njs_value_t *holder, uint32_t atom_id, njs_int_t depth,
     njs_value_t *retval)
 {
     int64_t       k, length;
     njs_int_t     ret;
-    njs_value_t   val, new_elem, index;
+    njs_value_t   val, new_elem;
     njs_value_t   arguments[3];
     njs_array_t   *keys;
 
@@ -843,7 +843,7 @@ njs_json_internalize_property(njs_vm_t *vm, njs_function_t *reviver,
         return NJS_ERROR;
     }
 
-    ret = njs_value_property(vm, holder, name, &val);
+    ret = njs_value_property(vm, holder, atom_id, &val);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
     }
@@ -859,19 +859,21 @@ njs_json_internalize_property(njs_vm_t *vm, njs_function_t *reviver,
 
             for (k = 0; k < keys->length; k++) {
                 ret = njs_json_internalize_property(vm, reviver, &val,
-                                                    &keys->start[k], depth,
-                                                    &new_elem);
+                                                    keys->start[k].atom_id,
+                                                    depth, &new_elem);
 
                 if (njs_slow_path(ret != NJS_OK)) {
                     goto done;
                 }
 
                 if (njs_is_undefined(&new_elem)) {
-                    ret = njs_value_property_delete(vm, &val, &keys->start[k],
+                    ret = njs_value_property_delete(vm, &val,
+                                                    keys->start[k].atom_id,
                                                     NULL, 0);
 
                 } else {
-                    ret = njs_value_property_set(vm, &val, &keys->start[k],
+                    ret = njs_value_property_set(vm, &val,
+                                                 keys->start[k].atom_id,
                                                  &new_elem);
                 }
 
@@ -888,12 +890,8 @@ njs_json_internalize_property(njs_vm_t *vm, njs_function_t *reviver,
             }
 
             for (k = 0; k < length; k++) {
-                ret = njs_int64_to_string(vm, &index, k);
-                if (njs_slow_path(ret != NJS_OK)) {
-                    return NJS_ERROR;
-                }
-
-                ret = njs_json_internalize_property(vm, reviver, &val, &index,
+                ret = njs_json_internalize_property(vm, reviver, &val,
+                                                    njs_number_atom(k),
                                                     depth, &new_elem);
 
                 if (njs_slow_path(ret != NJS_OK)) {
@@ -901,10 +899,13 @@ njs_json_internalize_property(njs_vm_t *vm, njs_function_t *reviver,
                 }
 
                 if (njs_is_undefined(&new_elem)) {
-                    ret = njs_value_property_delete(vm, &val, &index, NULL, 0);
+                    ret = njs_value_property_delete(vm, &val,
+                                                    njs_number_atom(k), NULL,
+                                                    0);
 
                 } else {
-                    ret = njs_value_property_set(vm, &val, &index, &new_elem);
+                    ret = njs_value_property_set(vm, &val, njs_number_atom(k),
+                                                 &new_elem);
                 }
 
                 if (njs_slow_path(ret == NJS_ERROR)) {
@@ -915,7 +916,7 @@ njs_json_internalize_property(njs_vm_t *vm, njs_function_t *reviver,
     }
 
     njs_value_assign(&arguments[0], holder);
-    njs_value_assign(&arguments[1], name);
+    njs_atom_to_value(vm, &arguments[1], atom_id);
     njs_value_assign(&arguments[2], &val);
 
     ret = njs_function_apply(vm, reviver, arguments, 3, retval);
@@ -1093,7 +1094,7 @@ njs_json_stringify_iterator(njs_json_stringify_t *stringify,
         goto memory_error;
     }
 
-    NJS_CHB_MP_INIT(&chain, stringify->vm);
+    NJS_CHB_MP_INIT(&chain, njs_vm_memory_pool(stringify->vm));
 
     for ( ;; ) {
         if (state->index == 0) {
@@ -1123,7 +1124,7 @@ njs_json_stringify_iterator(njs_json_stringify_t *stringify,
             key = &state->keys->start[state->index];
         }
 
-        ret = njs_value_property(stringify->vm, &state->value, key, value);
+        ret = njs_value_property_val(stringify->vm, &state->value, key, value);
         if (njs_slow_path(ret == NJS_ERROR)) {
             return ret;
         }
@@ -1161,7 +1162,7 @@ njs_json_stringify_iterator(njs_json_stringify_t *stringify,
         state->written = 1;
 
         if (!state->array) {
-            njs_json_append_string(&chain, key, '\"');
+            njs_json_append_string(stringify->vm, &chain, key, '\"');
             njs_chb_append_literal(&chain,":");
             if (stringify->space.length != 0) {
                 njs_chb_append_literal(&chain," ");
@@ -1232,14 +1233,13 @@ memory_error:
 static njs_function_t *
 njs_object_to_json_function(njs_vm_t *vm, njs_value_t *value)
 {
-    njs_int_t           ret;
-    njs_value_t         retval;
-    njs_lvlhsh_query_t  lhq;
-
-    static const njs_value_t  to_json_string = njs_string("toJSON");
+    njs_int_t            ret;
+    njs_value_t          retval;
+    njs_flathsh_query_t  lhq;
 
     if (njs_is_object(value)) {
-        njs_object_property_init(&lhq, &to_json_string, NJS_TO_JSON_HASH);
+        lhq.proto = &njs_object_hash_proto;
+        lhq.key_hash = NJS_ATOM_STRING_toJSON;
 
         ret = njs_object_property(vm, njs_object(value), &lhq, &retval);
 
@@ -1258,6 +1258,7 @@ static njs_int_t
 njs_json_stringify_to_json(njs_json_stringify_t* stringify,
     njs_json_state_t *state, njs_value_t *key, njs_value_t *value)
 {
+    njs_int_t       ret;
     njs_value_t     arguments[2];
     njs_function_t  *to_json;
 
@@ -1276,7 +1277,10 @@ njs_json_stringify_to_json(njs_json_stringify_t* stringify,
         arguments[1] = *key;
 
     } else {
-        njs_uint32_to_string(&arguments[1], state->index);
+        ret = njs_uint32_to_string(stringify->vm, &arguments[1], state->index);
+        if (njs_slow_path(ret != NJS_OK)) {
+            return NJS_ERROR;
+        }
     }
 
     return njs_function_apply(stringify->vm, to_json, arguments, 2,
@@ -1288,6 +1292,7 @@ static njs_int_t
 njs_json_stringify_replacer(njs_json_stringify_t* stringify,
     njs_json_state_t *state, njs_value_t *key, njs_value_t *value)
 {
+    njs_int_t    ret;
     njs_value_t  arguments[3];
 
     if (!njs_is_function(&stringify->replacer)) {
@@ -1301,7 +1306,10 @@ njs_json_stringify_replacer(njs_json_stringify_t* stringify,
         arguments[1] = *key;
 
     } else {
-        njs_uint32_to_string(&arguments[1], state->index);
+        ret = njs_uint32_to_string(stringify->vm, &arguments[1], state->index);
+        if (njs_slow_path(ret != NJS_OK)) {
+            return NJS_ERROR;
+        }
     }
 
     return njs_function_apply(stringify->vm, njs_function(&stringify->replacer),
@@ -1328,7 +1336,7 @@ njs_json_stringify_array(njs_json_stringify_t *stringify)
     }
 
     item = njs_array_push(stringify->vm, properties);
-    njs_value_assign(item, &njs_string_empty);
+    njs_set_empty_string(stringify->vm, item);
 
     for (i = 0; i < length; i++) {
         ret = njs_value_property_i64(stringify->vm, &stringify->replacer, i,
@@ -1373,7 +1381,9 @@ njs_json_stringify_array(njs_json_stringify_t *stringify)
         }
 
         for (k = 0; k < properties->length; k++) {
-            if (njs_values_strict_equal(value, &properties->start[k]) == 1) {
+            if (njs_values_strict_equal(stringify->vm, value,
+                                        &properties->start[k]) == 1)
+            {
                 break;
             }
         }
@@ -1428,7 +1438,7 @@ njs_json_append_value(njs_vm_t *vm, njs_chb_t *chain, njs_value_t *value)
 
     switch (value->type) {
     case NJS_STRING:
-        njs_json_append_string(chain, value, '\"');
+        njs_json_append_string(vm, chain, value, '\"');
         break;
 
     case NJS_NUMBER:
@@ -1459,7 +1469,8 @@ njs_json_append_value(njs_vm_t *vm, njs_chb_t *chain, njs_value_t *value)
 
 
 static void
-njs_json_append_string(njs_chb_t *chain, const njs_value_t *value, char quote)
+njs_json_append_string(njs_vm_t *vm, njs_chb_t *chain, const njs_value_t *value,
+    char quote)
 {
     size_t             size;
     u_char             c, *dst, *dst_end;
@@ -1470,7 +1481,7 @@ njs_json_append_string(njs_chb_t *chain, const njs_value_t *value, char quote)
     static char  hex2char[16] = { '0', '1', '2', '3', '4', '5', '6', '7',
                                   '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
 
-    (void) njs_string_prop(&string, value);
+    (void) njs_string_prop(vm, &string, value);
 
     p = string.start;
     end = p + string.size;
@@ -1588,9 +1599,9 @@ static njs_object_t *
 njs_json_wrap_value(njs_vm_t *vm, njs_value_t *wrapper,
     const njs_value_t *value)
 {
-    njs_int_t           ret;
-    njs_object_prop_t   *prop;
-    njs_lvlhsh_query_t  lhq;
+    njs_int_t            ret;
+    njs_object_prop_t    *prop;
+    njs_flathsh_query_t  lhq;
 
     wrapper->data.u.object = njs_object_alloc(vm);
     if (njs_slow_path(njs_object(wrapper) == NULL)) {
@@ -1600,40 +1611,36 @@ njs_json_wrap_value(njs_vm_t *vm, njs_value_t *wrapper,
     wrapper->type = NJS_OBJECT;
     wrapper->data.truth = 1;
 
+    lhq.key_hash = NJS_ATOM_STRING_empty;
     lhq.replace = 0;
-    lhq.proto = &njs_object_hash_proto;
     lhq.pool = vm->mem_pool;
-    lhq.key = njs_str_value("");
-    lhq.key_hash = NJS_DJB_HASH_INIT;
+    lhq.proto = &njs_object_hash_proto;
 
-    prop = njs_object_prop_alloc(vm, &njs_string_empty, value, 1);
-    if (njs_slow_path(prop == NULL)) {
-        return NULL;
-    }
-
-    lhq.value = prop;
-
-    ret = njs_lvlhsh_insert(njs_object_hash(wrapper), &lhq);
+    ret = njs_flathsh_unique_insert(njs_object_hash(wrapper), &lhq);
     if (njs_slow_path(ret != NJS_OK)) {
         return NULL;
     }
 
+    prop = lhq.value;
+
+    prop->type = NJS_PROPERTY;
+    prop->enumerable = 1;
+    prop->configurable = 1;
+    prop->writable = 1;
+    prop->u.value = *value;
+
     return wrapper->data.u.object;
 }
 
 
-static const njs_object_prop_t  njs_json_object_properties[] =
+static const njs_object_prop_init_t  njs_json_object_properties[] =
 {
-    {
-        .type = NJS_PROPERTY,
-        .name = njs_wellknown_symbol(NJS_SYMBOL_TO_STRING_TAG),
-        .u.value = njs_string("JSON"),
-        .configurable = 1,
-    },
+    NJS_DECLARE_PROP_VALUE(SYMBOL_toStringTag, njs_ascii_strval("JSON"),
+                           NJS_OBJECT_PROP_VALUE_C),
 
-    NJS_DECLARE_PROP_NATIVE("parse", njs_json_parse, 2, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_parse, njs_json_parse, 2, 0),
 
-    NJS_DECLARE_PROP_NATIVE("stringify", njs_json_stringify, 3, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_stringify, njs_json_stringify, 3, 0),
 };
 
 
@@ -1653,8 +1660,6 @@ njs_dump_terminal(njs_json_stringify_t *stringify, njs_chb_t *chain,
     njs_typed_array_t  *array;
     njs_string_prop_t  string;
 
-    static const njs_value_t  name_string = njs_string("name");
-
     njs_int_t   (*to_string)(njs_vm_t *, njs_value_t *, const njs_value_t *);
 
     switch (value->type) {
@@ -1677,10 +1682,10 @@ njs_dump_terminal(njs_json_stringify_t *stringify, njs_chb_t *chain,
         break;
 
     case NJS_STRING:
-        njs_string_get(value, &str);
+        njs_string_get(stringify->vm, value, &str);
 
         if (!console || stringify->depth != 0) {
-            njs_json_append_string(chain, value, '\'');
+            njs_json_append_string(stringify->vm, chain, value, '\'');
             return NJS_OK;
         }
 
@@ -1694,7 +1699,7 @@ njs_dump_terminal(njs_json_stringify_t *stringify, njs_chb_t *chain,
             return NJS_ERROR;
         }
 
-        njs_string_get(&str_val, &str);
+        njs_string_get(stringify->vm, &str_val, &str);
         njs_chb_append_str(chain, &str);
 
         break;
@@ -1731,7 +1736,7 @@ njs_dump_terminal(njs_json_stringify_t *stringify, njs_chb_t *chain,
                 return NJS_ERROR;
             }
 
-            njs_string_get(&str_val, &str);
+            njs_string_get(stringify->vm, &str_val, &str);
             njs_chb_sprintf(chain, 16 + str.length, "[Number: %V]", &str);
             break;
 
@@ -1741,7 +1746,7 @@ njs_dump_terminal(njs_json_stringify_t *stringify, njs_chb_t *chain,
                 return NJS_ERROR;
             }
 
-            njs_string_get(&str_val, &str);
+            njs_string_get(stringify->vm, &str_val, &str);
             njs_chb_sprintf(chain, 16 + str.length, "[Symbol: %V]", &str);
 
             break;
@@ -1749,7 +1754,7 @@ njs_dump_terminal(njs_json_stringify_t *stringify, njs_chb_t *chain,
         case NJS_STRING:
         default:
             njs_chb_append_literal(chain, "[String: ");
-            njs_json_append_string(chain, value, '\'');
+            njs_json_append_string(stringify->vm, chain, value, '\'');
             njs_chb_append_literal(chain, "]");
             break;
         }
@@ -1764,14 +1769,14 @@ njs_dump_terminal(njs_json_stringify_t *stringify, njs_chb_t *chain,
             str = njs_str_value("");
         }
 
-        ret = njs_value_property(stringify->vm, value,
-                                 njs_value_arg(&name_string), &tag);
+        ret = njs_value_property(stringify->vm, value, NJS_ATOM_STRING_name,
+                                 &tag);
         if (njs_slow_path(ret == NJS_ERROR)) {
             return ret;
         }
 
         if (njs_is_string(&tag)) {
-            njs_string_get(&tag, &str);
+            njs_string_get(stringify->vm, &tag, &str);
         }
 
         if (str.length != 0) {
@@ -1791,7 +1796,7 @@ njs_dump_terminal(njs_json_stringify_t *stringify, njs_chb_t *chain,
         }
 
         if (ret == NJS_OK) {
-            (void) njs_string_prop(&string, &tag);
+            (void) njs_string_prop(stringify->vm, &string, &tag);
             njs_chb_append(chain, string.start, string.size);
             njs_chb_append_literal(chain, " ");
         }
@@ -1841,7 +1846,7 @@ njs_dump_terminal(njs_json_stringify_t *stringify, njs_chb_t *chain,
             return NJS_ERROR;
         }
 
-        njs_string_get(&str_val, &str);
+        njs_string_get(stringify->vm, &str_val, &str);
         njs_chb_append_str(chain, &str);
 
         break;
@@ -1874,7 +1879,7 @@ njs_dump_visited(njs_vm_t *vm, njs_json_stringify_t *stringify,
     depth = stringify->depth - 1;
 
     for (; depth >= 0; depth--) {
-        if (njs_values_same(&stringify->states[depth].value, value)) {
+        if (njs_values_same(vm, &stringify->states[depth].value, value)) {
             return 1;
         }
     }
@@ -1943,11 +1948,6 @@ njs_dump_empty(njs_json_stringify_t *stringify, njs_json_state_t *state,
 }
 
 
-static const njs_value_t  string_get = njs_string("[Getter]");
-static const njs_value_t  string_set = njs_string("[Setter]");
-static const njs_value_t  string_get_set = njs_long_string("[Getter/Setter]");
-
-
 njs_int_t
 njs_vm_value_dump(njs_vm_t *vm, njs_str_t *retval, njs_value_t *value,
     njs_uint_t console, njs_uint_t indent)
@@ -1955,7 +1955,7 @@ njs_vm_value_dump(njs_vm_t *vm, njs_str_t *retval, njs_value_t *value,
     njs_int_t             ret;
     njs_chb_t             chain;
     njs_str_t             str;
-    njs_value_t           *key, *val, tag, exception;
+    njs_value_t           *key, *val, s, tag, exception;
     njs_json_state_t      *state;
     njs_string_prop_t     string;
     njs_object_prop_t     *prop;
@@ -1977,7 +1977,7 @@ njs_vm_value_dump(njs_vm_t *vm, njs_str_t *retval, njs_value_t *value,
         value = &exception;
     }
 
-    NJS_CHB_MP_INIT(&chain, vm);
+    NJS_CHB_MP_INIT(&chain, njs_vm_memory_pool(vm));
 
     if (!njs_dump_is_recursive(value)) {
         ret = njs_dump_terminal(stringify, &chain, value, console);
@@ -2009,7 +2009,7 @@ njs_vm_value_dump(njs_vm_t *vm, njs_str_t *retval, njs_value_t *value,
             }
 
             if (ret == NJS_OK) {
-                (void) njs_string_prop(&string, &tag);
+                (void) njs_string_prop(vm, &string, &tag);
                 njs_chb_append(&chain, string.start, string.size);
                 njs_chb_append_literal(&chain, " ");
             }
@@ -2032,22 +2032,17 @@ njs_vm_value_dump(njs_vm_t *vm, njs_str_t *retval, njs_value_t *value,
             continue;
         }
 
-        njs_property_query_init(&pq, NJS_PROPERTY_QUERY_GET, 0, 0);
+        njs_property_query_init(&pq, NJS_PROPERTY_QUERY_GET, 0);
 
         key = &state->keys->start[state->index++];
 
-        if(state->array) {
-            if (key->type == NJS_STRING) {
-                njs_string_get(key, &str);
-                if (str.length == 6 && memcmp(str.start, "length", 6) == 0) {
-                    continue;
-                }
-            }
+        if (state->array && key->atom_id == NJS_ATOM_STRING_length) {
+            continue;
         }
 
         state->key = key;
 
-        ret = njs_property_query(vm, &pq, &state->value, key);
+        ret = njs_property_query_val(vm, &pq, &state->value, key);
         if (njs_slow_path(ret != NJS_OK)) {
             if (ret == NJS_DECLINED) {
                 continue;
@@ -2074,7 +2069,7 @@ njs_vm_value_dump(njs_vm_t *vm, njs_str_t *retval, njs_value_t *value,
         njs_dump_empty(stringify, state, &chain, 1);
 
         if (!state->array || isnan(njs_key_to_index(key))) {
-            njs_key_string_get(vm, key, &pq.lhq.key);
+            njs_atom_string_get(vm, key->atom_id, &pq.lhq.key);
             njs_chb_append(&chain, pq.lhq.key.start, pq.lhq.key.length);
             njs_chb_append_literal(&chain, ":");
             if (stringify->space.length != 0) {
@@ -2087,7 +2082,8 @@ njs_vm_value_dump(njs_vm_t *vm, njs_str_t *retval, njs_value_t *value,
         if (prop->type == NJS_PROPERTY_HANDLER) {
             pq.scratch = *prop;
             prop = &pq.scratch;
-            ret = njs_prop_handler(prop)(vm, prop, &state->value, NULL,
+            ret = njs_prop_handler(prop)(vm, prop, pq.lhq.key_hash,
+                                         &state->value, NULL,
                                          njs_prop_value(prop));
 
             if (njs_slow_path(ret == NJS_ERROR)) {
@@ -2100,15 +2096,17 @@ njs_vm_value_dump(njs_vm_t *vm, njs_str_t *retval, njs_value_t *value,
         if (njs_is_accessor_descriptor(prop)) {
             if (njs_prop_getter(prop) != NULL) {
                 if (njs_prop_setter(prop) != NULL) {
-                    val = njs_value_arg(&string_get_set);
+                    njs_atom_to_value(vm, &s, NJS_ATOM_STRING__Getter_Setter_);
 
                 } else {
-                    val = njs_value_arg(&string_get);
+                    njs_atom_to_value(vm, &s, NJS_ATOM_STRING__Getter_);
                 }
 
             } else {
-                val = njs_value_arg(&string_set);
+                njs_atom_to_value(vm, &s, NJS_ATOM_STRING__Setter_);
             }
+
+            val = &s;
         }
 
         if (njs_dump_is_recursive(val)) {
