@@ -8,67 +8,20 @@
 #include <njs_main.h>
 
 
-static njs_object_prop_t *njs_object_prop_alloc2(njs_vm_t *vm,
-    const njs_value_t *name, njs_object_prop_type_t type, unsigned flags);
 static njs_object_prop_t *njs_descriptor_prop(njs_vm_t *vm,
-    const njs_value_t *name, const njs_value_t *desc);
+    const njs_value_t *desc, njs_object_prop_t *prop,
+    uint32_t *unset_enumerable, uint32_t *unset_configuarble,
+    uint32_t *enum_writable);
 
 
-njs_object_prop_t *
-njs_object_prop_alloc(njs_vm_t *vm, const njs_value_t *name,
-    const njs_value_t *value, uint8_t attributes)
+void
+njs_object_prop_init(njs_object_prop_t *prop, njs_object_prop_type_t type,
+    uint8_t flags)
 {
-    unsigned           flags;
-    njs_object_prop_t  *prop;
-
-    switch (attributes) {
-    case NJS_ATTRIBUTE_FALSE:
-    case NJS_ATTRIBUTE_TRUE:
-        flags = attributes ? NJS_OBJECT_PROP_VALUE_ECW : 0;
-        break;
-
-    case NJS_ATTRIBUTE_UNSET:
-    default:
-        flags = NJS_OBJECT_PROP_UNSET;
-        break;
-    }
-
-    prop = njs_object_prop_alloc2(vm, name, NJS_PROPERTY, flags);
-    if (njs_slow_path(prop == NULL)) {
-        return NULL;
-    }
-
-    njs_value_assign(njs_prop_value(prop), value);
-
-    return prop;
-}
-
-
-static njs_object_prop_t *
-njs_object_prop_alloc2(njs_vm_t *vm, const njs_value_t *name,
-    njs_object_prop_type_t type, unsigned flags)
-{
-    njs_int_t          ret;
-    njs_object_prop_t  *prop;
-
-    prop = njs_mp_align(vm->mem_pool, sizeof(njs_value_t),
-                        sizeof(njs_object_prop_t));
-    if (njs_slow_path(prop == NULL)) {
-        njs_memory_error(vm);
-        return NULL;
-    }
-
-    njs_value_assign(&prop->name, name);
-
-    if (njs_slow_path(!njs_is_key(&prop->name))) {
-        ret = njs_value_to_key(vm, &prop->name, &prop->name);
-        if (njs_slow_path(ret != NJS_OK)) {
-            return NULL;
-        }
-    }
+    prop->next_elt = 0;
+    prop->atom_id = 0;
 
     prop->type = type;
-    prop->enum_in_object_hash = 0;
 
     if (flags != NJS_OBJECT_PROP_UNSET) {
         prop->enumerable = !!(flags & NJS_OBJECT_PROP_ENUMERABLE);
@@ -78,29 +31,27 @@ njs_object_prop_alloc2(njs_vm_t *vm, const njs_value_t *name,
             prop->writable = !!(flags & NJS_OBJECT_PROP_WRITABLE);
 
         } else {
-            prop->writable = NJS_ATTRIBUTE_UNSET;
+            prop->writable = 0;
         }
 
     } else {
-        prop->enumerable = NJS_ATTRIBUTE_UNSET;
-        prop->configurable = NJS_ATTRIBUTE_UNSET;
-        prop->writable = NJS_ATTRIBUTE_UNSET;
+        prop->enumerable = 0;
+        prop->configurable = 0;
+        prop->writable = 0;
     }
-
-    return prop;
 }
 
 
 njs_int_t
-njs_object_property(njs_vm_t *vm, njs_object_t *object, njs_lvlhsh_query_t *lhq,
-    njs_value_t *retval)
+njs_object_property(njs_vm_t *vm, njs_object_t *object,
+    njs_flathsh_query_t *lhq, njs_value_t *retval)
 {
     njs_int_t          ret;
     njs_value_t        value;
     njs_object_prop_t  *prop;
 
     do {
-        ret = njs_lvlhsh_find(&object->hash, lhq);
+        ret = njs_flathsh_unique_find(&object->hash, lhq);
 
         if (njs_fast_path(ret == NJS_OK)) {
             prop = lhq->value;
@@ -110,7 +61,7 @@ njs_object_property(njs_vm_t *vm, njs_object_t *object, njs_lvlhsh_query_t *lhq,
             }
         }
 
-        ret = njs_lvlhsh_find(&object->shared_hash, lhq);
+        ret = njs_flathsh_unique_find(&object->shared_hash, lhq);
 
         if (njs_fast_path(ret == NJS_OK)) {
             goto found;
@@ -145,37 +96,32 @@ found:
 
 
 njs_object_prop_t *
-njs_object_property_add(njs_vm_t *vm, njs_value_t *object, njs_value_t *key,
+njs_object_property_add(njs_vm_t *vm, njs_value_t *object, unsigned atom_id,
     njs_bool_t replace)
 {
-    njs_int_t           ret;
-    njs_value_t         key_value;
-    njs_object_prop_t   *prop;
-    njs_lvlhsh_query_t  lhq;
+    njs_int_t            ret;
+    njs_object_prop_t    *prop;
+    njs_flathsh_query_t  lhq;
 
-    prop = njs_object_prop_alloc(vm, key, &njs_value_invalid, 1);
-    if (njs_slow_path(prop == NULL)) {
-        return NULL;
-    }
-
-    ret = njs_value_to_key(vm, &key_value, key);
-    if (njs_slow_path(ret != NJS_OK)) {
-        return NULL;
-    }
-
-    lhq.proto = &njs_object_hash_proto;
-    njs_string_get(&key_value, &lhq.key);
-    lhq.key_hash = njs_djb_hash(lhq.key.start, lhq.key.length);
-    lhq.value = prop;
+    lhq.key_hash = atom_id;
     lhq.replace = replace;
     lhq.pool = vm->mem_pool;
+    lhq.proto = &njs_object_hash_proto;
 
-    ret = njs_lvlhsh_insert(njs_object_hash(object), &lhq);
+    ret = njs_flathsh_unique_insert(njs_object_hash(object), &lhq);
     if (njs_slow_path(ret != NJS_OK)) {
         njs_internal_error(vm, "lvlhsh insert failed");
         return NULL;
     }
 
+    prop = lhq.value;
+
+    prop->type = NJS_PROPERTY;
+    prop->enumerable = 1;
+    prop->configurable = 1;
+    prop->writable = 1;
+    prop->u.value = njs_value_invalid;
+
     return prop;
 }
 
@@ -184,32 +130,29 @@ njs_object_property_add(njs_vm_t *vm, njs_value_t *object, njs_value_t *key,
  * ES5.1, 8.12.9: [[DefineOwnProperty]]
  */
 njs_int_t
-njs_object_prop_define(njs_vm_t *vm, njs_value_t *object,
-    njs_value_t *name, njs_value_t *value, unsigned flags, uint32_t hash)
+njs_object_prop_define(njs_vm_t *vm, njs_value_t *object, unsigned atom_id,
+    njs_value_t *value, unsigned flags)
 {
-    uint32_t              length, index;
+    uint32_t              length, index, set_enumerable, set_configurable,
+                          set_writable;
     njs_int_t             ret;
     njs_array_t           *array;
-    njs_value_t           retval;
-    njs_object_prop_t     *prop, *prev;
+    njs_value_t           key, retval;
+    njs_object_prop_t     _prop;
+    njs_object_prop_t     *prop = &_prop, *prev, *obj_prop;
     njs_property_query_t  pq;
 
-    static const njs_str_t  length_key = njs_str("length");
-
-    if (njs_slow_path(!njs_is_index_or_key(name))) {
-        ret = njs_value_to_key(vm, name, name);
-        if (njs_slow_path(ret != NJS_OK)) {
-            return ret;
-        }
-    }
-
 again:
 
-    njs_property_query_init(&pq, NJS_PROPERTY_QUERY_SET, hash, 1);
+    set_enumerable = 1;
+    set_configurable = 1;
+    set_writable = 1;
+
+    njs_property_query_init(&pq, NJS_PROPERTY_QUERY_SET, 1);
 
     ret = (flags & NJS_OBJECT_PROP_CREATE)
                 ? NJS_DECLINED
-                : njs_property_query(vm, &pq, object, name);
+                : njs_property_query(vm, &pq, object, atom_id);
 
     if (njs_slow_path(ret == NJS_ERROR)) {
         return ret;
@@ -217,7 +160,8 @@ again:
 
     switch (njs_prop_type(flags)) {
     case NJS_OBJECT_PROP_DESCRIPTOR:
-        prop = njs_descriptor_prop(vm, name, value);
+        prop = njs_descriptor_prop(vm, value, prop, &set_enumerable,
+                                   &set_configurable, &set_writable);
         if (njs_slow_path(prop == NULL)) {
             return NJS_ERROR;
         }
@@ -227,25 +171,19 @@ again:
     case NJS_OBJECT_PROP_VALUE:
         if (((flags & NJS_OBJECT_PROP_VALUE_ECW) == NJS_OBJECT_PROP_VALUE_ECW)
             && njs_is_fast_array(object)
-            && njs_is_number(name))
+            && njs_atom_is_number(atom_id))
         {
-            if (njs_number_is_integer_index(njs_number(name))) {
-                array = njs_array(object);
-                index = (uint32_t) njs_number(name);
+            array = njs_array(object);
+            index = njs_atom_number(atom_id);
 
-                if (index < array->length) {
-                    njs_value_assign(&array->start[index], value);
-                    return NJS_OK;
-                }
+            if (index < array->length) {
+                njs_value_assign(&array->start[index], value);
+                return NJS_OK;
             }
         }
 
-        prop = njs_object_prop_alloc2(vm, name, NJS_PROPERTY,
-                                      flags & NJS_OBJECT_PROP_VALUE_ECW);
-        if (njs_slow_path(prop == NULL)) {
-            return NJS_ERROR;
-        }
-
+        njs_object_prop_init(prop, NJS_PROPERTY,
+                              flags & NJS_OBJECT_PROP_VALUE_ECW);
         njs_value_assign(njs_prop_value(prop), value);
         break;
 
@@ -254,11 +192,9 @@ again:
     default:
         njs_assert(njs_is_function(value));
 
-        prop = njs_object_prop_alloc2(vm, name, NJS_ACCESSOR,
-                                      NJS_OBJECT_PROP_VALUE_EC);
-        if (njs_slow_path(prop == NULL)) {
-            return NJS_ERROR;
-        }
+        njs_object_prop_init(prop, NJS_ACCESSOR, NJS_OBJECT_PROP_VALUE_EC);
+
+        set_writable = 0;
 
         if (njs_prop_type(flags) == NJS_OBJECT_PROP_GETTER) {
             njs_prop_getter(prop) = njs_function(value);
@@ -277,17 +213,23 @@ again:
 set_prop:
 
         if (!njs_object(object)->extensible) {
-            njs_key_string_get(vm, name,  &pq.lhq.key);
+            njs_atom_string_get(vm, atom_id, &pq.lhq.key);
             njs_type_error(vm, "Cannot add property \"%V\", "
                            "object is not extensible", &pq.lhq.key);
             return NJS_ERROR;
         }
 
-        if (njs_slow_path(njs_is_typed_array(object)
-                          && njs_is_string(name)))
+        if (njs_slow_path(njs_is_typed_array(object) &&
+           (flags & NJS_OBJECT_PROP_IS_STRING)))
         {
             /* Integer-Indexed Exotic Objects [[DefineOwnProperty]]. */
-            if (!isnan(njs_string_to_index(name))) {
+
+            ret = njs_atom_to_value(vm, &key, atom_id);
+            if (njs_slow_path(ret != NJS_OK)) {
+                return ret;
+            }
+
+            if (!isnan(njs_string_to_index(&key))) {
                 njs_type_error(vm, "Invalid typed array index");
                 return NJS_ERROR;
             }
@@ -305,7 +247,7 @@ set_prop:
             }
 
         } else {
-            if (prop->writable == NJS_ATTRIBUTE_UNSET) {
+            if (!set_writable) {
                 prop->writable = 0;
             }
 
@@ -314,11 +256,11 @@ set_prop:
             }
         }
 
-        if (prop->enumerable == NJS_ATTRIBUTE_UNSET) {
+        if (!set_enumerable) {
             prop->enumerable = 0;
         }
 
-        if (prop->configurable == NJS_ATTRIBUTE_UNSET) {
+        if (!set_configurable) {
             prop->configurable = 0;
         }
 
@@ -327,41 +269,30 @@ set_prop:
 
             if (njs_slow_path(prev->type == NJS_WHITEOUT)) {
                 /* Previously deleted property.  */
+                prop->atom_id = prev->atom_id;
+                prop->next_elt = prev->next_elt;
                 *prev = *prop;
             }
 
         } else {
 
-            if ((flags & NJS_OBJECT_PROP_CREATE)) {
-                ret = njs_primitive_value_to_key(vm, &pq.key, name);
-                if (njs_slow_path(ret != NJS_OK)) {
-                    return NJS_ERROR;
-                }
-
-                if (njs_is_symbol(name)) {
-                    pq.lhq.key_hash = njs_symbol_key(name);
-                    pq.lhq.key.start = NULL;
-
-                } else {
-                    njs_string_get(&pq.key, &pq.lhq.key);
-                    pq.lhq.key_hash = (hash == 0)
-                                           ? njs_djb_hash(pq.lhq.key.start,
-                                                          pq.lhq.key.length)
-                                           : hash;
-                }
-
-                pq.lhq.proto = &njs_object_hash_proto;
-            }
-
-            pq.lhq.value = prop;
+            pq.lhq.key_hash = atom_id;
+            pq.lhq.proto = &njs_object_hash_proto;
             pq.lhq.replace = 0;
             pq.lhq.pool = vm->mem_pool;
 
-            ret = njs_lvlhsh_insert(njs_object_hash(object), &pq.lhq);
+            ret = njs_flathsh_unique_insert(njs_object_hash(object), &pq.lhq);
             if (njs_slow_path(ret != NJS_OK)) {
                 njs_internal_error(vm, "lvlhsh insert failed");
                 return NJS_ERROR;
             }
+
+            obj_prop = pq.lhq.value;
+            obj_prop->enumerable = prop->enumerable;
+            obj_prop->configurable = prop->configurable;
+            obj_prop->writable = prop->writable;
+            obj_prop->type = prop->type;
+            obj_prop->u.value = prop->u.value;
         }
 
         return NJS_OK;
@@ -381,9 +312,9 @@ set_prop:
     case NJS_PROPERTY_PLACE_REF:
         if (prev->type == NJS_PROPERTY_REF
             && !njs_is_accessor_descriptor(prop)
-            && prop->configurable != NJS_ATTRIBUTE_FALSE
-            && prop->enumerable != NJS_ATTRIBUTE_FALSE
-            && prop->writable != NJS_ATTRIBUTE_FALSE)
+            && (!set_configurable || prop->configurable)
+            && (!set_enumerable || prop->enumerable)
+            && (!set_writable || prop->writable))
         {
             if (njs_is_valid(njs_prop_value(prop))) {
                 njs_value_assign(njs_prop_ref(prev), njs_prop_value(prop));
@@ -414,9 +345,9 @@ set_prop:
             goto exception;
         }
 
-        if (prop->configurable == NJS_ATTRIBUTE_TRUE ||
-            prop->enumerable == NJS_ATTRIBUTE_FALSE ||
-            prop->writable == NJS_ATTRIBUTE_FALSE)
+        if ((set_configurable && prop->configurable)
+            || (set_enumerable && !prop->enumerable)
+            || (set_writable && !prop->writable))
         {
             goto exception;
         }
@@ -442,22 +373,24 @@ set_prop:
 
     if (!prev->configurable) {
 
-        if (prop->configurable == NJS_ATTRIBUTE_TRUE) {
+        if (prop->configurable) {
             goto exception;
         }
 
-        if (prop->enumerable != NJS_ATTRIBUTE_UNSET
-            && prev->enumerable != prop->enumerable)
-        {
+        if (set_enumerable && prev->enumerable != prop->enumerable) {
             goto exception;
         }
     }
 
-    if (njs_is_generic_descriptor(prop)) {
+    if (!(set_writable || njs_is_data_descriptor(prop))
+        && !njs_is_accessor_descriptor(prop))
+    {
         goto done;
     }
 
-    if (njs_is_data_descriptor(prev) != njs_is_data_descriptor(prop)) {
+    if (njs_is_data_descriptor(prev)
+        != (set_writable || njs_is_data_descriptor(prop)))
+    {
         if (!prev->configurable) {
             goto exception;
         }
@@ -476,12 +409,12 @@ set_prop:
         }
 
         if (njs_is_data_descriptor(prev)) {
-            prev->writable = NJS_ATTRIBUTE_UNSET;
+            set_writable = 0;
             njs_prop_getter(prev) = NULL;
             njs_prop_setter(prev) = NULL;
 
         } else {
-            prev->writable = NJS_ATTRIBUTE_FALSE;
+            prev->writable = 0;
 
             njs_set_undefined(njs_prop_value(prev));
         }
@@ -489,16 +422,17 @@ set_prop:
         prev->type = prop->type;
 
     } else if (njs_is_data_descriptor(prev)
-               && njs_is_data_descriptor(prop))
+               && (set_writable || njs_is_data_descriptor(prop)))
     {
         if (!prev->configurable && !prev->writable) {
-            if (prop->writable == NJS_ATTRIBUTE_TRUE) {
+            if (prop->writable) {
                 goto exception;
             }
 
             if (njs_is_valid(njs_prop_value(prop))
                 && prev->type != NJS_PROPERTY_HANDLER
-                && !njs_values_same(njs_prop_value(prop), njs_prop_value(prev)))
+                && !njs_values_same(vm, njs_prop_value(prop),
+                                    njs_prop_value(prev)))
             {
                 goto exception;
             }
@@ -523,9 +457,8 @@ set_prop:
 done:
 
     if (njs_slow_path(njs_is_fast_array(object)
-                      && pq.lhq.key_hash == NJS_LENGTH_HASH)
-                      && njs_strstr_eq(&pq.lhq.key, &length_key)
-                      && prop->writable == NJS_ATTRIBUTE_FALSE)
+                      && pq.lhq.key_hash == NJS_ATOM_STRING_length)
+                      && (set_writable && !prop->writable))
     {
         array = njs_array(object);
         length = array->length;
@@ -558,7 +491,7 @@ done:
 
         if (prev->type == NJS_PROPERTY_HANDLER) {
             if (prev->writable) {
-                ret = njs_prop_handler(prev)(vm, prev, object,
+                ret = njs_prop_handler(prev)(vm, prev, atom_id, object,
                                              njs_prop_value(prop), &retval);
                 if (njs_slow_path(ret == NJS_ERROR)) {
                     return ret;
@@ -578,19 +511,18 @@ done:
         } else {
 
             if (njs_slow_path(njs_is_array(object)
-                              && pq.lhq.key_hash == NJS_LENGTH_HASH)
-                              && njs_strstr_eq(&pq.lhq.key, &length_key))
+                              && pq.lhq.key_hash == NJS_ATOM_STRING_length))
             {
-                if (prev->configurable != NJS_ATTRIBUTE_TRUE
-                    && prev->writable != NJS_ATTRIBUTE_TRUE
-                    && !njs_values_strict_equal(njs_prop_value(prev),
-                                                 njs_prop_value(prop)))
+                if (!prev->configurable
+                    && !prev->writable
+                    && !njs_values_strict_equal(vm, njs_prop_value(prev),
+                                                njs_prop_value(prop)))
                 {
                     njs_type_error(vm, "Cannot redefine property: \"length\"");
                     return NJS_ERROR;
                 }
 
-                if (prop->writable != NJS_ATTRIBUTE_UNSET) {
+                if (set_writable) {
                     prev->writable = prop->writable;
                 }
 
@@ -607,15 +539,15 @@ done:
      * attribute of the property named P of object O to the value of the field.
      */
 
-    if (prop->writable != NJS_ATTRIBUTE_UNSET) {
+    if (set_writable) {
         prev->writable = prop->writable;
     }
 
-    if (prop->enumerable != NJS_ATTRIBUTE_UNSET) {
+    if (set_enumerable) {
         prev->enumerable = prop->enumerable;
     }
 
-    if (prop->configurable != NJS_ATTRIBUTE_UNSET) {
+    if (set_configurable) {
         prev->configurable = prop->configurable;
     }
 
@@ -623,7 +555,7 @@ done:
 
 exception:
 
-    njs_key_string_get(vm, &pq.key,  &pq.lhq.key);
+    njs_atom_string_get(vm, atom_id, &pq.lhq.key);
     njs_type_error(vm, "Cannot redefine property: \"%V\"", &pq.lhq.key);
 
     return NJS_ERROR;
@@ -635,31 +567,29 @@ njs_prop_private_copy(njs_vm_t *vm, njs_property_query_t *pq,
     njs_object_t *proto)
 {
     njs_int_t          ret;
-    njs_value_t        *value;
+    njs_value_t        *value, prop_name;
     njs_object_t       *object;
     njs_function_t     *function;
     njs_object_prop_t  *prop, *shared;
 
-    prop = njs_mp_align(vm->mem_pool, sizeof(njs_value_t),
-                        sizeof(njs_object_prop_t));
-    if (njs_slow_path(prop == NULL)) {
-        njs_memory_error(vm);
-        return NJS_ERROR;
-    }
-
     shared = pq->lhq.value;
-    *prop = *shared;
 
     pq->lhq.replace = 0;
-    pq->lhq.value = prop;
     pq->lhq.pool = vm->mem_pool;
 
-    ret = njs_lvlhsh_insert(&proto->hash, &pq->lhq);
+    ret = njs_flathsh_unique_insert(&proto->hash, &pq->lhq);
     if (njs_slow_path(ret != NJS_OK)) {
         njs_internal_error(vm, "lvlhsh insert failed");
         return NJS_ERROR;
     }
 
+    prop = pq->lhq.value;
+    prop->enumerable = shared->enumerable;
+    prop->configurable = shared->configurable;
+    prop->writable = shared->writable;
+    prop->type = shared->type;
+    prop->u.value = shared->u.value;
+
     if (njs_is_accessor_descriptor(prop)) {
         if (njs_prop_getter(prop) != NULL) {
             function = njs_function_copy(vm, njs_prop_getter(prop));
@@ -710,7 +640,12 @@ njs_prop_private_copy(njs_vm_t *vm, njs_property_query_t *pq,
             return NJS_ERROR;
         }
 
-        return njs_function_name_set(vm, function, &prop->name, NULL);
+        ret = njs_atom_to_value(vm, &prop_name, pq->lhq.key_hash);
+        if (ret != NJS_OK) {
+            return NJS_ERROR;
+        }
+
+        return njs_function_name_set(vm, function, &prop_name, NULL);
 
     default:
         break;
@@ -721,29 +656,28 @@ njs_prop_private_copy(njs_vm_t *vm, njs_property_query_t *pq,
 
 
 static njs_object_prop_t *
-njs_descriptor_prop(njs_vm_t *vm, const njs_value_t *name,
-    const njs_value_t *desc)
+njs_descriptor_prop(njs_vm_t *vm, const njs_value_t *desc,
+    njs_object_prop_t *prop, uint32_t *set_enumerable,
+    uint32_t *set_configurable, uint32_t *set_writable)
 {
-    njs_int_t           ret;
-    njs_bool_t          data, accessor;
-    njs_value_t         value;
-    njs_object_t        *desc_object;
-    njs_function_t      *getter, *setter;
-    njs_object_prop_t   *prop;
-    njs_lvlhsh_query_t  lhq;
-
-    static const njs_value_t  get_string = njs_string("get");
+    njs_int_t            ret;
+    njs_bool_t           data, accessor;
+    njs_value_t          value;
+    njs_object_t         *desc_object;
+    njs_function_t       *getter, *setter;
+    njs_flathsh_query_t  lhq;
 
     if (!njs_is_object(desc)) {
         njs_type_error(vm, "property descriptor must be an object");
         return NULL;
     }
 
-    prop = njs_object_prop_alloc(vm, name, &njs_value_invalid,
-                                 NJS_ATTRIBUTE_UNSET);
-    if (njs_slow_path(prop == NULL)) {
-        return NULL;
-    }
+    njs_object_prop_init(prop, NJS_PROPERTY, NJS_OBJECT_PROP_UNSET);
+    *njs_prop_value(prop) = njs_value_invalid;
+
+    *set_enumerable = 0;
+    *set_configurable = 0;
+    *set_writable = 0;
 
     data = 0;
     accessor = 0;
@@ -751,7 +685,8 @@ njs_descriptor_prop(njs_vm_t *vm, const njs_value_t *name,
     setter = NJS_PROP_PTR_UNSET;
     desc_object = njs_object(desc);
 
-    njs_object_property_init(&lhq, &get_string, NJS_GET_HASH);
+    lhq.proto = &njs_object_hash_proto;
+    lhq.key_hash = NJS_ATOM_STRING_get;
 
     ret = njs_object_property(vm, desc_object, &lhq, &value);
     if (njs_slow_path(ret == NJS_ERROR)) {
@@ -768,8 +703,7 @@ njs_descriptor_prop(njs_vm_t *vm, const njs_value_t *name,
         getter = njs_is_function(&value) ? njs_function(&value) : NULL;
     }
 
-    lhq.key = njs_str_value("set");
-    lhq.key_hash = NJS_SET_HASH;
+    lhq.key_hash = NJS_ATOM_STRING_set;
 
     ret = njs_object_property(vm, desc_object, &lhq, &value);
     if (njs_slow_path(ret == NJS_ERROR)) {
@@ -786,8 +720,7 @@ njs_descriptor_prop(njs_vm_t *vm, const njs_value_t *name,
         setter = njs_is_function(&value) ? njs_function(&value) : NULL;
     }
 
-    lhq.key = njs_str_value("value");
-    lhq.key_hash = NJS_VALUE_HASH;
+    lhq.key_hash = NJS_ATOM_STRING_value;
 
     ret = njs_object_property(vm, desc_object, &lhq, &value);
     if (njs_slow_path(ret == NJS_ERROR)) {
@@ -799,8 +732,7 @@ njs_descriptor_prop(njs_vm_t *vm, const njs_value_t *name,
         njs_value_assign(njs_prop_value(prop), &value);
     }
 
-    lhq.key = njs_str_value("writable");
-    lhq.key_hash = NJS_WRITABABLE_HASH;
+    lhq.key_hash = NJS_ATOM_STRING_writable;
 
     ret = njs_object_property(vm, desc_object, &lhq, &value);
     if (njs_slow_path(ret == NJS_ERROR)) {
@@ -810,6 +742,7 @@ njs_descriptor_prop(njs_vm_t *vm, const njs_value_t *name,
     if (ret == NJS_OK) {
         data = 1;
         prop->writable = njs_is_true(&value);
+        *set_writable = 1;
     }
 
     if (accessor && data) {
@@ -818,8 +751,7 @@ njs_descriptor_prop(njs_vm_t *vm, const njs_value_t *name,
         return NULL;
     }
 
-    lhq.key = njs_str_value("enumerable");
-    lhq.key_hash = NJS_ENUMERABLE_HASH;
+    lhq.key_hash = NJS_ATOM_STRING_enumerable;
 
     ret = njs_object_property(vm, desc_object, &lhq, &value);
     if (njs_slow_path(ret == NJS_ERROR)) {
@@ -828,10 +760,10 @@ njs_descriptor_prop(njs_vm_t *vm, const njs_value_t *name,
 
     if (ret == NJS_OK) {
         prop->enumerable = njs_is_true(&value);
+        *set_enumerable = 1;
     }
 
-    lhq.key = njs_str_value("configurable");
-    lhq.key_hash = NJS_CONFIGURABLE_HASH;
+    lhq.key_hash = NJS_ATOM_STRING_configurable;
 
     ret = njs_object_property(vm, desc_object, &lhq, &value);
     if (njs_slow_path(ret == NJS_ERROR)) {
@@ -840,6 +772,7 @@ njs_descriptor_prop(njs_vm_t *vm, const njs_value_t *name,
 
     if (ret == NJS_OK) {
         prop->configurable = njs_is_true(&value);
+        *set_configurable = 1;
     }
 
     if (accessor) {
@@ -852,17 +785,6 @@ njs_descriptor_prop(njs_vm_t *vm, const njs_value_t *name,
 }
 
 
-static const njs_value_t  njs_object_value_string = njs_string("value");
-static const njs_value_t  njs_object_get_string = njs_string("get");
-static const njs_value_t  njs_object_set_string = njs_string("set");
-static const njs_value_t  njs_object_writable_string =
-                                                    njs_string("writable");
-static const njs_value_t  njs_object_enumerable_string =
-                                                    njs_string("enumerable");
-static const njs_value_t  njs_object_configurable_string =
-                                                    njs_string("configurable");
-
-
 njs_int_t
 njs_object_prop_descriptor(njs_vm_t *vm, njs_value_t *dest,
     njs_value_t *value, njs_value_t *key)
@@ -871,10 +793,10 @@ njs_object_prop_descriptor(njs_vm_t *vm, njs_value_t *dest,
     njs_object_t          *desc;
     njs_object_prop_t     *pr, *prop;
     const njs_value_t     *setval;
-    njs_lvlhsh_query_t    lhq;
+    njs_flathsh_query_t   lhq;
     njs_property_query_t  pq;
 
-    njs_property_query_init(&pq, NJS_PROPERTY_QUERY_GET, 0, 1);
+    njs_property_query_init(&pq, NJS_PROPERTY_QUERY_GET, 1);
 
     if (njs_slow_path(!njs_is_key(key))) {
         ret = njs_value_to_key(vm, key, key);
@@ -883,7 +805,7 @@ njs_object_prop_descriptor(njs_vm_t *vm, njs_value_t *dest,
         }
     }
 
-    ret = njs_property_query(vm, &pq, value, key);
+    ret = njs_property_query_val(vm, &pq, value, key);
 
     switch (ret) {
     case NJS_OK:
@@ -897,7 +819,7 @@ njs_object_prop_descriptor(njs_vm_t *vm, njs_value_t *dest,
         case NJS_PROPERTY_HANDLER:
             pq.scratch = *prop;
             prop = &pq.scratch;
-            ret = njs_prop_handler(prop)(vm, prop, value, NULL,
+            ret = njs_prop_handler(prop)(vm, prop, key->atom_id, value, NULL,
                                          njs_prop_value(prop));
             if (njs_slow_path(ret == NJS_ERROR)) {
                 return ret;
@@ -932,123 +854,119 @@ njs_object_prop_descriptor(njs_vm_t *vm, njs_value_t *dest,
     lhq.pool = vm->mem_pool;
 
     if (njs_is_data_descriptor(prop)) {
+        lhq.key_hash = NJS_ATOM_STRING_value;
 
-        lhq.key = njs_str_value("value");
-        lhq.key_hash = NJS_VALUE_HASH;
-
-        pr = njs_object_prop_alloc(vm, &njs_object_value_string,
-                                   njs_prop_value(prop), 1);
-        if (njs_slow_path(pr == NULL)) {
-            return NJS_ERROR;
-        }
-
-        lhq.value = pr;
-
-        ret = njs_lvlhsh_insert(&desc->hash, &lhq);
+        ret = njs_flathsh_unique_insert(&desc->hash, &lhq);
         if (njs_slow_path(ret != NJS_OK)) {
             njs_internal_error(vm, "lvlhsh insert failed");
             return NJS_ERROR;
         }
 
-        lhq.key = njs_str_value("writable");
-        lhq.key_hash = NJS_WRITABABLE_HASH;
+        pr = lhq.value;
+
+        pr->type = NJS_PROPERTY;
+        pr->enumerable = 1;
+        pr->configurable = 1;
+        pr->writable = 1;
+        pr->u.value = *(njs_prop_value(prop));
+
+        lhq.key_hash = NJS_ATOM_STRING_writable;
 
         setval = (prop->writable == 1) ? &njs_value_true : &njs_value_false;
 
-        pr = njs_object_prop_alloc(vm, &njs_object_writable_string, setval, 1);
-        if (njs_slow_path(pr == NULL)) {
-            return NJS_ERROR;
-        }
-
-        lhq.value = pr;
-
-        ret = njs_lvlhsh_insert(&desc->hash, &lhq);
+        ret = njs_flathsh_unique_insert(&desc->hash, &lhq);
         if (njs_slow_path(ret != NJS_OK)) {
             njs_internal_error(vm, "lvlhsh insert failed");
             return NJS_ERROR;
         }
 
+        pr = lhq.value;
+
+        pr->type = NJS_PROPERTY;
+        pr->enumerable = 1;
+        pr->configurable = 1;
+        pr->writable = 1;
+        pr->u.value = *setval;
+
     } else {
 
-        lhq.key = njs_str_value("get");
-        lhq.key_hash = NJS_GET_HASH;
+        lhq.key_hash = NJS_ATOM_STRING_get;
 
-        pr = njs_object_prop_alloc(vm, &njs_object_get_string,
-                                   &njs_value_undefined, 1);
-        if (njs_slow_path(pr == NULL)) {
+        ret = njs_flathsh_unique_insert(&desc->hash, &lhq);
+        if (njs_slow_path(ret != NJS_OK)) {
+            njs_internal_error(vm, "lvlhsh insert failed");
             return NJS_ERROR;
         }
 
+        pr = lhq.value;
+
+        pr->type = NJS_PROPERTY;
+        pr->enumerable = 1;
+        pr->configurable = 1;
+        pr->writable = 1;
+        pr->u.value = njs_value_undefined;
+
         if (njs_prop_getter(prop) != NULL) {
             njs_set_function(njs_prop_value(pr), njs_prop_getter(prop));
         }
 
-        lhq.value = pr;
+        lhq.key_hash = NJS_ATOM_STRING_set;
 
-        ret = njs_lvlhsh_insert(&desc->hash, &lhq);
+        ret = njs_flathsh_unique_insert(&desc->hash, &lhq);
         if (njs_slow_path(ret != NJS_OK)) {
             njs_internal_error(vm, "lvlhsh insert failed");
             return NJS_ERROR;
         }
 
-        lhq.key = njs_str_value("set");
-        lhq.key_hash = NJS_SET_HASH;
+        pr = lhq.value;
 
-        pr = njs_object_prop_alloc(vm, &njs_object_set_string,
-                                   &njs_value_undefined, 1);
-        if (njs_slow_path(pr == NULL)) {
-            return NJS_ERROR;
-        }
+        pr->type = NJS_PROPERTY;
+        pr->enumerable = 1;
+        pr->configurable = 1;
+        pr->writable = 1;
+        pr->u.value = njs_value_undefined;
 
         if (njs_prop_setter(prop) != NULL) {
             njs_set_function(njs_prop_value(pr), njs_prop_setter(prop));
         }
-
-        lhq.value = pr;
-
-        ret = njs_lvlhsh_insert(&desc->hash, &lhq);
-        if (njs_slow_path(ret != NJS_OK)) {
-            njs_internal_error(vm, "lvlhsh insert failed");
-            return NJS_ERROR;
-        }
     }
 
-    lhq.key = njs_str_value("enumerable");
-    lhq.key_hash = NJS_ENUMERABLE_HASH;
+    lhq.key_hash = NJS_ATOM_STRING_enumerable;
 
     setval = (prop->enumerable == 1) ? &njs_value_true : &njs_value_false;
 
-    pr = njs_object_prop_alloc(vm, &njs_object_enumerable_string, setval, 1);
-    if (njs_slow_path(pr == NULL)) {
-        return NJS_ERROR;
-    }
-
-    lhq.value = pr;
-
-    ret = njs_lvlhsh_insert(&desc->hash, &lhq);
+    ret = njs_flathsh_unique_insert(&desc->hash, &lhq);
     if (njs_slow_path(ret != NJS_OK)) {
         njs_internal_error(vm, "lvlhsh insert failed");
         return NJS_ERROR;
     }
 
-    lhq.key = njs_str_value("configurable");
-    lhq.key_hash = NJS_CONFIGURABLE_HASH;
+    pr = lhq.value;
+
+    pr->type = NJS_PROPERTY;
+    pr->enumerable = 1;
+    pr->configurable = 1;
+    pr->writable = 1;
+    pr->u.value = *setval;
+
+    lhq.key_hash = NJS_ATOM_STRING_configurable;
 
     setval = (prop->configurable == 1) ? &njs_value_true : &njs_value_false;
 
-    pr = njs_object_prop_alloc(vm, &njs_object_configurable_string, setval, 1);
-    if (njs_slow_path(pr == NULL)) {
-        return NJS_ERROR;
-    }
-
-    lhq.value = pr;
-
-    ret = njs_lvlhsh_insert(&desc->hash, &lhq);
+    ret = njs_flathsh_unique_insert(&desc->hash, &lhq);
     if (njs_slow_path(ret != NJS_OK)) {
         njs_internal_error(vm, "lvlhsh insert failed");
         return NJS_ERROR;
     }
 
+    pr = lhq.value;
+
+    pr->type = NJS_PROPERTY;
+    pr->enumerable = 1;
+    pr->configurable = 1;
+    pr->writable = 1;
+    pr->u.value = *setval;
+
     njs_set_object(dest, desc);
 
     return NJS_OK;
@@ -1072,6 +990,9 @@ njs_prop_type_string(njs_object_prop_type_t type)
     case NJS_PROPERTY:
         return "property";
 
+    case NJS_FREE_FLATHSH_ELEMENT:
+        return "free hash element";
+
     default:
         return "unknown";
     }
@@ -1079,13 +1000,14 @@ njs_prop_type_string(njs_object_prop_type_t type)
 
 
 njs_int_t
-njs_object_prop_init(njs_vm_t *vm, const njs_object_init_t* init,
-    const njs_object_prop_t *base, njs_value_t *value, njs_value_t *retval)
+njs_object_props_init(njs_vm_t *vm, const njs_object_init_t* init,
+    njs_object_prop_t *base, uint32_t atom_id, njs_value_t *value,
+    njs_value_t *retval)
 {
-    njs_int_t           ret;
-    njs_object_t        *object;
-    njs_object_prop_t   *prop;
-    njs_lvlhsh_query_t  lhq;
+    njs_int_t            ret;
+    njs_object_t         *object;
+    njs_object_prop_t    *prop;
+    njs_flathsh_query_t  lhq;
 
     object = njs_object_alloc(vm);
     if (object == NULL) {
@@ -1098,32 +1020,26 @@ njs_object_prop_init(njs_vm_t *vm, const njs_object_init_t* init,
         return NJS_ERROR;
     }
 
-    prop = njs_mp_align(vm->mem_pool, sizeof(njs_value_t),
-                        sizeof(njs_object_prop_t));
-    if (njs_slow_path(prop == NULL)) {
-        njs_memory_error(vm);
+    lhq.key_hash = atom_id;
+    lhq.replace = 1;
+    lhq.pool = vm->mem_pool;
+    lhq.proto = &njs_object_hash_proto;
+
+    ret = njs_flathsh_unique_insert(njs_object_hash(value), &lhq);
+    if (njs_slow_path(ret != NJS_OK)) {
+        njs_internal_error(vm, "lvlhsh insert failed");
         return NJS_ERROR;
     }
 
-    *prop = *base;
+    prop = lhq.value;
 
+    prop->enumerable = base->enumerable;
+    prop->configurable = base->configurable;
+    prop->writable = base->writable;
     prop->type = NJS_PROPERTY;
     njs_set_object(njs_prop_value(prop), object);
 
-    lhq.proto = &njs_object_hash_proto;
-    njs_string_get(&prop->name, &lhq.key);
-    lhq.key_hash = njs_djb_hash(lhq.key.start, lhq.key.length);
-    lhq.value = prop;
-    lhq.replace = 1;
-    lhq.pool = vm->mem_pool;
+    njs_value_assign(retval, njs_prop_value(prop));
 
-    ret = njs_lvlhsh_insert(njs_object_hash(value), &lhq);
-    if (njs_fast_path(ret == NJS_OK)) {
-        njs_value_assign(retval, njs_prop_value(prop));
-        return NJS_OK;
-    }
-
-    njs_internal_error(vm, "lvlhsh insert failed");
-
-    return NJS_ERROR;
+    return NJS_OK;
 }
