@@ -61,8 +61,6 @@ static njs_int_t njs_promise_value_constructor(njs_vm_t *vm, njs_value_t *value,
 static njs_int_t njs_promise_capability_executor(njs_vm_t *vm,
     njs_value_t *args, njs_uint_t nargs, njs_index_t unused,
     njs_value_t *retval);
-static njs_int_t njs_promise_host_rejection_tracker(njs_vm_t *vm,
-    njs_promise_t *promise, njs_promise_rejection_type_t operation);
 static njs_int_t njs_promise_resolve_function(njs_vm_t *vm, njs_value_t *args,
     njs_uint_t nargs, njs_index_t unused, njs_value_t *retval);
 static njs_int_t njs_promise_reject_function(njs_vm_t *vm, njs_value_t *args,
@@ -107,11 +105,6 @@ static njs_int_t njs_promise_perform_race_handler(njs_vm_t *vm,
     njs_value_t *retval);
 
 
-static const njs_value_t  string_resolve = njs_string("resolve");
-static const njs_value_t  string_any_rejected =
-                                 njs_long_string("All promises were rejected");
-
-
 static njs_promise_t *
 njs_promise_alloc(njs_vm_t *vm)
 {
@@ -393,15 +386,12 @@ njs_promise_value_constructor(njs_vm_t *vm, njs_value_t *value,
 {
     njs_int_t  ret;
 
-    static const njs_value_t  string_constructor = njs_string("constructor");
-
     if (njs_is_function(value)) {
         *dst = *value;
         return NJS_OK;
     }
 
-    ret = njs_value_property(vm, value, njs_value_arg(&string_constructor),
-                             dst);
+    ret = njs_value_property(vm, value, NJS_ATOM_STRING_constructor, dst);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return ret;
     }
@@ -449,39 +439,6 @@ njs_promise_capability_executor(njs_vm_t *vm, njs_value_t *args,
 }
 
 
-njs_inline njs_int_t
-njs_promise_add_event(njs_vm_t *vm, njs_function_t *function, njs_value_t *args,
-    njs_uint_t nargs)
-{
-    njs_event_t  *event;
-
-    event = njs_mp_zalloc(vm->mem_pool, sizeof(njs_event_t));
-    if (njs_slow_path(event == NULL)) {
-        njs_memory_error(vm);
-        return NJS_ERROR;
-    }
-
-    event->function = function;
-    event->once = 1;
-
-    if (nargs != 0) {
-        event->args = njs_mp_alloc(vm->mem_pool, sizeof(njs_value_t) * nargs);
-        if (njs_slow_path(event->args == NULL)) {
-            njs_memory_error(vm);
-            return NJS_ERROR;
-        }
-
-        memcpy(event->args, args, sizeof(njs_value_t) * nargs);
-
-        event->nargs = nargs;
-    }
-
-    njs_queue_insert_tail(&vm->promise_events, &event->link);
-
-    return NJS_OK;
-}
-
-
 njs_inline njs_value_t *
 njs_promise_trigger_reactions(njs_vm_t *vm, njs_value_t *value,
     njs_queue_t *queue)
@@ -500,12 +457,16 @@ njs_promise_trigger_reactions(njs_vm_t *vm, njs_value_t *value,
 
         function = njs_promise_create_function(vm,
                                                sizeof(njs_promise_context_t));
+        if (njs_slow_path(function == NULL)) {
+            return njs_value_arg(&njs_value_null);
+        }
+
         function->u.native = njs_promise_reaction_job;
 
         njs_set_data(&arguments[0], reaction, 0);
         arguments[1] = *value;
 
-        ret = njs_promise_add_event(vm, function, arguments, 2);
+        ret = njs_vm_enqueue_job(vm, function, arguments, 2);
         if (njs_slow_path(ret != NJS_OK)) {
             return njs_value_arg(&njs_value_null);
         }
@@ -546,8 +507,8 @@ njs_promise_fulfill(njs_vm_t *vm, njs_promise_t *promise, njs_value_t *value)
 njs_inline njs_value_t *
 njs_promise_reject(njs_vm_t *vm, njs_promise_t *promise, njs_value_t *reason)
 {
-    njs_int_t           ret;
     njs_queue_t         queue;
+    njs_value_t         promise_value;
     njs_promise_data_t  *data;
 
     data = njs_data(&promise->value);
@@ -556,10 +517,10 @@ njs_promise_reject(njs_vm_t *vm, njs_promise_t *promise, njs_value_t *reason)
     data->state = NJS_PROMISE_REJECTED;
 
     if (!data->is_handled) {
-        ret = njs_promise_host_rejection_tracker(vm, promise,
-                                                 NJS_PROMISE_REJECT);
-        if (njs_slow_path(ret != NJS_OK)) {
-            return njs_value_arg(&njs_value_null);
+        if (vm->rejection_tracker != NULL) {
+            njs_set_promise(&promise_value, promise);
+            vm->rejection_tracker(vm, vm->rejection_tracker_opaque, 0,
+                                  &promise_value, reason);
         }
     }
 
@@ -580,58 +541,6 @@ njs_promise_reject(njs_vm_t *vm, njs_promise_t *promise, njs_value_t *reason)
 }
 
 
-static njs_int_t
-njs_promise_host_rejection_tracker(njs_vm_t *vm, njs_promise_t *promise,
-    njs_promise_rejection_type_t operation)
-{
-    uint32_t            i, length;
-    njs_value_t         *value;
-    njs_promise_data_t  *data;
-
-    if (vm->options.unhandled_rejection
-        == NJS_VM_OPT_UNHANDLED_REJECTION_IGNORE)
-    {
-        return NJS_OK;
-    }
-
-    if (vm->promise_reason == NULL) {
-        vm->promise_reason = njs_array_alloc(vm, 1, 0, NJS_ARRAY_SPARE);
-        if (njs_slow_path(vm->promise_reason == NULL)) {
-            return NJS_ERROR;
-        }
-    }
-
-    data = njs_data(&promise->value);
-
-    if (operation == NJS_PROMISE_REJECT) {
-        if (vm->promise_reason != NULL) {
-            return njs_array_add(vm, vm->promise_reason, &data->result);
-        }
-
-    } else {
-        value = vm->promise_reason->start;
-        length = vm->promise_reason->length;
-
-        for (i = 0; i < length; i++) {
-            if (njs_values_same(&value[i], &data->result)) {
-                length--;
-
-                if (i < length) {
-                    memmove(&value[i], &value[i + 1],
-                            sizeof(njs_value_t) * (length - i));
-                }
-
-                break;
-            }
-        }
-
-        vm->promise_reason->length = length;
-    }
-
-    return NJS_OK;
-}
-
-
 static njs_int_t
 njs_promise_invoke_then(njs_vm_t *vm, njs_value_t *promise, njs_value_t *args,
     njs_int_t nargs, njs_value_t *retval)
@@ -639,10 +548,7 @@ njs_promise_invoke_then(njs_vm_t *vm, njs_value_t *promise, njs_value_t *args,
     njs_int_t    ret;
     njs_value_t  function;
 
-    static const njs_value_t  string_then = njs_string("then");
-
-    ret = njs_value_property(vm, promise, njs_value_arg(&string_then),
-                             &function);
+    ret = njs_value_property(vm, promise, NJS_ATOM_STRING_then, &function);
     if (njs_slow_path(ret != NJS_OK)) {
         if (ret == NJS_DECLINED) {
             goto failed;
@@ -675,8 +581,6 @@ njs_promise_resolve_function(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     njs_native_frame_t     *active_frame;
     njs_promise_context_t  *context;
 
-    static const njs_value_t  string_then = njs_string("then");
-
     active_frame = vm->top_frame;
     context = active_frame->function->context;
     promise = njs_promise(&context->promise);
@@ -690,7 +594,7 @@ njs_promise_resolve_function(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
     resolution = njs_arg(args, nargs, 1);
 
-    if (njs_values_same(resolution, &context->promise)) {
+    if (njs_values_same(vm, resolution, &context->promise)) {
         njs_error_fmt_new(vm, &error, NJS_OBJ_TYPE_TYPE_ERROR,
                           "promise self resolution");
         if (njs_slow_path(!njs_is_error(&error))) {
@@ -706,8 +610,7 @@ njs_promise_resolve_function(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         goto fulfill;
     }
 
-    ret = njs_value_property(vm, resolution, njs_value_arg(&string_then),
-                             &then);
+    ret = njs_value_property(vm, resolution, NJS_ATOM_STRING_then, &then);
     if (njs_slow_path(ret == NJS_ERROR)) {
         if (njs_slow_path(njs_is_memory_error(vm, &vm->exception))) {
             return NJS_ERROR;
@@ -738,7 +641,7 @@ njs_promise_resolve_function(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
     function->u.native = njs_promise_resolve_thenable_job;
 
-    ret = njs_promise_add_event(vm, function, arguments, 3);
+    ret = njs_vm_enqueue_job(vm, function, arguments, 3);
     if (njs_slow_path(ret != NJS_OK)) {
         return ret;
     }
@@ -762,58 +665,49 @@ static njs_int_t
 njs_promise_object_resolve(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     njs_index_t unused, njs_value_t *retval)
 {
-    njs_promise_t  *promise;
-
     if (njs_slow_path(!njs_is_object(njs_argument(args, 0)))) {
         njs_type_error(vm, "this value is not an object");
         return NJS_ERROR;
     }
 
-    promise = njs_promise_resolve(vm, njs_argument(args, 0),
-                                  njs_arg(args, nargs, 1));
-    if (njs_slow_path(promise == NULL)) {
-        return NJS_ERROR;
-    }
-
-    njs_set_promise(retval, promise);
-
-    return NJS_OK;
+    return njs_promise_resolve(vm, njs_argument(args, 0),
+                               njs_arg(args, nargs, 1), retval);
 }
 
 
-njs_promise_t *
-njs_promise_resolve(njs_vm_t *vm, njs_value_t *constructor, njs_value_t *x)
+njs_int_t
+njs_promise_resolve(njs_vm_t *vm, njs_value_t *constructor, njs_value_t *x,
+    njs_value_t *retval)
 {
     njs_int_t                 ret;
     njs_value_t               value;
     njs_promise_capability_t  *capability;
 
-    static const njs_value_t  string_constructor = njs_string("constructor");
-
     if (njs_is_promise(x)) {
-        ret = njs_value_property(vm, x, njs_value_arg(&string_constructor),
-                                 &value);
+        ret = njs_value_property(vm, x, NJS_ATOM_STRING_constructor, &value);
         if (njs_slow_path(ret == NJS_ERROR)) {
-            return NULL;
+            return NJS_ERROR;
         }
 
-        if (njs_values_same(&value, constructor)) {
-            return njs_promise(x);
+        if (njs_values_same(vm, &value, constructor)) {
+            njs_value_assign(retval, x);
+            return NJS_OK;
         }
     }
 
     capability = njs_promise_new_capability(vm, constructor);
     if (njs_slow_path(capability == NULL)) {
-        return NULL;
+        return NJS_ERROR;
     }
 
     ret = njs_function_call(vm, njs_function(&capability->resolve),
                             &njs_value_undefined, x, 1, &value);
     if (njs_slow_path(ret != NJS_OK)) {
-        return NULL;
+        return ret;
     }
 
-    return njs_promise(&capability->promise);
+    njs_value_assign(retval, &capability->promise);
+    return NJS_OK;
 }
 
 
@@ -894,6 +788,11 @@ njs_promise_prototype_then(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     }
 
     function = njs_promise_create_function(vm, sizeof(njs_promise_context_t));
+    if (njs_slow_path(function == NULL)) {
+        /* vm error is already set by njs_promise_create_function */
+        return NJS_ERROR;
+    }
+
     function->u.native = njs_promise_constructor;
 
     njs_set_function(&constructor, function);
@@ -929,7 +828,7 @@ njs_promise_perform_then(njs_vm_t *vm, njs_value_t *value,
     njs_promise_capability_t *capability, njs_value_t *retval)
 {
     njs_int_t               ret;
-    njs_value_t             arguments[2];
+    njs_value_t             arguments[2], promise_value;
     njs_promise_t           *promise;
     njs_function_t          *function;
     njs_promise_data_t      *data;
@@ -982,10 +881,10 @@ njs_promise_perform_then(njs_vm_t *vm, njs_value_t *value,
         if (data->state == NJS_PROMISE_REJECTED) {
             njs_set_data(&arguments[0], rejected_reaction, 0);
 
-            ret = njs_promise_host_rejection_tracker(vm, promise,
-                                                     NJS_PROMISE_HANDLE);
-            if (njs_slow_path(ret != NJS_OK)) {
-                return ret;
+            if (vm->rejection_tracker != NULL) {
+                njs_set_promise(&promise_value, promise);
+                vm->rejection_tracker(vm, vm->rejection_tracker_opaque, 1,
+                                      &promise_value, &data->result);
             }
 
         } else {
@@ -994,7 +893,7 @@ njs_promise_perform_then(njs_vm_t *vm, njs_value_t *value,
 
         arguments[1] = data->result;
 
-        ret = njs_promise_add_event(vm, function, arguments, 2);
+        ret = njs_vm_enqueue_job(vm, function, arguments, 2);
         if (njs_slow_path(ret != NJS_OK)) {
             return ret;
         }
@@ -1104,7 +1003,6 @@ njs_promise_then_finally_function(njs_vm_t *vm, njs_value_t *args,
 {
     njs_int_t              ret;
     njs_value_t            value, argument;
-    njs_promise_t          *promise;
     njs_function_t         *function;
     njs_native_frame_t     *frame;
     njs_promise_context_t  *context;
@@ -1118,13 +1016,11 @@ njs_promise_then_finally_function(njs_vm_t *vm, njs_value_t *args,
         return ret;
     }
 
-    promise = njs_promise_resolve(vm, &context->constructor, &value);
-    if (njs_slow_path(promise == NULL)) {
+    ret = njs_promise_resolve(vm, &context->constructor, &value, &value);
+    if (njs_slow_path(ret != NJS_OK)) {
         return NJS_ERROR;
     }
 
-    njs_set_promise(&value, promise);
-
     function = njs_promise_create_function(vm, sizeof(njs_value_t));
     if (njs_slow_path(function == NULL)) {
         return NJS_ERROR;
@@ -1269,7 +1165,7 @@ njs_promise_all(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         return NJS_ERROR;
     }
 
-    ret = njs_value_property(vm, promise_ctor, njs_value_arg(&string_resolve),
+    ret = njs_value_property(vm, promise_ctor, NJS_ATOM_STRING_resolve,
                              &resolve);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return ret;
@@ -1309,7 +1205,7 @@ njs_promise_perform_all(njs_vm_t *vm, njs_value_t *iterator,
 {
     int64_t       length;
     njs_int_t     ret;
-    njs_value_t   argument;
+    njs_value_t   argument, message;
     njs_object_t  *error;
 
     if (njs_slow_path(!njs_is_object(pargs->constructor))) {
@@ -1349,11 +1245,13 @@ njs_promise_perform_all(njs_vm_t *vm, njs_value_t *iterator,
         njs_mp_free(vm->mem_pool, pargs->remaining);
 
         njs_set_array(&argument, pargs->args.data);
+        njs_atom_to_value(vm, &message,
+                          NJS_ATOM_STRING_All_promises_were_rejected);
 
         if (handler == njs_promise_perform_any_handler) {
             error = njs_error_alloc(vm,
                                 njs_vm_proto(vm, NJS_OBJ_TYPE_AGGREGATE_ERROR),
-                                NULL, &string_any_rejected, &argument);
+                                NULL, &message, &argument);
             if (njs_slow_path(error == NULL)) {
                 return NJS_ERROR;
             }
@@ -1563,17 +1461,12 @@ njs_promise_all_settled_element_functions(njs_vm_t *vm,
     njs_value_t *retval)
 {
     njs_int_t                  ret;
+    uint32_t                   set_atom_id;
+    njs_value_t                status;
     njs_value_t                obj_value, arr_value;
     njs_object_t               *obj;
-    const njs_value_t          *status, *set;
     njs_promise_all_context_t  *context;
 
-    static const njs_value_t  string_status = njs_string("status");
-    static const njs_value_t  string_fulfilled = njs_string("fulfilled");
-    static const njs_value_t  string_value = njs_string("value");
-    static const njs_value_t  string_rejected = njs_string("rejected");
-    static const njs_value_t  string_reason = njs_string("reason");
-
     context = vm->top_frame->function->context;
 
     if (context->already_called) {
@@ -1591,21 +1484,21 @@ njs_promise_all_settled_element_functions(njs_vm_t *vm,
     njs_set_object(&obj_value, obj);
 
     if (rejected) {
-        status = &string_rejected;
-        set = &string_reason;
+        njs_atom_to_value(vm, &status, NJS_ATOM_STRING_rejected);
+        set_atom_id = NJS_ATOM_STRING_reason;
 
     } else {
-        status = &string_fulfilled;
-        set = &string_value;
+        njs_atom_to_value(vm, &status, NJS_ATOM_STRING_fulfilled);
+        set_atom_id = NJS_ATOM_STRING_value;
     }
 
-    ret = njs_value_property_set(vm, &obj_value, njs_value_arg(&string_status),
-                                 njs_value_arg(status));
+    ret = njs_value_property_set(vm, &obj_value, NJS_ATOM_STRING_status,
+                                 njs_value_arg(&status));
     if (njs_slow_path(ret == NJS_ERROR)) {
         return ret;
     }
 
-    ret = njs_value_property_set(vm, &obj_value, njs_value_arg(set),
+    ret = njs_value_property_set(vm, &obj_value, set_atom_id,
                                  njs_arg(args, nargs, 1));
     if (njs_slow_path(ret == NJS_ERROR)) {
         return ret;
@@ -1705,7 +1598,7 @@ njs_promise_any_reject_element_functions(njs_vm_t *vm, njs_value_t *args,
     njs_uint_t nargs, njs_index_t unused, njs_value_t *retval)
 {
     njs_int_t                  ret;
-    njs_value_t                argument, arr_value;
+    njs_value_t                argument, message, arr_value;
     njs_object_t               *error;
     njs_promise_all_context_t  *context;
 
@@ -1729,9 +1622,12 @@ njs_promise_any_reject_element_functions(njs_vm_t *vm, njs_value_t *args,
     if (--(*context->remaining_elements) == 0) {
         njs_mp_free(vm->mem_pool, context->remaining_elements);
 
+        njs_atom_to_value(vm, &message,
+                          NJS_ATOM_STRING_All_promises_were_rejected);
+
         error = njs_error_alloc(vm,
                                 njs_vm_proto(vm, NJS_OBJ_TYPE_AGGREGATE_ERROR),
-                                NULL, &string_any_rejected, &arr_value);
+                                NULL, &message, &arr_value);
         if (njs_slow_path(error == NULL)) {
             return NJS_ERROR;
         }
@@ -1766,7 +1662,7 @@ njs_promise_race(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         return NJS_ERROR;
     }
 
-    ret = njs_value_property(vm, promise_ctor, njs_value_arg(&string_resolve),
+    ret = njs_value_property(vm, promise_ctor, NJS_ATOM_STRING_resolve,
                              &resolve);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return ret;
@@ -1844,34 +1740,32 @@ njs_promise_species(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 }
 
 
-static const njs_object_prop_t  njs_promise_constructor_properties[] =
+static const njs_object_prop_init_t  njs_promise_constructor_properties[] =
 {
     NJS_DECLARE_PROP_LENGTH(1),
 
     NJS_DECLARE_PROP_NAME("Promise"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 
-    NJS_DECLARE_PROP_NATIVE("resolve", njs_promise_object_resolve, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_resolve, njs_promise_object_resolve, 1,
+                            0),
 
-    NJS_DECLARE_PROP_NATIVE("reject", njs_promise_object_reject, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_reject, njs_promise_object_reject, 1, 0),
 
-    NJS_DECLARE_PROP_NATIVE("all", njs_promise_all, 1, NJS_PROMISE_ALL),
+    NJS_DECLARE_PROP_NATIVE(STRING_all, njs_promise_all, 1,
+                            NJS_PROMISE_ALL),
 
-    NJS_DECLARE_PROP_NATIVE("allSettled", njs_promise_all, 1,
+    NJS_DECLARE_PROP_NATIVE(STRING_allSettled, njs_promise_all, 1,
                              NJS_PROMISE_ALL_SETTLED),
 
-    NJS_DECLARE_PROP_NATIVE("any", njs_promise_all, 1, NJS_PROMISE_ANY),
+    NJS_DECLARE_PROP_NATIVE(STRING_any, njs_promise_all, 1,
+                            NJS_PROMISE_ANY),
 
-    NJS_DECLARE_PROP_NATIVE("race", njs_promise_race, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_race, njs_promise_race, 1, 0),
 
-    {
-        .type = NJS_ACCESSOR,
-        .name = njs_wellknown_symbol(NJS_SYMBOL_SPECIES),
-        .u.accessor = njs_getter(njs_promise_species, 0),
-        .writable = NJS_ATTRIBUTE_UNSET,
-        .configurable = 1,
-    },
+    NJS_DECLARE_PROP_GETTER(SYMBOL_species, njs_promise_species, 0),
 };
 
 
@@ -1881,24 +1775,22 @@ const njs_object_init_t  njs_promise_constructor_init = {
 };
 
 
-static const njs_object_prop_t  njs_promise_prototype_properties[] =
+static const njs_object_prop_init_t  njs_promise_prototype_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    {
-        .type = NJS_PROPERTY,
-        .name = njs_wellknown_symbol(NJS_SYMBOL_TO_STRING_TAG),
-        .u.value = njs_string("Promise"),
-        .configurable = 1,
-    },
+    NJS_DECLARE_PROP_VALUE(SYMBOL_toStringTag, njs_ascii_strval("Promise"),
+                           NJS_OBJECT_PROP_VALUE_C),
 
-    NJS_DECLARE_PROP_NATIVE("then", njs_promise_prototype_then, 2, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_then, njs_promise_prototype_then, 2, 0),
 
-    NJS_DECLARE_PROP_NATIVE("catch", njs_promise_prototype_catch, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_catch, njs_promise_prototype_catch, 1,
+                            0),
 
-    NJS_DECLARE_PROP_NATIVE("finally", njs_promise_prototype_finally, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_finally, njs_promise_prototype_finally,
+                            1, 0),
 };
 
 
