@@ -30,11 +30,6 @@ static njs_int_t njs_regexp_exec(njs_vm_t *vm, njs_value_t *r, njs_value_t *s,
     unsigned flags, njs_value_t *retval);
 static njs_array_t *njs_regexp_exec_result(njs_vm_t *vm, njs_value_t *r,
     njs_utf8_t utf8, njs_string_prop_t *string, njs_regex_match_data_t *data);
-static njs_int_t njs_regexp_string_create(njs_vm_t *vm, njs_value_t *value,
-    u_char *start, uint32_t size, int32_t length);
-
-
-const njs_value_t  njs_string_lindex = njs_string("lastIndex");
 
 
 njs_int_t
@@ -136,9 +131,6 @@ njs_regexp_constructor(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
             if (njs_slow_path(ret != NJS_OK)) {
                 return ret;
             }
-
-        } else {
-            pattern = njs_value_arg(&njs_string_empty);
         }
 
         re_flags = 0;
@@ -152,7 +144,7 @@ njs_regexp_constructor(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
             return ret;
         }
 
-        njs_string_get(flags, &string);
+        njs_string_get(vm, flags, &string);
 
         start = string.start;
 
@@ -165,7 +157,12 @@ njs_regexp_constructor(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         }
     }
 
-    njs_string_get(pattern, &string);
+    if (njs_is_defined(pattern)) {
+        njs_string_get(vm, pattern, &string);
+
+    } else {
+        string = njs_str_value("");
+    }
 
     return njs_regexp_create(vm, retval, string.start, string.length,
                              re_flags);
@@ -515,7 +512,7 @@ njs_regexp_alloc(njs_vm_t *vm, njs_regexp_pattern_t *pattern)
         regexp->object.error_data = 0;
         njs_set_number(&regexp->last_index, 0);
         regexp->pattern = pattern;
-        njs_string_short_set(&regexp->string, 0, 0);
+        njs_set_empty_string(vm, &regexp->string);
         return regexp;
     }
 
@@ -527,7 +524,8 @@ njs_regexp_alloc(njs_vm_t *vm, njs_regexp_pattern_t *pattern)
 
 static njs_int_t
 njs_regexp_prototype_last_index(njs_vm_t *vm, njs_object_prop_t *unused,
-    njs_value_t *value, njs_value_t *setval, njs_value_t *retval)
+    uint32_t unused2, njs_value_t *value, njs_value_t *setval,
+    njs_value_t *retval)
 {
     njs_regexp_t  *regexp;
 
@@ -559,11 +557,6 @@ njs_regexp_prototype_flags(njs_vm_t *vm, njs_value_t *args,
     njs_value_t  *this, value;
     u_char       dst[4];
 
-    static const njs_value_t  string_global = njs_string("global");
-    static const njs_value_t  string_ignore_case = njs_string("ignoreCase");
-    static const njs_value_t  string_multiline = njs_string("multiline");
-    static const njs_value_t  string_sticky = njs_string("sticky");
-
     this = njs_argument(args, 0);
     if (njs_slow_path(!njs_is_object(this))) {
         njs_type_error(vm, "\"this\" argument is not an object");
@@ -572,8 +565,7 @@ njs_regexp_prototype_flags(njs_vm_t *vm, njs_value_t *args,
 
     p = &dst[0];
 
-    ret = njs_value_property(vm, this, njs_value_arg(&string_global),
-                             &value);
+    ret = njs_value_property(vm, this, NJS_ATOM_STRING_global, &value);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
     }
@@ -582,8 +574,7 @@ njs_regexp_prototype_flags(njs_vm_t *vm, njs_value_t *args,
         *p++ = 'g';
     }
 
-    ret = njs_value_property(vm, this, njs_value_arg(&string_ignore_case),
-                             &value);
+    ret = njs_value_property(vm, this, NJS_ATOM_STRING_ignoreCase, &value);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
     }
@@ -592,8 +583,7 @@ njs_regexp_prototype_flags(njs_vm_t *vm, njs_value_t *args,
         *p++ = 'i';
     }
 
-    ret = njs_value_property(vm, this, njs_value_arg(&string_multiline),
-                             &value);
+    ret = njs_value_property(vm, this, NJS_ATOM_STRING_multiline, &value);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
     }
@@ -602,8 +592,7 @@ njs_regexp_prototype_flags(njs_vm_t *vm, njs_value_t *args,
         *p++ = 'm';
     }
 
-    ret = njs_value_property(vm, this, njs_value_arg(&string_sticky),
-                             &value);
+    ret = njs_value_property(vm, this, NJS_ATOM_STRING_sticky, &value);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
     }
@@ -683,7 +672,7 @@ njs_regexp_prototype_source(njs_vm_t *vm, njs_value_t *args,
 
     if (njs_slow_path(!njs_is_regexp(this))) {
         if (njs_object(this) == njs_vm_proto(vm, NJS_OBJ_TYPE_REGEXP)) {
-            njs_value_assign(retval, &njs_string_empty_regexp);
+            njs_atom_to_value(vm, retval, NJS_ATOM_STRING_spec_EMPTY_REGEXP);
             return NJS_OK;
         }
 
@@ -710,9 +699,6 @@ njs_regexp_prototype_to_string(njs_vm_t *vm, njs_value_t *args,
     njs_value_t        *r, source, flags;
     njs_string_prop_t  source_string, flags_string;
 
-    static const njs_value_t  string_source = njs_string("source");
-    static const njs_value_t  string_flags = njs_string("flags");
-
     r = njs_argument(args, 0);
 
     if (njs_slow_path(!njs_is_object(r))) {
@@ -720,8 +706,7 @@ njs_regexp_prototype_to_string(njs_vm_t *vm, njs_value_t *args,
         return NJS_ERROR;
     }
 
-    ret = njs_value_property(vm, r, njs_value_arg(&string_source),
-                             &source);
+    ret = njs_value_property(vm, r, NJS_ATOM_STRING_source, &source);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
     }
@@ -731,8 +716,7 @@ njs_regexp_prototype_to_string(njs_vm_t *vm, njs_value_t *args,
         return NJS_ERROR;
     }
 
-    ret = njs_value_property(vm, r, njs_value_arg(&string_flags),
-                             &flags);
+    ret = njs_value_property(vm, r, NJS_ATOM_STRING_flags, &flags);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
     }
@@ -742,8 +726,8 @@ njs_regexp_prototype_to_string(njs_vm_t *vm, njs_value_t *args,
         return NJS_ERROR;
     }
 
-    (void) njs_string_prop(&source_string, &source);
-    (void) njs_string_prop(&flags_string, &flags);
+    (void) njs_string_prop(vm, &source_string, &source);
+    (void) njs_string_prop(vm, &flags_string, &flags);
 
     size = source_string.size + flags_string.size + njs_length("//");
     length = source_string.length + flags_string.length + njs_length("//");
@@ -881,8 +865,7 @@ njs_regexp_builtin_exec(njs_vm_t *vm, njs_value_t *r, njs_value_t *s,
     regexp->string = *s;
     pattern = regexp->pattern;
 
-    ret = njs_value_property(vm, r, njs_value_arg(&njs_string_lindex),
-                             &value);
+    ret = njs_value_property(vm, r, NJS_ATOM_STRING_lastIndex, &value);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
     }
@@ -896,7 +879,7 @@ njs_regexp_builtin_exec(njs_vm_t *vm, njs_value_t *r, njs_value_t *s,
         last_index = 0;
     }
 
-    length = njs_string_prop(&string, s);
+    length = njs_string_prop(vm, &string, s);
 
     if (njs_slow_path((size_t) last_index > length)) {
         goto not_found;
@@ -954,8 +937,7 @@ njs_regexp_builtin_exec(njs_vm_t *vm, njs_value_t *r, njs_value_t *s,
             }
 
             njs_set_number(&value, index);
-            ret = njs_value_property_set(vm, r,
-                                         njs_value_arg(&njs_string_lindex),
+            ret = njs_value_property_set(vm, r, NJS_ATOM_STRING_lastIndex,
                                          &value);
             if (njs_slow_path(ret != NJS_OK)) {
                 njs_regex_match_data_free(match_data, vm->regex_generic_ctx);
@@ -991,8 +973,7 @@ not_found:
 
     if (pattern->global || pattern->sticky) {
         njs_set_number(&value, 0);
-        ret = njs_value_property_set(vm, r, njs_value_arg(&njs_string_lindex),
-                                     &value);
+        ret = njs_value_property_set(vm, r, NJS_ATOM_STRING_lastIndex, &value);
         if (njs_slow_path(ret != NJS_OK)) {
             return NJS_ERROR;
         }
@@ -1016,7 +997,7 @@ njs_regexp_exec_result(njs_vm_t *vm, njs_value_t *r, njs_utf8_t utf8,
 {
     u_char                *start;
     size_t                c;
-    int32_t               size, length;
+    int32_t               size;
     uint32_t              index;
     njs_int_t             ret;
     njs_uint_t            i, n;
@@ -1026,13 +1007,9 @@ njs_regexp_exec_result(njs_vm_t *vm, njs_value_t *r, njs_utf8_t utf8,
     njs_regexp_t          *regexp;
     njs_object_prop_t     *prop;
     njs_regexp_group_t    *group;
-    njs_lvlhsh_query_t    lhq;
+    njs_flathsh_query_t   lhq;
     njs_regexp_pattern_t  *pattern;
 
-    static const njs_value_t  string_index = njs_string("index");
-    static const njs_value_t  string_input = njs_string("input");
-    static const njs_value_t  string_groups = njs_string("groups");
-
     regexp = njs_regexp(r);
     pattern = regexp->pattern;
     array = njs_array_alloc(vm, 0, pattern->ncaptures, 0);
@@ -1050,15 +1027,7 @@ njs_regexp_exec_result(njs_vm_t *vm, njs_value_t *r, njs_utf8_t utf8,
             start = &string->start[c];
             size = njs_regex_capture(match_data, n + 1) - c;
 
-            if (utf8 == NJS_STRING_UTF8) {
-                length = njs_max(njs_utf8_length(start, size), 0);
-
-            } else {
-                length = size;
-            }
-
-            ret = njs_regexp_string_create(vm, &array->start[i], start, size,
-                                           length);
+            ret = njs_string_create(vm, &array->start[i], start, size);
             if (njs_slow_path(ret != NJS_OK)) {
                 goto fail;
             }
@@ -1069,11 +1038,24 @@ njs_regexp_exec_result(njs_vm_t *vm, njs_value_t *r, njs_utf8_t utf8,
     }
 
     /* FIXME: implement fast CreateDataPropertyOrThrow(). */
-    prop = njs_object_prop_alloc(vm, &string_index, &njs_value_undefined, 1);
-    if (njs_slow_path(prop == NULL)) {
-        goto fail;
+
+    lhq.key_hash = NJS_ATOM_STRING_index;
+    lhq.replace = 0;
+    lhq.pool = vm->mem_pool;
+    lhq.proto = &njs_object_hash_proto;
+
+    ret = njs_flathsh_unique_insert(&array->object.hash, &lhq);
+    if (njs_slow_path(ret != NJS_OK)) {
+        goto insert_fail;
     }
 
+    prop = lhq.value;
+
+    prop->type = NJS_PROPERTY;
+    prop->enumerable = 1;
+    prop->configurable = 1;
+    prop->writable = 1;
+
     c = njs_regex_capture(match_data, 0);
 
     if (utf8 == NJS_STRING_UTF8) {
@@ -1085,45 +1067,35 @@ njs_regexp_exec_result(njs_vm_t *vm, njs_value_t *r, njs_utf8_t utf8,
 
     njs_set_number(&prop->u.value, index);
 
-    lhq.key_hash = NJS_INDEX_HASH;
-    lhq.key = njs_str_value("index");
-    lhq.replace = 0;
-    lhq.value = prop;
-    lhq.pool = vm->mem_pool;
-    lhq.proto = &njs_object_hash_proto;
+    lhq.key_hash = NJS_ATOM_STRING_input;
 
-    ret = njs_lvlhsh_insert(&array->object.hash, &lhq);
+    ret = njs_flathsh_unique_insert(&array->object.hash, &lhq);
     if (njs_slow_path(ret != NJS_OK)) {
         goto insert_fail;
     }
 
-    prop = njs_object_prop_alloc(vm, &string_input, &regexp->string, 1);
-    if (njs_slow_path(prop == NULL)) {
-        goto fail;
-    }
+    prop = lhq.value;
 
-    lhq.key_hash = NJS_INPUT_HASH;
-    lhq.key = njs_str_value("input");
-    lhq.value = prop;
+    prop->type = NJS_PROPERTY;
+    prop->enumerable = 1;
+    prop->configurable = 1;
+    prop->writable = 1;
+    prop->u.value = regexp->string;
 
-    ret = njs_lvlhsh_insert(&array->object.hash, &lhq);
+    lhq.key_hash = NJS_ATOM_STRING_groups;
+
+    ret = njs_flathsh_unique_insert(&array->object.hash, &lhq);
     if (njs_slow_path(ret != NJS_OK)) {
         goto insert_fail;
     }
 
-    prop = njs_object_prop_alloc(vm, &string_groups, &njs_value_undefined, 1);
-    if (njs_slow_path(prop == NULL)) {
-        goto fail;
-    }
+    prop = lhq.value;
 
-    lhq.key_hash = NJS_GROUPS_HASH;
-    lhq.key = njs_str_value("groups");
-    lhq.value = prop;
-
-    ret = njs_lvlhsh_insert(&array->object.hash, &lhq);
-    if (njs_slow_path(ret != NJS_OK)) {
-        goto insert_fail;
-    }
+    prop->type = NJS_PROPERTY;
+    prop->enumerable = 1;
+    prop->configurable = 1;
+    prop->writable = 1;
+    prop->u.value = njs_value_undefined;
 
     if (pattern->ngroups != 0) {
         groups = njs_object_alloc(vm);
@@ -1138,27 +1110,27 @@ njs_regexp_exec_result(njs_vm_t *vm, njs_value_t *r, njs_utf8_t utf8,
         do {
             group = &pattern->groups[i];
 
-            ret = njs_string_create(vm, &name, group->name.start,
-                                    group->name.length);
+            ret = njs_atom_string_create(vm, &name, group->name.start,
+                                         group->name.length);
             if (njs_slow_path(ret != NJS_OK)) {
                 goto fail;
             }
 
-            prop = njs_object_prop_alloc(vm, &name,
-                                         &array->start[group->capture], 1);
-            if (njs_slow_path(prop == NULL)) {
-                goto fail;
-            }
+            lhq.key_hash = name.atom_id;
 
-            lhq.key_hash = group->hash;
-            lhq.key = group->name;
-            lhq.value = prop;
-
-            ret = njs_lvlhsh_insert(&groups->hash, &lhq);
+            ret = njs_flathsh_unique_insert(&groups->hash, &lhq);
             if (njs_slow_path(ret != NJS_OK)) {
                 goto insert_fail;
             }
 
+            prop = lhq.value;
+
+            prop->type = NJS_PROPERTY;
+            prop->enumerable = 1;
+            prop->configurable = 1;
+            prop->writable = 1;
+            prop->u.value = array->start[group->capture];
+
             i++;
 
         } while (i < pattern->ngroups);
@@ -1184,38 +1156,11 @@ done:
 static void
 njs_regexp_exec_result_free(njs_vm_t *vm, njs_array_t *result)
 {
-    njs_uint_t          n;
-    njs_value_t         *start;
-    njs_flathsh_t       *hash;
-    njs_object_prop_t   *prop;
-    njs_lvlhsh_each_t   lhe;
-    njs_lvlhsh_query_t  lhq;
-
-    if (result->object.fast_array) {
-        start = result->start;
-
-        for (n = 0; n < result->length; n++) {
-            if (start[n].short_string.size == NJS_STRING_LONG) {
-                njs_mp_free(vm->mem_pool, start[n].long_string.data);
-            }
-        }
-    }
-
-    njs_lvlhsh_each_init(&lhe, &njs_object_hash_proto);
+    njs_flathsh_t        *hash;
+    njs_flathsh_query_t  lhq;
 
     hash = &result->object.hash;
 
-    for ( ;; ) {
-        prop = njs_flathsh_each(hash, &lhe);
-
-        if (prop == NULL) {
-            break;
-        }
-
-        njs_mp_free(vm->mem_pool, prop);
-    }
-
-
     lhq.pool = vm->mem_pool;
     lhq.proto = &njs_object_hash_proto;
 
@@ -1267,9 +1212,7 @@ njs_regexp_exec(njs_vm_t *vm, njs_value_t *r, njs_value_t *s, unsigned flags,
     njs_value_t  exec;
     njs_value_t  arguments[2];
 
-    static const njs_value_t  string_exec = njs_string("exec");
-
-    ret = njs_value_property(vm, r, njs_value_arg(&string_exec), &exec);
+    ret = njs_value_property(vm, r, NJS_ATOM_STRING_exec, &exec);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
     }
@@ -1316,16 +1259,6 @@ njs_regexp_exec(njs_vm_t *vm, njs_value_t *r, njs_value_t *s, unsigned flags,
 }
 
 
-static njs_int_t
-njs_regexp_string_create(njs_vm_t *vm, njs_value_t *value, u_char *start,
-    uint32_t size, int32_t length)
-{
-    length = (length >= 0) ? length : 0;
-
-    return njs_string_new(vm, value, start, size, length);
-}
-
-
 njs_int_t
 njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
     njs_uint_t nargs, njs_index_t unused, njs_value_t *retval)
@@ -1344,10 +1277,6 @@ njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
     njs_function_t     *func_replace;
     njs_string_prop_t  s;
 
-    static const njs_value_t  string_global = njs_string("global");
-    static const njs_value_t  string_groups = njs_string("groups");
-    static const njs_value_t  string_index = njs_string("index");
-
     rx = njs_argument(args, 0);
 
     if (njs_slow_path(!njs_is_object(rx))) {
@@ -1362,7 +1291,7 @@ njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
         return ret;
     }
 
-    length = njs_string_prop(&s, string);
+    length = njs_string_prop(vm, &s, string);
 
     rep.start = NULL;
     rep.length = 0;
@@ -1377,7 +1306,7 @@ njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
         }
     }
 
-    ret = njs_value_property(vm, rx, njs_value_arg(&string_global), &value);
+    ret = njs_value_property(vm, rx, NJS_ATOM_STRING_global, &value);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
     }
@@ -1386,14 +1315,13 @@ njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
 
     if (global) {
         njs_set_number(&value, 0);
-        ret = njs_value_property_set(vm, rx, njs_value_arg(&njs_string_lindex),
-                                     &value);
+        ret = njs_value_property_set(vm, rx, NJS_ATOM_STRING_lastIndex, &value);
         if (njs_slow_path(ret != NJS_OK)) {
             return NJS_ERROR;
         }
     }
 
-    NJS_CHB_MP_INIT(&chain, vm);
+    NJS_CHB_MP_INIT(&chain, njs_vm_memory_pool(vm));
 
     results.separate = 0;
     results.pointer = 0;
@@ -1438,12 +1366,13 @@ njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
             goto exception;
         }
 
-        if (njs_string_length(&value) != 0) {
+        njs_string_get(vm, &value, &m);
+
+        if (m.length != 0) {
             continue;
         }
 
-        ret = njs_value_property(vm, rx, njs_value_arg(&njs_string_lindex),
-                                 &value);
+        ret = njs_value_property(vm, rx, NJS_ATOM_STRING_lastIndex, &value);
         if (njs_slow_path(ret == NJS_ERROR)) {
             goto exception;
         }
@@ -1454,8 +1383,7 @@ njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
         }
 
         njs_set_number(&value, last_index + 1);
-        ret = njs_value_property_set(vm, rx, njs_value_arg(&njs_string_lindex),
-                                     &value);
+        ret = njs_value_property_set(vm, rx, NJS_ATOM_STRING_lastIndex, &value);
         if (njs_slow_path(ret != NJS_OK)) {
             goto exception;
         }
@@ -1482,7 +1410,7 @@ njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
             goto exception;
         }
 
-        ret = njs_value_property(vm, r, njs_value_arg(&string_index), &value);
+        ret = njs_value_property(vm, r, NJS_ATOM_STRING_index, &value);
         if (njs_slow_path(ret == NJS_ERROR)) {
             goto exception;
         }
@@ -1525,7 +1453,7 @@ njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
             }
         }
 
-        ret = njs_value_property(vm, r, njs_value_arg(&string_groups), &groups);
+        ret = njs_value_property(vm, r, NJS_ATOM_STRING_groups, &groups);
         if (njs_slow_path(ret == NJS_ERROR)) {
             goto exception;
         }
@@ -1578,10 +1506,10 @@ njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
         if (p >= next) {
             njs_chb_append(&chain, next, p - next);
 
-            njs_string_get(retval, &rep);
+            njs_string_get(vm, retval, &rep);
             njs_chb_append_str(&chain, &rep);
 
-            njs_string_get(&matched, &m);
+            njs_string_get(vm, &matched, &m);
 
             next = p + m.length;
         }
@@ -1633,12 +1561,9 @@ njs_regexp_prototype_symbol_split(njs_vm_t *vm, njs_value_t *args,
     njs_value_t        r, z, this, s_lvalue, setval, constructor;
     njs_object_t       *object;
     const u_char       *start, *end;
-    njs_string_prop_t  s, sv;
+    njs_string_prop_t  s;
     njs_value_t        arguments[2];
 
-    static const njs_value_t  string_lindex = njs_string("lastIndex");
-    static const njs_value_t  string_flags = njs_string("flags");
-
     rx = njs_argument(args, 0);
 
     if (njs_slow_path(!njs_is_object(rx))) {
@@ -1660,7 +1585,7 @@ njs_regexp_prototype_symbol_split(njs_vm_t *vm, njs_value_t *args,
         return ret;
     }
 
-    ret = njs_value_property(vm, rx, njs_value_arg(&string_flags), retval);
+    ret = njs_value_property(vm, rx, NJS_ATOM_STRING_flags, retval);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
     }
@@ -1670,7 +1595,7 @@ njs_regexp_prototype_symbol_split(njs_vm_t *vm, njs_value_t *args,
         return ret;
     }
 
-    (void) njs_string_prop(&s, retval);
+    (void) njs_string_prop(vm, &s, retval);
 
     sticky = memchr(s.start, 'y', s.size) != NULL;
 
@@ -1725,7 +1650,7 @@ njs_regexp_prototype_symbol_split(njs_vm_t *vm, njs_value_t *args,
         goto done;
     }
 
-    length = njs_string_prop(&s, string);
+    length = njs_string_prop(vm, &s, string);
 
     if (njs_slow_path(s.size == 0)) {
         ret = njs_regexp_exec(vm, rx, string, NJS_REGEXP_FLAG_TEST, &z);
@@ -1751,8 +1676,7 @@ njs_regexp_prototype_symbol_split(njs_vm_t *vm, njs_value_t *args,
 
     while (q < length) {
         njs_set_number(&setval, q);
-        ret = njs_value_property_set(vm, rx, njs_value_arg(&string_lindex),
-                                     &setval);
+        ret = njs_value_property_set(vm, rx, NJS_ATOM_STRING_lastIndex, &setval);
         if (njs_slow_path(ret != NJS_OK)) {
             return NJS_ERROR;
         }
@@ -1767,8 +1691,7 @@ njs_regexp_prototype_symbol_split(njs_vm_t *vm, njs_value_t *args,
             continue;
         }
 
-        ret = njs_value_property(vm, rx, njs_value_arg(&string_lindex),
-                                 retval);
+        ret = njs_value_property(vm, rx, NJS_ATOM_STRING_lastIndex, retval);
         if (njs_slow_path(ret == NJS_ERROR)) {
             return NJS_ERROR;
         }
@@ -1828,10 +1751,7 @@ njs_regexp_prototype_symbol_split(njs_vm_t *vm, njs_value_t *args,
                 return NJS_ERROR;
             }
 
-            (void) njs_string_prop(&sv, retval);
-
-            ret = njs_array_string_add(vm, array, sv.start, sv.size,
-                                       sv.length);
+            ret = njs_array_add(vm, array, retval);
             if (njs_slow_path(ret != NJS_OK)) {
                 return ret;
             }
@@ -1889,13 +1809,14 @@ done:
 }
 
 
-static const njs_object_prop_t  njs_regexp_constructor_properties[] =
+static const njs_object_prop_init_t  njs_regexp_constructor_properties[] =
 {
     NJS_DECLARE_PROP_LENGTH(2),
 
     NJS_DECLARE_PROP_NAME("RegExp"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 };
 
 
@@ -1905,56 +1826,48 @@ const njs_object_init_t  njs_regexp_constructor_init = {
 };
 
 
-static const njs_object_prop_t  njs_regexp_prototype_properties[] =
+static const njs_object_prop_init_t  njs_regexp_prototype_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_GETTER("flags", njs_regexp_prototype_flags, 0),
+    NJS_DECLARE_PROP_GETTER(STRING_flags, njs_regexp_prototype_flags, 0),
 
-    NJS_DECLARE_PROP_GETTER("global", njs_regexp_prototype_flag,
+    NJS_DECLARE_PROP_GETTER(STRING_global, njs_regexp_prototype_flag,
                             NJS_REGEX_GLOBAL),
 
-    NJS_DECLARE_PROP_GETTER("ignoreCase", njs_regexp_prototype_flag,
+    NJS_DECLARE_PROP_GETTER(STRING_ignoreCase, njs_regexp_prototype_flag,
                             NJS_REGEX_IGNORE_CASE),
 
-    NJS_DECLARE_PROP_GETTER("multiline", njs_regexp_prototype_flag,
+    NJS_DECLARE_PROP_GETTER(STRING_multiline, njs_regexp_prototype_flag,
                             NJS_REGEX_MULTILINE),
 
-    NJS_DECLARE_PROP_GETTER("source", njs_regexp_prototype_source, 0),
+    NJS_DECLARE_PROP_GETTER(STRING_source, njs_regexp_prototype_source, 0),
 
-    NJS_DECLARE_PROP_GETTER("sticky", njs_regexp_prototype_flag,
+    NJS_DECLARE_PROP_GETTER(STRING_sticky, njs_regexp_prototype_flag,
                             NJS_REGEX_STICKY),
 
-    NJS_DECLARE_PROP_NATIVE("toString", njs_regexp_prototype_to_string, 0, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_toString,
+                            njs_regexp_prototype_to_string, 0, 0),
 
-    NJS_DECLARE_PROP_NATIVE("test", njs_regexp_prototype_test, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_test, njs_regexp_prototype_test, 1, 0),
 
-    NJS_DECLARE_PROP_NATIVE("exec", njs_regexp_prototype_exec, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_exec, njs_regexp_prototype_exec, 1, 0),
 
-    {
-        .type = NJS_PROPERTY,
-        .name = njs_wellknown_symbol(NJS_SYMBOL_REPLACE),
-        .u.value = njs_native_function(njs_regexp_prototype_symbol_replace, 2),
-        .writable = 1,
-        .configurable = 1,
-    },
+    NJS_DECLARE_PROP_NATIVE(SYMBOL_replace,
+                            njs_regexp_prototype_symbol_replace, 2, 0),
 
-    {
-        .type = NJS_PROPERTY,
-        .name = njs_wellknown_symbol(NJS_SYMBOL_SPLIT),
-        .u.value = njs_native_function(njs_regexp_prototype_symbol_split, 2),
-        .writable = 1,
-        .configurable = 1,
-    },
+    NJS_DECLARE_PROP_NATIVE(SYMBOL_split,
+                            njs_regexp_prototype_symbol_split, 2, 0),
 };
 
 
-const njs_object_prop_t  njs_regexp_instance_properties[] =
+const njs_object_prop_init_t  njs_regexp_instance_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("lastIndex", njs_regexp_prototype_last_index,
-                             0, 0, NJS_OBJECT_PROP_VALUE_W),
+    NJS_DECLARE_PROP_HANDLER(STRING_lastIndex,
+                             njs_regexp_prototype_last_index, 0,
+                             NJS_OBJECT_PROP_VALUE_W),
 };
 
 
