@@ -25,22 +25,6 @@ njs_scope_temp_index(njs_parser_scope_t *scope)
 }
 
 
-njs_value_t *
-njs_scope_create_index_value(njs_vm_t *vm, njs_index_t index)
-{
-    njs_value_t  *value;
-
-    value = njs_mp_alloc(vm->mem_pool, sizeof(njs_value_t));
-    if (njs_slow_path(value == NULL)) {
-        return NULL;
-    }
-
-    njs_scope_value_set(vm, index, value);
-
-    return value;
-}
-
-
 njs_value_t **
 njs_scope_make(njs_vm_t *vm, uint32_t count)
 {
@@ -124,10 +108,11 @@ njs_scope_values_hash_test(njs_lvlhsh_query_t *lhq, void *data)
     njs_str_t    string;
     njs_value_t  *value;
 
-    value = data;
+    value = *(njs_value_t **) data;
 
     if (njs_is_string(value)) {
-        njs_string_get(value, &string);
+        /* parser strings are always initialized. */
+        njs_string_get_unsafe(value, &string);
 
     } else {
         string.start = (u_char *) value;
@@ -167,24 +152,24 @@ njs_scope_value_index(njs_vm_t *vm, const njs_value_t *src, njs_uint_t runtime,
     uint32_t            value_size, size, length;
     njs_int_t           ret;
     njs_str_t           str;
-    njs_bool_t          long_string;
+    njs_bool_t          is_string;
     njs_value_t         *value;
     njs_string_t        *string;
     njs_lvlhsh_t        *values_hash;
+    njs_object_prop_t   *pr;
     njs_lvlhsh_query_t  lhq;
 
-    long_string = 0;
+    is_string = 0;
     value_size = sizeof(njs_value_t);
 
     if (njs_is_string(src)) {
-        njs_string_get(src, &str);
+        /* parser strings are always initialized. */
+        njs_string_get_unsafe(src, &str);
 
         size = (uint32_t) str.length;
         start = str.start;
 
-        if (src->short_string.size == NJS_STRING_LONG) {
-            long_string = 1;
-        }
+        is_string = 1;
 
     } else {
         size = value_size;
@@ -197,18 +182,18 @@ njs_scope_value_index(njs_vm_t *vm, const njs_value_t *src, njs_uint_t runtime,
     lhq.proto = &njs_values_hash_proto;
 
     if (njs_lvlhsh_find(&vm->shared->values_hash, &lhq) == NJS_OK) {
-        value = lhq.value;
+        value = ((njs_object_prop_t *) lhq.value)->u.val;
 
         *index = (njs_index_t *) ((u_char *) value + sizeof(njs_value_t));
 
     } else if (runtime && njs_lvlhsh_find(&vm->values_hash, &lhq) == NJS_OK) {
-        value = lhq.value;
+        value = ((njs_object_prop_t *) lhq.value)->u.val;
 
         *index = (njs_index_t *) ((u_char *) value + sizeof(njs_value_t));
 
     } else {
-        if (long_string) {
-            length = src->long_string.data->length;
+        if (is_string) {
+            length = src->string.data->length;
 
             if (size != length && length > NJS_STRING_MAP_STRIDE) {
                 size = njs_string_map_offset(size)
@@ -227,14 +212,15 @@ njs_scope_value_index(njs_vm_t *vm, const njs_value_t *src, njs_uint_t runtime,
 
         *value = *src;
 
-        if (long_string) {
+        if (is_string) {
             string = (njs_string_t *) ((u_char *) value + sizeof(njs_value_t)
                                        + sizeof(njs_index_t));
 
-            value->long_string.data = string;
+            value->string.data = string;
 
             string->start = (u_char *) string + sizeof(njs_string_t);
-            string->length = src->long_string.data->length;
+            string->length = src->string.data->length;
+            string->size = src->string.data->size;
 
             memcpy(string->start, start, size);
         }
@@ -243,7 +229,6 @@ njs_scope_value_index(njs_vm_t *vm, const njs_value_t *src, njs_uint_t runtime,
         **index = NJS_INDEX_ERROR;
 
         lhq.replace = 0;
-        lhq.value = value;
         lhq.pool = vm->mem_pool;
 
         values_hash = runtime ? &vm->values_hash : &vm->shared->values_hash;
@@ -252,6 +237,9 @@ njs_scope_value_index(njs_vm_t *vm, const njs_value_t *src, njs_uint_t runtime,
         if (njs_slow_path(ret != NJS_OK)) {
             return NULL;
         }
+
+        pr = lhq.value;
+        pr->u.val = value;
     }
 
     if (start != (u_char *) src) {
