@@ -324,13 +324,14 @@ static njs_int_t
 njs_typed_array_from(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     njs_index_t unused, njs_value_t *retval)
 {
-    double             num;
-    int64_t            length, i;
-    njs_int_t          ret;
-    njs_value_t        *this, *source, *mapfn;
-    njs_value_t        arguments[3], value;
-    njs_function_t     *function;
-    njs_typed_array_t  *array;
+    double              num;
+    int64_t             length, i;
+    njs_int_t           ret;
+    njs_value_t         *this, *source, *mapfn;
+    njs_value_t         arguments[3], value;
+    njs_function_t      *function;
+    njs_typed_array_t   *array;
+    njs_array_buffer_t  *buffer;
 
     this = njs_argument(args, 0);
 
@@ -371,6 +372,7 @@ njs_typed_array_from(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     }
 
     array = njs_typed_array(retval);
+    buffer = njs_typed_array_buffer(array);
     arguments[0] = *njs_arg(args, nargs, 3);
 
     for (i = 0; i < length; i++) {
@@ -393,7 +395,9 @@ njs_typed_array_from(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
             return NJS_ERROR;
         }
 
-        njs_typed_array_prop_set(vm, array, i, num);
+        if (!njs_is_detached_buffer(buffer)) {
+            njs_typed_array_prop_set(vm, array, i, num);
+        }
     }
 
     njs_set_typed_array(retval, array);
@@ -433,37 +437,22 @@ njs_typed_array_writable(njs_vm_t *vm, njs_typed_array_t *array)
 }
 
 
-static const njs_value_t  njs_typed_array_uint8_tag = njs_string("Uint8Array");
-static const njs_value_t  njs_typed_array_uint8_clamped_tag =
-                                        njs_long_string("Uint8ClampedArray");
-static const njs_value_t  njs_typed_array_int8_tag = njs_string("Int8Array");
-static const njs_value_t  njs_typed_array_uint16_tag =
-                                                    njs_string("Uint16Array");
-static const njs_value_t  njs_typed_array_int16_tag = njs_string("Int16Array");
-static const njs_value_t  njs_typed_array_uint32_tag =
-                                                    njs_string("Uint32Array");
-static const njs_value_t  njs_typed_array_int32_tag = njs_string("Int32Array");
-static const njs_value_t  njs_typed_array_float32_tag =
-                                                    njs_string("Float32Array");
-static const njs_value_t  njs_typed_array_float64_tag =
-                                                    njs_string("Float64Array");
-
 static njs_int_t
 njs_typed_array_get_string_tag(njs_vm_t *vm, njs_value_t *args,
     njs_uint_t nargs, njs_index_t unused, njs_value_t *retval)
 {
     njs_value_t  *this;
 
-    static const njs_value_t  *tags[NJS_OBJ_TYPE_TYPED_ARRAY_SIZE] = {
-        &njs_typed_array_uint8_tag,
-        &njs_typed_array_uint8_clamped_tag,
-        &njs_typed_array_int8_tag,
-        &njs_typed_array_uint16_tag,
-        &njs_typed_array_int16_tag,
-        &njs_typed_array_uint32_tag,
-        &njs_typed_array_int32_tag,
-        &njs_typed_array_float32_tag,
-        &njs_typed_array_float64_tag,
+    static const uint32_t  tags[NJS_OBJ_TYPE_TYPED_ARRAY_SIZE] = {
+        NJS_ATOM_STRING_Uint8Array,
+        NJS_ATOM_STRING_Uint8ClampedArray,
+        NJS_ATOM_STRING_Int8Array,
+        NJS_ATOM_STRING_Uint16Array,
+        NJS_ATOM_STRING_Int16Array,
+        NJS_ATOM_STRING_Uint32Array,
+        NJS_ATOM_STRING_Int32Array,
+        NJS_ATOM_STRING_Float32Array,
+        NJS_ATOM_STRING_Float64Array,
     };
 
     this = njs_argument(args, 0);
@@ -473,8 +462,8 @@ njs_typed_array_get_string_tag(njs_vm_t *vm, njs_value_t *args,
         return NJS_OK;
     }
 
-    njs_value_assign(retval,
-                     tags[njs_typed_array_index(njs_typed_array(this)->type)]);
+    njs_atom_to_value(vm, retval,
+                      tags[njs_typed_array_index(njs_typed_array(this)->type)]);
 
     return NJS_OK;
 }
@@ -666,6 +655,11 @@ njs_typed_array_set_value(njs_vm_t *vm, njs_typed_array_t *array,
         return ret;
     }
 
+    buffer = njs_typed_array_buffer(array);
+    if (njs_slow_path(njs_is_detached_buffer(buffer))) {
+        return NJS_OK;
+    }
+
     buffer = njs_typed_array_writable(vm, array);
     if (njs_slow_path(buffer == NULL)) {
         return NJS_ERROR;
@@ -923,6 +917,20 @@ njs_typed_array_prototype_fill(njs_vm_t *vm, njs_value_t *args,
 }
 
 
+static void
+njs_slice_memcpy(uint8_t *dst, const uint8_t *src, size_t len)
+{
+    if (dst + len <= src || dst >= src + len) {
+        /* no overlap: can use memcpy */
+        memcpy(dst, src, len);
+
+    } else {
+        while (len-- != 0)
+            *dst++ = *src++;
+    }
+}
+
+
 njs_int_t
 njs_typed_array_prototype_slice(njs_vm_t *vm, njs_value_t *args,
     njs_uint_t nargs, njs_index_t copy, njs_value_t *retval)
@@ -1001,7 +1009,7 @@ njs_typed_array_prototype_slice(njs_vm_t *vm, njs_value_t *args,
             start = start * element_size;
             count = count * element_size;
 
-            memcpy(&new_buffer->u.u8[0], &buffer->u.u8[start], count);
+            njs_slice_memcpy(&new_buffer->u.u8[0], &buffer->u.u8[start], count);
 
         } else {
             for (i = 0; i < count; i++) {
@@ -2062,11 +2070,15 @@ njs_typed_array_to_chain(njs_vm_t *vm, njs_chb_t *chain,
     uint32_t           i;
     njs_string_prop_t  separator;
 
-    if (sep == NULL) {
-        sep = njs_value_arg(&njs_string_comma);
+    if (sep != NULL && njs_is_string(sep)) {
+        (void) njs_string_prop(vm, &separator, sep);
+
+    } else {
+        separator.start = (u_char *) ",";
+        separator.size = 1;
+        separator.length = 1;
     }
 
-    (void) njs_string_prop(&separator, sep);
 
     length = njs_typed_array_length(array);
 
@@ -2110,10 +2122,7 @@ njs_typed_array_prototype_join(njs_vm_t *vm, njs_value_t *args,
     separator = njs_arg(args, nargs, 1);
 
     if (njs_slow_path(!njs_is_string(separator))) {
-        if (njs_is_undefined(separator)) {
-            separator = njs_value_arg(&njs_string_comma);
-
-        } else {
+        if (njs_is_defined(separator)) {
             ret = njs_value_to_string(vm, separator, separator);
             if (njs_slow_path(ret != NJS_OK)) {
                 return ret;
@@ -2122,7 +2131,7 @@ njs_typed_array_prototype_join(njs_vm_t *vm, njs_value_t *args,
     }
 
     if (length == 0) {
-        njs_value_assign(retval, &njs_string_empty);
+        njs_set_empty_string(vm, retval);
         return NJS_OK;
     }
 
@@ -2131,7 +2140,7 @@ njs_typed_array_prototype_join(njs_vm_t *vm, njs_value_t *args,
         return NJS_ERROR;
     }
 
-    NJS_CHB_MP_INIT(&chain, vm);
+    NJS_CHB_MP_INIT(&chain, njs_vm_memory_pool(vm));
 
     njs_typed_array_to_chain(vm, &chain, array, separator);
 
@@ -2176,26 +2185,20 @@ njs_typed_array_constructor_intrinsic(njs_vm_t *vm, njs_value_t *args,
 }
 
 
-static const njs_object_prop_t  njs_typed_array_constructor_props[] =
+static const njs_object_prop_init_t  njs_typed_array_constructor_props[] =
 {
     NJS_DECLARE_PROP_LENGTH(0),
 
     NJS_DECLARE_PROP_NAME("TypedArray"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 
-    {
-        .type = NJS_ACCESSOR,
-        .name = njs_wellknown_symbol(NJS_SYMBOL_SPECIES),
-        .u.accessor = njs_getter(njs_typed_array_get_this, 0),
-        .writable = NJS_ATTRIBUTE_UNSET,
-        .configurable = 1,
-        .enumerable = 0,
-    },
+    NJS_DECLARE_PROP_GETTER(SYMBOL_species, njs_typed_array_get_this, 0),
 
-    NJS_DECLARE_PROP_NATIVE("of", njs_typed_array_of, 0, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_of, njs_typed_array_of, 0, 0),
 
-    NJS_DECLARE_PROP_NATIVE("from", njs_typed_array_from, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_from, njs_typed_array_from, 1, 0),
 };
 
 
@@ -2205,118 +2208,120 @@ static const njs_object_init_t  njs_typed_array_constructor_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_prototype_properties[] =
+static const njs_object_prop_init_t  njs_typed_array_prototype_properties[] =
 {
-    {
-        .type = NJS_ACCESSOR,
-        .name = njs_wellknown_symbol(NJS_SYMBOL_TO_STRING_TAG),
-        .u.accessor = njs_getter(njs_typed_array_get_string_tag, 0),
-        .writable = NJS_ATTRIBUTE_UNSET,
-        .configurable = 1,
-        .enumerable = 0,
-    },
+    NJS_DECLARE_PROP_GETTER(SYMBOL_toStringTag,
+                            njs_typed_array_get_string_tag, 0),
 
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_GETTER("buffer", njs_typed_array_prototype_buffer, 0),
+    NJS_DECLARE_PROP_GETTER(STRING_buffer, njs_typed_array_prototype_buffer,
+                            0),
 
-    NJS_DECLARE_PROP_GETTER("byteLength",
+    NJS_DECLARE_PROP_GETTER(STRING_byteLength,
                             njs_typed_array_prototype_byte_length, 0),
 
-    NJS_DECLARE_PROP_GETTER("byteOffset",
+    NJS_DECLARE_PROP_GETTER(STRING_byteOffset,
                             njs_typed_array_prototype_byte_offset, 0),
 
-    NJS_DECLARE_PROP_GETTER("length", njs_typed_array_prototype_length, 0),
+    NJS_DECLARE_PROP_GETTER(STRING_length, njs_typed_array_prototype_length,
+                            0),
 
-    NJS_DECLARE_PROP_NATIVE("copyWithin",
+    NJS_DECLARE_PROP_NATIVE(STRING_copyWithin,
                             njs_typed_array_prototype_copy_within, 2, 0),
 
-    NJS_DECLARE_PROP_NATIVE("entries",
+    NJS_DECLARE_PROP_NATIVE(STRING_entries,
                             njs_typed_array_prototype_iterator_obj, 0,
                             NJS_ENUM_BOTH),
 
-    NJS_DECLARE_PROP_NATIVE("every",
+    NJS_DECLARE_PROP_NATIVE(STRING_every,
                             njs_typed_array_prototype_iterator, 1,
                             NJS_ARRAY_EVERY),
 
-    NJS_DECLARE_PROP_NATIVE("filter",
+    NJS_DECLARE_PROP_NATIVE(STRING_filter,
                             njs_typed_array_prototype_iterator, 1,
                             NJS_ARRAY_FILTER),
 
-    NJS_DECLARE_PROP_NATIVE("find",
+    NJS_DECLARE_PROP_NATIVE(STRING_find,
                             njs_typed_array_prototype_iterator, 1,
                             NJS_ARRAY_FIND),
 
-    NJS_DECLARE_PROP_NATIVE("findIndex",
+    NJS_DECLARE_PROP_NATIVE(STRING_findIndex,
                             njs_typed_array_prototype_iterator, 1,
                             NJS_ARRAY_FIND_INDEX),
 
-    NJS_DECLARE_PROP_NATIVE("forEach",
+    NJS_DECLARE_PROP_NATIVE(STRING_forEach,
                             njs_typed_array_prototype_iterator, 1,
                             NJS_ARRAY_FOR_EACH),
 
-    NJS_DECLARE_PROP_NATIVE("includes",
-                             njs_typed_array_prototype_index_of, 1, 1),
+    NJS_DECLARE_PROP_NATIVE(STRING_includes,
+                            njs_typed_array_prototype_index_of, 1, 1),
 
-    NJS_DECLARE_PROP_NATIVE("indexOf",
-                             njs_typed_array_prototype_index_of, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_indexOf,
+                            njs_typed_array_prototype_index_of, 1, 0),
 
-    NJS_DECLARE_PROP_NATIVE("join", njs_typed_array_prototype_join, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_join,
+                            njs_typed_array_prototype_join, 1, 0),
 
-    NJS_DECLARE_PROP_NATIVE("fill", njs_typed_array_prototype_fill, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_fill,
+                            njs_typed_array_prototype_fill, 1, 0),
 
-    NJS_DECLARE_PROP_NATIVE("keys", njs_typed_array_prototype_iterator_obj, 0,
+    NJS_DECLARE_PROP_NATIVE(STRING_keys,
+                            njs_typed_array_prototype_iterator_obj, 0,
                             NJS_ENUM_KEYS),
 
-    NJS_DECLARE_PROP_NATIVE("lastIndexOf",
+    NJS_DECLARE_PROP_NATIVE(STRING_lastIndexOf,
                             njs_typed_array_prototype_index_of, 1, 2),
 
-    NJS_DECLARE_PROP_NATIVE("map",
+    NJS_DECLARE_PROP_NATIVE(STRING_map,
                             njs_typed_array_prototype_iterator, 1,
                             NJS_ARRAY_MAP),
 
-    NJS_DECLARE_PROP_NATIVE("reduce", njs_typed_array_prototype_reduce, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_reduce,
+                            njs_typed_array_prototype_reduce, 1, 0),
 
-    NJS_DECLARE_PROP_NATIVE("reduceRight", njs_typed_array_prototype_reduce,
-                            1, 1),
+    NJS_DECLARE_PROP_NATIVE(STRING_reduceRight,
+                            njs_typed_array_prototype_reduce, 1, 1),
 
-    NJS_DECLARE_PROP_NATIVE("reverse", njs_typed_array_prototype_reverse, 0, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_reverse,
+                            njs_typed_array_prototype_reverse, 0, 0),
 
-    NJS_DECLARE_PROP_NATIVE("set", njs_typed_array_prototype_set, 2, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_set,
+                            njs_typed_array_prototype_set, 2, 0),
 
-    NJS_DECLARE_PROP_NATIVE("slice", njs_typed_array_prototype_slice, 2, 1),
+    NJS_DECLARE_PROP_NATIVE(STRING_slice,
+                            njs_typed_array_prototype_slice, 2, 1),
 
-    NJS_DECLARE_PROP_NATIVE("some",
+    NJS_DECLARE_PROP_NATIVE(STRING_some,
                             njs_typed_array_prototype_iterator, 1,
                             NJS_ARRAY_SOME),
 
-    NJS_DECLARE_PROP_NATIVE("sort", njs_typed_array_prototype_sort, 1, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_sort,
+                            njs_typed_array_prototype_sort, 1, 0),
 
-    NJS_DECLARE_PROP_NATIVE("subarray", njs_typed_array_prototype_slice, 2, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_subarray,
+                            njs_typed_array_prototype_slice, 2, 0),
 
-    NJS_DECLARE_PROP_NATIVE("toReversed", njs_typed_array_prototype_reverse, 0,
-                            1),
+    NJS_DECLARE_PROP_NATIVE(STRING_toReversed,
+                            njs_typed_array_prototype_reverse, 0, 1),
 
-    NJS_DECLARE_PROP_NATIVE("toSorted", njs_typed_array_prototype_sort, 1, 1),
+    NJS_DECLARE_PROP_NATIVE(STRING_toSorted,
+                            njs_typed_array_prototype_sort, 1, 1),
 
-    NJS_DECLARE_PROP_NATIVE("toString", njs_array_prototype_to_string, 0, 0),
+    NJS_DECLARE_PROP_NATIVE(STRING_toString, njs_array_prototype_to_string,
+                            0, 0),
 
-    NJS_DECLARE_PROP_NATIVE("values", njs_typed_array_prototype_iterator_obj,
-                            0, NJS_ENUM_VALUES),
+    NJS_DECLARE_PROP_NATIVE(STRING_values,
+                            njs_typed_array_prototype_iterator_obj, 0,
+                            NJS_ENUM_VALUES),
 
-    {
-        .type = NJS_PROPERTY,
-        .name = njs_wellknown_symbol(NJS_SYMBOL_ITERATOR),
-        .u.value = njs_native_function2(njs_typed_array_prototype_iterator_obj,
-                                        0, NJS_ENUM_VALUES),
-        .writable = 1,
-        .configurable = 1,
-    },
+    NJS_DECLARE_PROP_NATIVE(SYMBOL_iterator,
+                            njs_typed_array_prototype_iterator_obj, 0,
+                            NJS_ENUM_VALUES),
 };
 
-
 static const njs_object_init_t  njs_typed_array_prototype_init = {
     njs_typed_array_prototype_properties,
     njs_nitems(njs_typed_array_prototype_properties),
@@ -2413,13 +2418,14 @@ memory_error:
 }
 
 
-static const njs_object_prop_t  njs_data_view_constructor_props[] =
+static const njs_object_prop_init_t  njs_data_view_constructor_props[] =
 {
     NJS_DECLARE_PROP_LENGTH(1),
 
     NJS_DECLARE_PROP_NAME("DataView"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 };
 
 
@@ -2658,74 +2664,71 @@ njs_data_view_prototype_set(njs_vm_t *vm, njs_value_t *args,
 }
 
 
-static const njs_object_prop_t  njs_data_view_prototype_properties[] =
+static const njs_object_prop_init_t  njs_data_view_prototype_properties[] =
 {
-    {
-        .type = NJS_PROPERTY,
-        .name = njs_wellknown_symbol(NJS_SYMBOL_TO_STRING_TAG),
-        .u.value = njs_string("DataView"),
-        .configurable = 1,
-    },
+    NJS_DECLARE_PROP_VALUE(SYMBOL_toStringTag, njs_ascii_strval("DataView"),
+                           NJS_OBJECT_PROP_VALUE_C),
 
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_GETTER("buffer", njs_typed_array_prototype_buffer, 0),
+    NJS_DECLARE_PROP_GETTER(STRING_buffer, njs_typed_array_prototype_buffer,
+                            0),
 
-    NJS_DECLARE_PROP_GETTER("byteLength",
+    NJS_DECLARE_PROP_GETTER(STRING_byteLength,
                             njs_typed_array_prototype_byte_length, 0),
 
-    NJS_DECLARE_PROP_GETTER("byteOffset",
+    NJS_DECLARE_PROP_GETTER(STRING_byteOffset,
                             njs_typed_array_prototype_byte_offset, 0),
 
-    NJS_DECLARE_PROP_NATIVE("getUint8", njs_data_view_prototype_get, 1,
+    NJS_DECLARE_PROP_NATIVE(STRING_getUint8, njs_data_view_prototype_get, 1,
                             NJS_OBJ_TYPE_UINT8_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("getInt8", njs_data_view_prototype_get, 1,
+    NJS_DECLARE_PROP_NATIVE(STRING_getInt8, njs_data_view_prototype_get, 1,
                             NJS_OBJ_TYPE_INT8_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("getUint16", njs_data_view_prototype_get, 1,
-                            NJS_OBJ_TYPE_UINT16_ARRAY),
+    NJS_DECLARE_PROP_NATIVE(STRING_getUint16, njs_data_view_prototype_get,
+                            1, NJS_OBJ_TYPE_UINT16_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("getInt16", njs_data_view_prototype_get, 1,
+    NJS_DECLARE_PROP_NATIVE(STRING_getInt16, njs_data_view_prototype_get, 1,
                             NJS_OBJ_TYPE_INT16_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("getUint32", njs_data_view_prototype_get, 1,
-                            NJS_OBJ_TYPE_UINT32_ARRAY),
+    NJS_DECLARE_PROP_NATIVE(STRING_getUint32, njs_data_view_prototype_get,
+                            1, NJS_OBJ_TYPE_UINT32_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("getInt32", njs_data_view_prototype_get, 1,
+    NJS_DECLARE_PROP_NATIVE(STRING_getInt32, njs_data_view_prototype_get, 1,
                             NJS_OBJ_TYPE_INT32_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("getFloat32", njs_data_view_prototype_get, 1,
-                            NJS_OBJ_TYPE_FLOAT32_ARRAY),
+    NJS_DECLARE_PROP_NATIVE(STRING_getFloat32, njs_data_view_prototype_get,
+                            1, NJS_OBJ_TYPE_FLOAT32_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("getFloat64", njs_data_view_prototype_get, 1,
-                            NJS_OBJ_TYPE_FLOAT64_ARRAY),
+    NJS_DECLARE_PROP_NATIVE(STRING_getFloat64, njs_data_view_prototype_get,
+                            1, NJS_OBJ_TYPE_FLOAT64_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("setUint8", njs_data_view_prototype_set, 2,
+    NJS_DECLARE_PROP_NATIVE(STRING_setUint8, njs_data_view_prototype_set, 2,
                             NJS_OBJ_TYPE_UINT8_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("setInt8", njs_data_view_prototype_set, 2,
+    NJS_DECLARE_PROP_NATIVE(STRING_setInt8, njs_data_view_prototype_set, 2,
                             NJS_OBJ_TYPE_INT8_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("setUint16", njs_data_view_prototype_set, 2,
-                            NJS_OBJ_TYPE_UINT16_ARRAY),
+    NJS_DECLARE_PROP_NATIVE(STRING_setUint16, njs_data_view_prototype_set,
+                            2, NJS_OBJ_TYPE_UINT16_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("setInt16", njs_data_view_prototype_set, 2,
+    NJS_DECLARE_PROP_NATIVE(STRING_setInt16, njs_data_view_prototype_set, 2,
                             NJS_OBJ_TYPE_INT16_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("setUint32", njs_data_view_prototype_set, 2,
-                            NJS_OBJ_TYPE_UINT32_ARRAY),
+    NJS_DECLARE_PROP_NATIVE(STRING_setUint32, njs_data_view_prototype_set,
+                            2, NJS_OBJ_TYPE_UINT32_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("setInt32", njs_data_view_prototype_set, 2,
+    NJS_DECLARE_PROP_NATIVE(STRING_setInt32, njs_data_view_prototype_set, 2,
                             NJS_OBJ_TYPE_INT32_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("setFloat32", njs_data_view_prototype_set, 2,
-                            NJS_OBJ_TYPE_FLOAT32_ARRAY),
+    NJS_DECLARE_PROP_NATIVE(STRING_setFloat32, njs_data_view_prototype_set,
+                            2, NJS_OBJ_TYPE_FLOAT32_ARRAY),
 
-    NJS_DECLARE_PROP_NATIVE("setFloat64", njs_data_view_prototype_set, 2,
-                            NJS_OBJ_TYPE_FLOAT64_ARRAY),
+    NJS_DECLARE_PROP_NATIVE(STRING_setFloat64, njs_data_view_prototype_set,
+                            2, NJS_OBJ_TYPE_FLOAT64_ARRAY),
 };
 
 
@@ -2743,16 +2746,17 @@ const njs_object_type_init_t  njs_data_view_type_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_u8_constructor_props[] =
+static const njs_object_prop_init_t  njs_typed_array_u8_constructor_props[] =
 {
     NJS_DECLARE_PROP_LENGTH(3),
 
     NJS_DECLARE_PROP_NAME("Uint8Array"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 1),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 1), 0),
 };
 
 
@@ -2762,14 +2766,14 @@ static const njs_object_init_t  njs_typed_array_u8_constructor_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_u8_prototype_properties[] =
+static const njs_object_prop_init_t  njs_typed_array_u8_prototype_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 1),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 1), 0),
 };
 
 
@@ -2788,21 +2792,18 @@ const njs_object_type_init_t  njs_typed_array_u8_type_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_u8c_constructor_props[] =
+static const njs_object_prop_init_t  njs_typed_array_u8c_constructor_props[] =
 {
-    {
-        .type = NJS_PROPERTY,
-        .name = njs_string("name"),
-        .u.value = njs_long_string("Uint8ClampedArray"),
-        .configurable = 1,
-    },
+    NJS_DECLARE_PROP_VALUE(STRING_name, njs_ascii_strval("Uint8ClampedArray"),
+                           NJS_OBJECT_PROP_VALUE_C),
 
     NJS_DECLARE_PROP_LENGTH(3),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 1),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 1), 0),
 };
 
 
@@ -2812,14 +2813,14 @@ static const njs_object_init_t  njs_typed_array_u8c_constructor_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_u8c_prototype_properties[] =
+static const njs_object_prop_init_t  njs_typed_array_u8c_prototype_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 1),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 1), 0),
 };
 
 
@@ -2838,16 +2839,17 @@ const njs_object_type_init_t  njs_typed_array_u8clamped_type_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_i8_constructor_props[] =
+static const njs_object_prop_init_t  njs_typed_array_i8_constructor_props[] =
 {
     NJS_DECLARE_PROP_LENGTH(3),
 
     NJS_DECLARE_PROP_NAME("Int8Array"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 1),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 1), 0),
 };
 
 
@@ -2857,14 +2859,14 @@ static const njs_object_init_t  njs_typed_array_i8_constructor_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_i8_prototype_properties[] =
+static const njs_object_prop_init_t  njs_typed_array_i8_prototype_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 1),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 1), 0),
 };
 
 
@@ -2883,16 +2885,17 @@ const njs_object_type_init_t  njs_typed_array_i8_type_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_u16_constructor_props[] =
+static const njs_object_prop_init_t  njs_typed_array_u16_constructor_props[] =
 {
     NJS_DECLARE_PROP_LENGTH(3),
 
     NJS_DECLARE_PROP_NAME("Uint16Array"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 2),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 2), 0),
 };
 
 
@@ -2902,14 +2905,14 @@ static const njs_object_init_t  njs_typed_array_u16_constructor_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_u16_prototype_properties[] =
+static const njs_object_prop_init_t  njs_typed_array_u16_prototype_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 2),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 2), 0),
 };
 
 
@@ -2928,16 +2931,17 @@ const njs_object_type_init_t  njs_typed_array_u16_type_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_i16_constructor_props[] =
+static const njs_object_prop_init_t  njs_typed_array_i16_constructor_props[] =
 {
     NJS_DECLARE_PROP_LENGTH(3),
 
     NJS_DECLARE_PROP_NAME("Int16Array"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 2),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 2), 0),
 };
 
 
@@ -2947,14 +2951,14 @@ static const njs_object_init_t  njs_typed_array_i16_constructor_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_i16_prototype_properties[] =
+static const njs_object_prop_init_t  njs_typed_array_i16_prototype_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 2),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 2), 0),
 };
 
 
@@ -2973,16 +2977,17 @@ const njs_object_type_init_t  njs_typed_array_i16_type_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_u32_constructor_props[] =
+static const njs_object_prop_init_t  njs_typed_array_u32_constructor_props[] =
 {
     NJS_DECLARE_PROP_LENGTH(3),
 
     NJS_DECLARE_PROP_NAME("Uint32Array"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 4),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 4), 0),
 };
 
 
@@ -2992,14 +2997,14 @@ static const njs_object_init_t  njs_typed_array_u32_constructor_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_u32_prototype_properties[] =
+static const njs_object_prop_init_t  njs_typed_array_u32_prototype_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 4),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 4), 0),
 };
 
 
@@ -3018,16 +3023,17 @@ const njs_object_type_init_t  njs_typed_array_u32_type_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_i32_constructor_props[] =
+static const njs_object_prop_init_t  njs_typed_array_i32_constructor_props[] =
 {
     NJS_DECLARE_PROP_LENGTH(3),
 
     NJS_DECLARE_PROP_NAME("Int32Array"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 4),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 4), 0),
 };
 
 
@@ -3037,14 +3043,14 @@ static const njs_object_init_t  njs_typed_array_i32_constructor_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_i32_prototype_properties[] =
+static const njs_object_prop_init_t  njs_typed_array_i32_prototype_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 4),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 4), 0),
 };
 
 
@@ -3063,16 +3069,17 @@ const njs_object_type_init_t  njs_typed_array_i32_type_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_f32_constructor_props[] =
+static const njs_object_prop_init_t  njs_typed_array_f32_constructor_props[] =
 {
     NJS_DECLARE_PROP_LENGTH(3),
 
     NJS_DECLARE_PROP_NAME("Float32Array"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype, njs_object_prototype_create,
+                             0, 0),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 4),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 4), 0),
 };
 
 
@@ -3082,14 +3089,14 @@ static const njs_object_init_t  njs_typed_array_f32_constructor_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_f32_prototype_properties[] =
+static const njs_object_prop_init_t  njs_typed_array_f32_prototype_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 4),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 4), 0),
 };
 
 
@@ -3108,16 +3115,17 @@ const njs_object_type_init_t  njs_typed_array_f32_type_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_f64_constructor_props[] =
+static const njs_object_prop_init_t  njs_typed_array_f64_constructor_props[] =
 {
     NJS_DECLARE_PROP_LENGTH(3),
 
     NJS_DECLARE_PROP_NAME("Float64Array"),
 
-    NJS_DECLARE_PROP_HANDLER("prototype", njs_object_prototype_create, 0, 0, 0),
+    NJS_DECLARE_PROP_HANDLER(STRING_prototype,
+                             njs_object_prototype_create, 0, 0),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 8),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 8), 0),
 };
 
 
@@ -3127,14 +3135,14 @@ static const njs_object_init_t  njs_typed_array_f64_constructor_init = {
 };
 
 
-static const njs_object_prop_t  njs_typed_array_f64_prototype_properties[] =
+static const njs_object_prop_init_t  njs_typed_array_f64_prototype_properties[] =
 {
-    NJS_DECLARE_PROP_HANDLER("constructor",
-                             njs_object_prototype_create_constructor,
-                             0, 0, NJS_OBJECT_PROP_VALUE_CW),
+    NJS_DECLARE_PROP_HANDLER(STRING_constructor,
+                             njs_object_prototype_create_constructor, 0,
+                             NJS_OBJECT_PROP_VALUE_CW),
 
-    NJS_DECLARE_PROP_LVALUE("BYTES_PER_ELEMENT", njs_value(NJS_NUMBER, 1, 8),
-                            0),
+    NJS_DECLARE_PROP_VALUE(STRING_BYTES_PER_ELEMENT,
+                           njs_value(NJS_NUMBER, 1, 8), 0),
 };
 
 
