@@ -8,11 +8,12 @@
 #include <njs_main.h>
 
 
-static njs_int_t njs_object_property_query(njs_vm_t *vm,
-    njs_property_query_t *pq, njs_object_t *object,
-    const njs_value_t *key);
+njs_inline njs_int_t
+njs_object_property_query(njs_vm_t *vm, njs_property_query_t *pq,
+    njs_object_t *object, uint32_t atom_id);
 static njs_int_t njs_array_property_query(njs_vm_t *vm,
-    njs_property_query_t *pq, njs_array_t *array, uint32_t index);
+    njs_property_query_t *pq, njs_array_t *array, uint32_t index,
+    uint32_t atom_id);
 static njs_int_t njs_typed_array_property_query(njs_vm_t *vm,
     njs_property_query_t *pq, njs_typed_array_t *array, uint32_t index);
 static njs_int_t njs_string_property_query(njs_vm_t *vm,
@@ -29,35 +30,6 @@ const njs_value_t  njs_value_zero =         njs_value(NJS_NUMBER, 0, 0.0);
 const njs_value_t  njs_value_nan =          njs_value(NJS_NUMBER, 0, NAN);
 const njs_value_t  njs_value_invalid =      njs_value(NJS_INVALID, 0, 0.0);
 
-const njs_value_t  njs_string_empty =       njs_string("");
-const njs_value_t  njs_string_empty_regexp =
-                                            njs_string("(?:)");
-const njs_value_t  njs_string_comma =       njs_string(",");
-const njs_value_t  njs_string_null =        njs_string("null");
-const njs_value_t  njs_string_undefined =   njs_string("undefined");
-const njs_value_t  njs_string_boolean =     njs_string("boolean");
-const njs_value_t  njs_string_false =       njs_string("false");
-const njs_value_t  njs_string_true =        njs_string("true");
-const njs_value_t  njs_string_number =      njs_string("number");
-const njs_value_t  njs_string_minus_zero =  njs_string("-0");
-const njs_value_t  njs_string_minus_infinity =
-                                            njs_string("-Infinity");
-const njs_value_t  njs_string_plus_infinity =
-                                            njs_string("Infinity");
-const njs_value_t  njs_string_nan =         njs_string("NaN");
-const njs_value_t  njs_string_symbol =      njs_string("symbol");
-const njs_value_t  njs_string_string =      njs_string("string");
-const njs_value_t  njs_string_name =        njs_string("name");
-const njs_value_t  njs_string_data =        njs_string("data");
-const njs_value_t  njs_string_type =        njs_string("type");
-const njs_value_t  njs_string_external =    njs_string("external");
-const njs_value_t  njs_string_invalid =     njs_string("invalid");
-const njs_value_t  njs_string_object =      njs_string("object");
-const njs_value_t  njs_string_function =    njs_string("function");
-const njs_value_t  njs_string_anonymous =   njs_string("anonymous");
-const njs_value_t  njs_string_memory_error = njs_string("MemoryError");
-
-
 /*
  * A hint value is 0 for numbers and 1 for strings.  The value chooses
  * method calls order specified by ECMAScript 5.1: "valueOf", "toString"
@@ -68,22 +40,16 @@ njs_int_t
 njs_value_to_primitive(njs_vm_t *vm, njs_value_t *dst, njs_value_t *value,
     njs_uint_t hint)
 {
-    njs_int_t           ret;
-    njs_uint_t          tries;
-    njs_value_t         method, retval;
-    njs_lvlhsh_query_t  lhq;
+    njs_int_t            ret;
+    njs_uint_t           tries;
+    njs_value_t          method, retval;
+    njs_flathsh_query_t  lhq;
 
-    static const uint32_t  hashes[] = {
-        NJS_VALUE_OF_HASH,
-        NJS_TO_STRING_HASH,
+    static const uint32_t atoms[] = {
+        NJS_ATOM_STRING_valueOf,
+        NJS_ATOM_STRING_toString,
     };
 
-    static const njs_str_t  names[] = {
-        njs_str("valueOf"),
-        njs_str("toString"),
-    };
-
-
     if (njs_is_primitive(value)) {
         *dst = *value;
         return NJS_OK;
@@ -98,8 +64,7 @@ njs_value_to_primitive(njs_vm_t *vm, njs_value_t *dst, njs_value_t *value,
         if (njs_is_object(value) && tries < 2) {
             hint ^= tries++;
 
-            lhq.key_hash = hashes[hint];
-            lhq.key = names[hint];
+            lhq.key_hash = atoms[hint];
 
             ret = njs_object_property(vm, njs_object(value), &lhq, &method);
 
@@ -207,7 +172,7 @@ njs_value_own_enumerate(njs_vm_t *vm, njs_value_t *value, uint32_t flags)
             dst = values->start;
 
             while (k < end) {
-                ret = njs_value_property(vm, value, k++, dst++);
+                ret = njs_value_property_val(vm, value, k++, dst++);
                 if (njs_slow_path(ret != NJS_OK)) {
                     return NULL;
                 }
@@ -234,7 +199,7 @@ njs_value_own_enumerate(njs_vm_t *vm, njs_value_t *value, uint32_t flags)
                     return NULL;
                 }
 
-                ret = njs_value_property(vm, value, k, &entry->start[1]);
+                ret = njs_value_property_val(vm, value, k, &entry->start[1]);
                 if (njs_slow_path(ret != NJS_OK)) {
                     return NULL;
                 }
@@ -265,14 +230,11 @@ njs_value_of(njs_vm_t *vm, njs_value_t *value, njs_value_t *retval)
 
     njs_int_t  ret;
 
-    static const njs_value_t  value_of = njs_string("valueOf");
-
     if (njs_slow_path(!njs_is_object(value))) {
         return NJS_DECLINED;
     }
 
-    ret = njs_value_property(vm, value, njs_value_arg(&value_of),
-                             retval);
+    ret = njs_value_property(vm, value, NJS_ATOM_STRING_valueOf, retval);
     if (njs_slow_path(ret != NJS_OK)) {
         return ret;
     }
@@ -292,7 +254,7 @@ njs_value_length(njs_vm_t *vm, njs_value_t *value, int64_t *length)
     njs_string_prop_t  string_prop;
 
     if (njs_is_string(value)) {
-        *length = njs_string_prop(&string_prop, value);
+        *length = njs_string_prop(vm, &string_prop, value);
 
     } else if (njs_is_primitive(value)) {
         *length = 0;
@@ -595,15 +557,13 @@ njs_value_is_data_view(const njs_value_t *value)
 
 njs_int_t
 njs_property_query(njs_vm_t *vm, njs_property_query_t *pq, njs_value_t *value,
-    njs_value_t *key)
+    uint32_t atom_id)
 {
-    double          num;
     uint32_t        index;
     njs_int_t       ret;
     njs_object_t    *obj;
-    njs_function_t  *function;
 
-    njs_assert(njs_is_index_or_key(key));
+    njs_assert(atom_id != NJS_ATOM_STRING_unknown);
 
     switch (value->type) {
     case NJS_BOOLEAN:
@@ -614,11 +574,9 @@ njs_property_query(njs_vm_t *vm, njs_property_query_t *pq, njs_value_t *value,
         break;
 
     case NJS_STRING:
-        if (njs_fast_path(!njs_is_null_or_undefined_or_boolean(key))) {
-            num = njs_key_to_index(key);
-            if (njs_fast_path(njs_key_is_integer_index(num, key))) {
-                return njs_string_property_query(vm, pq, value, num);
-            }
+        if (njs_atom_is_number(atom_id)) {
+            return njs_string_property_query(vm, pq, value,
+                                             njs_atom_number(atom_id));
         }
 
         obj = &vm->string_object;
@@ -626,6 +584,7 @@ njs_property_query(njs_vm_t *vm, njs_property_query_t *pq, njs_value_t *value,
 
     case NJS_OBJECT:
     case NJS_ARRAY:
+    case NJS_FUNCTION:
     case NJS_ARRAY_BUFFER:
     case NJS_DATA_VIEW:
     case NJS_TYPED_ARRAY:
@@ -636,69 +595,33 @@ njs_property_query(njs_vm_t *vm, njs_property_query_t *pq, njs_value_t *value,
         obj = njs_object(value);
         break;
 
-    case NJS_FUNCTION:
-        function = njs_function_value_copy(vm, value);
-        if (njs_slow_path(function == NULL)) {
-            return NJS_ERROR;
-        }
-
-        obj = &function->object;
-        break;
-
     case NJS_UNDEFINED:
     case NJS_NULL:
     default:
-        ret = njs_primitive_value_to_string(vm, &pq->key, key);
-
-        if (njs_fast_path(ret == NJS_OK)) {
-            njs_string_get(&pq->key, &pq->lhq.key);
-            njs_type_error(vm, "cannot get property \"%V\" of %s",
-                           &pq->lhq.key, njs_is_null(value) ? "null"
-                                                            : "undefined");
-            return NJS_ERROR;
-        }
-
-        njs_type_error(vm, "cannot get property \"unknown\" of %s",
-                       njs_is_null(value) ? "null" : "undefined");
-
+        njs_atom_string_get(vm, atom_id, &pq->lhq.key);
+        njs_type_error(vm, "cannot get property \"%V\" of %s", &pq->lhq.key,
+                       njs_type_string(value->type));
         return NJS_ERROR;
     }
 
-    ret = njs_primitive_value_to_key(vm, &pq->key, key);
+    ret = njs_object_property_query(vm, pq, obj, atom_id);
 
-    if (njs_fast_path(ret == NJS_OK)) {
-
-        if (njs_is_symbol(key)) {
-            pq->lhq.key_hash = njs_symbol_key(key);
-            pq->lhq.key.start = NULL;
-
-        } else {
-            njs_string_get(&pq->key, &pq->lhq.key);
-
-            if (pq->lhq.key_hash == 0) {
-                pq->lhq.key_hash = njs_djb_hash(pq->lhq.key.start,
-                                                pq->lhq.key.length);
-            }
-        }
-
-        ret = njs_object_property_query(vm, pq, obj, key);
-
-        if (njs_slow_path(ret == NJS_DECLINED && obj->slots != NULL)) {
-            return njs_external_property_query(vm, pq, value);
-        }
+    if (njs_slow_path(ret == NJS_DECLINED && obj->slots != NULL)) {
+        return njs_external_property_query(vm, pq, value);
     }
 
     return ret;
 }
 
 
-static njs_int_t
+njs_inline njs_int_t
 njs_object_property_query(njs_vm_t *vm, njs_property_query_t *pq,
-    njs_object_t *object, const njs_value_t *key)
+    njs_object_t *object, uint32_t atom_id)
 {
     double              num;
     njs_int_t           ret;
     njs_bool_t          own;
+    njs_value_t         key;
     njs_array_t         *array;
     njs_object_t        *proto;
     njs_object_prop_t   *prop;
@@ -716,10 +639,24 @@ njs_object_property_query(njs_vm_t *vm, njs_property_query_t *pq,
         switch (proto->type) {
         case NJS_ARRAY:
             array = (njs_array_t *) proto;
-            num = njs_key_to_index(key);
 
-            if (njs_fast_path(njs_key_is_integer_index(num, key))) {
-                ret = njs_array_property_query(vm, pq, array, num);
+            if (njs_fast_path(njs_atom_is_number(atom_id))) {
+                ret = njs_array_property_query(vm, pq, array,
+                                               njs_atom_number(atom_id), atom_id);
+                if (njs_fast_path(ret != NJS_DECLINED)) {
+                    return (ret == NJS_DONE) ? NJS_DECLINED : ret;
+                }
+            }
+
+            ret = njs_atom_to_value(vm, &key, atom_id);
+            if (njs_slow_path(ret != NJS_OK)) {
+                return ret;
+            }
+
+            num = njs_key_to_index(&key);
+
+            if (njs_key_is_integer_index(num, &key)) {
+                ret = njs_array_property_query(vm, pq, array, num, atom_id);
                 if (njs_fast_path(ret != NJS_DECLINED)) {
                     return (ret == NJS_DONE) ? NJS_DECLINED : ret;
                 }
@@ -728,12 +665,19 @@ njs_object_property_query(njs_vm_t *vm, njs_property_query_t *pq,
             break;
 
         case NJS_TYPED_ARRAY:
-            num = njs_key_to_index(key);
-            if (njs_fast_path(njs_key_is_integer_index(num, key))) {
+            if (njs_fast_path(njs_atom_is_number(atom_id))) {
                 tarray = (njs_typed_array_t *) proto;
-                return njs_typed_array_property_query(vm, pq, tarray, num);
+                return njs_typed_array_property_query(vm, pq, tarray,
+                                                      njs_atom_number(atom_id));
             }
 
+            ret = njs_atom_to_value(vm, &key, atom_id);
+            if (njs_slow_path(ret != NJS_OK)) {
+                return ret;
+            }
+
+            num = njs_key_to_index(&key);
+
             if (!isnan(num)) {
                 return NJS_DECLINED;
             }
@@ -746,10 +690,10 @@ njs_object_property_query(njs_vm_t *vm, njs_property_query_t *pq,
                 break;
             }
 
-            num = njs_key_to_index(key);
-            if (njs_fast_path(njs_key_is_integer_index(num, key))) {
+            if (njs_fast_path(njs_atom_is_number(atom_id))) {
                 ov = (njs_object_value_t *) proto;
-                ret = njs_string_property_query(vm, pq, &ov->value, num);
+                ret = njs_string_property_query(vm, pq, &ov->value,
+                                                njs_atom_number(atom_id));
                 if (njs_fast_path(ret != NJS_DECLINED)) {
                     return ret;
                 }
@@ -761,7 +705,9 @@ njs_object_property_query(njs_vm_t *vm, njs_property_query_t *pq,
             break;
         }
 
-        ret = njs_lvlhsh_find(&proto->hash, &pq->lhq);
+        pq->lhq.key_hash = atom_id;
+
+        ret = njs_flathsh_unique_find(&proto->hash, &pq->lhq);
 
         if (ret == NJS_OK) {
             prop = pq->lhq.value;
@@ -775,8 +721,7 @@ njs_object_property_query(njs_vm_t *vm, njs_property_query_t *pq,
             }
 
         } else {
-            ret = njs_lvlhsh_find(&proto->shared_hash, &pq->lhq);
-
+            ret = njs_flathsh_unique_find(&proto->shared_hash, &pq->lhq);
             if (ret == NJS_OK) {
                 return njs_prop_private_copy(vm, pq, proto);
             }
@@ -797,7 +742,7 @@ njs_object_property_query(njs_vm_t *vm, njs_property_query_t *pq,
 
 static njs_int_t
 njs_array_property_query(njs_vm_t *vm, njs_property_query_t *pq,
-    njs_array_t *array, uint32_t index)
+    njs_array_t *array, uint32_t index, uint32_t atom_id)
 {
     int64_t            length;
     uint64_t           size;
@@ -858,7 +803,9 @@ njs_array_property_query(njs_vm_t *vm, njs_property_query_t *pq,
             }
         }
 
-        ret = njs_lvlhsh_find(&array->object.hash, &pq->lhq);
+        pq->lhq.key_hash = atom_id;
+
+        ret = njs_flathsh_unique_find(&array->object.hash, &pq->lhq);
         if (ret == NJS_OK) {
             prop = pq->lhq.value;
 
@@ -901,8 +848,6 @@ prop:
         prop->type = resized ? NJS_PROPERTY_PLACE_REF : NJS_PROPERTY_REF;
     }
 
-    njs_set_number(&prop->name, index);
-
     prop->writable = 1;
     prop->enumerable = 1;
     prop->configurable = 1;
@@ -963,7 +908,7 @@ njs_string_property_query(njs_vm_t *vm, njs_property_query_t *pq,
 
     slice.start = index;
     slice.length = 1;
-    slice.string_length = njs_string_prop(&string, object);
+    slice.string_length = njs_string_prop(vm, &string, object);
 
     if (slice.start < slice.string_length) {
         /*
@@ -979,12 +924,6 @@ njs_string_property_query(njs_vm_t *vm, njs_property_query_t *pq,
 
         pq->lhq.value = prop;
 
-        if (pq->query != NJS_PROPERTY_QUERY_GET) {
-            /* pq->lhq.key is used by NJS_VMCODE_PROPERTY_SET for TypeError */
-            njs_uint32_to_string(&pq->key, index);
-            njs_string_get(&pq->key, &pq->lhq.key);
-        }
-
         return NJS_OK;
     }
 
@@ -1018,10 +957,10 @@ njs_external_property_query(njs_vm_t *vm, njs_property_query_t *pq,
      */
 
     njs_prop_magic32(prop) = slots->magic32;
-    prop->name = pq->key;
 
     pq->lhq.value = prop;
 
+    prop->type = NJS_PROPERTY;
     prop->writable = slots->writable;
     prop->configurable = slots->configurable;
     prop->enumerable = slots->enumerable;
@@ -1029,7 +968,8 @@ njs_external_property_query(njs_vm_t *vm, njs_property_query_t *pq,
     switch (pq->query) {
 
     case NJS_PROPERTY_QUERY_GET:
-        return slots->prop_handler(vm, prop, value, NULL, njs_prop_value(prop));
+        return slots->prop_handler(vm, prop, pq->lhq.key_hash, value, NULL,
+                                   njs_prop_value(prop));
 
     case NJS_PROPERTY_QUERY_SET:
         if (slots->writable == 0) {
@@ -1054,10 +994,9 @@ njs_external_property_query(njs_vm_t *vm, njs_property_query_t *pq,
 
 
 njs_int_t
-njs_value_property(njs_vm_t *vm, njs_value_t *value, njs_value_t *key,
+njs_value_property(njs_vm_t *vm, njs_value_t *value, uint32_t atom_id,
     njs_value_t *retval)
 {
-    double                num;
     uint32_t              index;
     njs_int_t             ret;
     njs_array_t          *array;
@@ -1065,23 +1004,14 @@ njs_value_property(njs_vm_t *vm, njs_value_t *value, njs_value_t *key,
     njs_typed_array_t     *tarray;
     njs_property_query_t  pq;
 
-    njs_assert(njs_is_index_or_key(key));
-
-    if (njs_fast_path(njs_is_number(key))) {
-        num = njs_number(key);
-
-        if (njs_slow_path(!njs_number_is_integer_index(num))) {
-            goto slow_path;
-        }
-
-        index = (uint32_t) num;
+    if (njs_fast_path(njs_atom_is_number(atom_id))) {
+        index = njs_atom_number(atom_id);
 
         if (njs_is_typed_array(value)) {
             tarray = njs_typed_array(value);
 
             if (njs_slow_path(njs_is_detached_buffer(tarray->buffer))) {
-                njs_type_error(vm, "detached buffer");
-                return NJS_ERROR;
+                goto not_found;
             }
 
             if (njs_slow_path(index >= njs_typed_array_length(tarray))) {
@@ -1116,9 +1046,9 @@ njs_value_property(njs_vm_t *vm, njs_value_t *value, njs_value_t *key,
 
 slow_path:
 
-    njs_property_query_init(&pq, NJS_PROPERTY_QUERY_GET, 0, 0);
+    njs_property_query_init(&pq, NJS_PROPERTY_QUERY_GET, 0);
 
-    ret = njs_property_query(vm, &pq, value, key);
+    ret = njs_property_query(vm, &pq, value, atom_id);
 
     switch (ret) {
 
@@ -1144,7 +1074,7 @@ slow_path:
         case NJS_PROPERTY_HANDLER:
             pq.scratch = *prop;
             prop = &pq.scratch;
-            ret = njs_prop_handler(prop)(vm, prop, value, NULL,
+            ret = njs_prop_handler(prop)(vm, prop, atom_id, value, NULL,
                                          njs_prop_value(prop));
 
             if (njs_slow_path(ret != NJS_OK)) {
@@ -1170,6 +1100,7 @@ slow_path:
         break;
 
     case NJS_DECLINED:
+not_found:
         njs_set_undefined(retval);
 
         return NJS_DECLINED;
@@ -1185,32 +1116,21 @@ slow_path:
 
 
 njs_int_t
-njs_value_property_set(njs_vm_t *vm, njs_value_t *value, njs_value_t *key,
+njs_value_property_set(njs_vm_t *vm, njs_value_t *value, uint32_t atom_id,
     njs_value_t *setval)
 {
-    double                num;
     uint32_t              index;
     njs_int_t             ret;
     njs_array_t           *array;
-    njs_value_t           retval;
-    njs_flathsh_elt_t     *elt;
+    njs_value_t           retval, key;
     njs_object_prop_t     *prop;
     njs_typed_array_t     *tarray;
+    njs_flathsh_elt_t     *elt;
     njs_flathsh_descr_t   *h;
     njs_property_query_t  pq;
 
-    static const njs_str_t  length_key = njs_str("length");
-
-    njs_assert(njs_is_index_or_key(key));
-
-    if (njs_fast_path(njs_is_number(key))) {
-        num = njs_number(key);
-
-        if (njs_slow_path(!njs_number_is_integer_index(num))) {
-            goto slow_path;
-        }
-
-        index = (uint32_t) num;
+    if (njs_fast_path(njs_atom_is_number(atom_id))) {
+        index = njs_atom_number(atom_id);
 
         if (njs_is_typed_array(value)) {
             tarray = njs_typed_array(value);
@@ -1249,9 +1169,9 @@ slow_path:
         return NJS_ERROR;
     }
 
-    njs_property_query_init(&pq, NJS_PROPERTY_QUERY_SET, 0, 0);
+    njs_property_query_init(&pq, NJS_PROPERTY_QUERY_SET, 0);
 
-    ret = njs_property_query(vm, &pq, value, key);
+    ret = njs_property_query(vm, &pq, value, atom_id);
 
     switch (ret) {
 
@@ -1260,7 +1180,7 @@ slow_path:
 
         if (njs_is_data_descriptor(prop)) {
             if (!prop->writable) {
-                njs_key_string_get(vm, &pq.key,  &pq.lhq.key);
+                njs_atom_string_get(vm, atom_id, &pq.lhq.key);
                 njs_type_error(vm,
                              "Cannot assign to read-only property \"%V\" of %s",
                                &pq.lhq.key, njs_type_string(value->type));
@@ -1273,7 +1193,7 @@ slow_path:
                                          value, setval, 1, &retval);
             }
 
-            njs_key_string_get(vm, &pq.key,  &pq.lhq.key);
+            njs_atom_string_get(vm, atom_id, &pq.lhq.key);
             njs_type_error(vm,
                      "Cannot set property \"%V\" of %s which has only a getter",
                            &pq.lhq.key, njs_type_string(value->type));
@@ -1281,7 +1201,8 @@ slow_path:
         }
 
         if (prop->type == NJS_PROPERTY_HANDLER) {
-            ret = njs_prop_handler(prop)(vm, prop, value, setval, &retval);
+            ret = njs_prop_handler(prop)(vm, prop, atom_id, value,
+                                         setval, &retval);
             if (njs_slow_path(ret != NJS_DECLINED)) {
                 return ret;
             }
@@ -1291,10 +1212,8 @@ slow_path:
             switch (prop->type) {
             case NJS_PROPERTY:
                 if (njs_is_array(value)) {
-                    if (njs_slow_path(pq.lhq.key_hash == NJS_LENGTH_HASH)) {
-                        if (njs_strstr_eq(&pq.lhq.key, &length_key)) {
-                            return njs_array_length_set(vm, value, prop, setval);
-                        }
+                    if (njs_slow_path(atom_id == NJS_ATOM_STRING_length)) {
+                        return njs_array_length_set(vm, value, prop, setval);
                     }
                 }
 
@@ -1340,7 +1259,7 @@ slow_path:
 
             pq.lhq.pool = vm->mem_pool;
 
-            int rc = njs_lvlhsh_delete(pq.own_whiteout, &pq.lhq);
+            int rc = njs_flathsh_unique_delete(pq.own_whiteout, &pq.lhq);
             if (rc != NJS_OK) {
                 return NJS_ERROR;
             }
@@ -1361,10 +1280,7 @@ slow_path:
                 return NJS_ERROR;
             }
 
-            elt->value = (&pq.lhq)->value;
-
-            prop = (njs_object_prop_t *) elt->value;
-
+            prop = (njs_object_prop_t *) elt;
             prop->type = NJS_PROPERTY;
             prop->enumerable = 1;
             prop->configurable = 1;
@@ -1374,10 +1290,16 @@ slow_path:
         }
 
         if (njs_slow_path(pq.own && njs_is_typed_array(value)
-                          && njs_is_string(key)))
+                          && !njs_atom_is_number(atom_id)))
         {
             /* Integer-Indexed Exotic Objects [[DefineOwnProperty]]. */
-            if (!isnan(njs_string_to_index(key))) {
+
+            ret = njs_atom_to_value(vm, &key, atom_id);
+            if (njs_slow_path(ret != NJS_OK)) {
+                return ret;
+            }
+
+            if (!isnan(njs_string_to_index(&key))) {
                 return NJS_OK;
             }
         }
@@ -1394,21 +1316,23 @@ slow_path:
         goto fail;
     }
 
-    prop = njs_object_prop_alloc(vm, &pq.key, &njs_value_undefined, 1);
-    if (njs_slow_path(prop == NULL)) {
-        return NJS_ERROR;
-    }
 
     pq.lhq.replace = 0;
-    pq.lhq.value = prop;
+    pq.lhq.key_hash = atom_id;
     pq.lhq.pool = vm->mem_pool;
 
-    ret = njs_lvlhsh_insert(njs_object_hash(value), &pq.lhq);
+    ret = njs_flathsh_unique_insert(njs_object_hash(value), &pq.lhq);
     if (njs_slow_path(ret != NJS_OK)) {
         njs_internal_error(vm, "lvlhsh insert failed");
         return NJS_ERROR;
     }
 
+    prop = pq.lhq.value;
+    prop->type = NJS_PROPERTY;
+    prop->enumerable = 1;
+    prop->configurable = 1;
+    prop->writable = 1;
+
 found:
 
     njs_value_assign(njs_prop_value(prop), setval);
@@ -1417,7 +1341,7 @@ found:
 
 fail:
 
-    njs_key_string_get(vm, &pq.key, &pq.lhq.key);
+    njs_atom_string_get(vm, atom_id, &pq.lhq.key);
     njs_type_error(vm, "Cannot add property \"%V\", object is not extensible",
                    &pq.lhq.key);
 
@@ -1426,30 +1350,21 @@ fail:
 
 
 njs_int_t
-njs_value_property_delete(njs_vm_t *vm, njs_value_t *value, njs_value_t *key,
+njs_value_property_delete(njs_vm_t *vm, njs_value_t *value, uint32_t atom_id,
     njs_value_t *removed, njs_bool_t thrw)
 {
-    double                num;
     uint32_t              index;
     njs_int_t             ret;
     njs_array_t           *array;
     njs_object_prop_t     *prop;
     njs_property_query_t  pq;
 
-    njs_assert(njs_is_index_or_key(key));
-
-    if (njs_fast_path(njs_is_number(key))) {
+    if (njs_fast_path(njs_atom_is_number(atom_id))) {
         if (njs_slow_path(!(njs_is_fast_array(value)))) {
             goto slow_path;
         }
 
-        num = njs_number(key);
-
-        if (njs_slow_path(!njs_number_is_integer_index(num))) {
-            goto slow_path;
-        }
-
-        index = (uint32_t) num;
+        index = (uint32_t) njs_atom_number(atom_id);
 
         array = njs_array(value);
 
@@ -1464,9 +1379,9 @@ njs_value_property_delete(njs_vm_t *vm, njs_value_t *value, njs_value_t *key,
 
 slow_path:
 
-    njs_property_query_init(&pq, NJS_PROPERTY_QUERY_DELETE, 0, 1);
+    njs_property_query_init(&pq, NJS_PROPERTY_QUERY_DELETE, 1);
 
-    ret = njs_property_query(vm, &pq, value, key);
+    ret = njs_property_query(vm, &pq, value, atom_id);
     if (njs_slow_path(ret != NJS_OK)) {
         return ret;
     }
@@ -1475,7 +1390,7 @@ slow_path:
 
     if (njs_slow_path(!prop->configurable)) {
         if (thrw) {
-            njs_key_string_get(vm, &pq.key,  &pq.lhq.key);
+            njs_atom_string_get(vm, atom_id, &pq.lhq.key);
             njs_type_error(vm, "Cannot delete property \"%V\" of %s",
                            &pq.lhq.key, njs_type_string(value->type));
             return NJS_ERROR;
@@ -1487,7 +1402,8 @@ slow_path:
     switch (prop->type) {
     case NJS_PROPERTY_HANDLER:
         if (njs_is_object(value) && njs_object_slots(value) != NULL) {
-            ret = njs_prop_handler(prop)(vm, prop, value, NULL, NULL);
+            ret = njs_prop_handler(prop)(vm, prop, atom_id, value, NULL,
+                                         NULL);
             if (njs_slow_path(ret != NJS_DECLINED)) {
                 return ret;
             }
@@ -1535,7 +1451,6 @@ slow_path:
     }
 
     prop->type = NJS_WHITEOUT;
-    prop->enum_in_object_hash = 1;
 
     return NJS_OK;
 }
@@ -1550,16 +1465,22 @@ njs_primitive_value_to_string(njs_vm_t *vm, njs_value_t *dst,
     switch (src->type) {
 
     case NJS_NULL:
-        value = &njs_string_null;
-        break;
+        njs_atom_to_value(vm, dst, NJS_ATOM_STRING_null);
+        return NJS_OK;
 
     case NJS_UNDEFINED:
-        value = &njs_string_undefined;
-        break;
+        njs_atom_to_value(vm, dst, NJS_ATOM_STRING_undefined);
+        return NJS_OK;
 
     case NJS_BOOLEAN:
-        value = njs_is_true(src) ? &njs_string_true : &njs_string_false;
-        break;
+        if (njs_is_true(src)) {
+            njs_atom_to_value(vm, dst, NJS_ATOM_STRING_true);
+
+        } else {
+            njs_atom_to_value(vm, dst, NJS_ATOM_STRING_false);
+        }
+
+        return NJS_OK;
 
     case NJS_NUMBER:
         return njs_number_to_string(vm, dst, src);
@@ -1616,7 +1537,7 @@ njs_primitive_value_to_chain(njs_vm_t *vm, njs_chb_t *chain,
         return NJS_ERROR;
 
     case NJS_STRING:
-        (void) njs_string_prop(&string, src);
+        (void) njs_string_prop(vm, &string, src);
         njs_chb_append(chain, string.start, string.size);
         return string.length;
 
@@ -1712,11 +1633,7 @@ njs_value_species_constructor(njs_vm_t *vm, njs_value_t *object,
     njs_int_t    ret;
     njs_value_t  constructor, retval;
 
-    static const njs_value_t  string_constructor = njs_string("constructor");
-    static const njs_value_t  string_species =
-                                njs_wellknown_symbol(NJS_SYMBOL_SPECIES);
-
-    ret = njs_value_property(vm, object, njs_value_arg(&string_constructor),
+    ret = njs_value_property(vm, object, NJS_ATOM_STRING_constructor,
                              &constructor);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
@@ -1731,7 +1648,7 @@ njs_value_species_constructor(njs_vm_t *vm, njs_value_t *object,
         return NJS_ERROR;
     }
 
-    ret = njs_value_property(vm, &constructor, njs_value_arg(&string_species),
+    ret = njs_value_property(vm, &constructor, NJS_ATOM_SYMBOL_species,
                              &retval);
     if (njs_slow_path(ret == NJS_ERROR)) {
         return NJS_ERROR;
@@ -1759,7 +1676,7 @@ default_constructor:
 
 
 njs_int_t
-njs_value_method(njs_vm_t *vm, njs_value_t *value, njs_value_t *key,
+njs_value_method(njs_vm_t *vm, njs_value_t *value, uint32_t atom_id,
     njs_value_t *retval)
 {
     njs_int_t  ret;
@@ -1769,7 +1686,7 @@ njs_value_method(njs_vm_t *vm, njs_value_t *value, njs_value_t *key,
         return ret;
     }
 
-    ret = njs_value_property(vm, value, key, retval);
+    ret = njs_value_property(vm, value, atom_id, retval);
     if (njs_slow_path(ret != NJS_OK)) {
         return (ret == NJS_DECLINED) ? NJS_OK : ret;
     }
