@@ -138,8 +138,6 @@ njs_vm_create(njs_vm_opt_t *options)
         }
     }
 
-    vm->symbol_generator = NJS_SYMBOL_KNOWN_MAX;
-
     if (njs_scope_undefined_index(vm, 0) == NJS_INDEX_ERROR) {
         return NULL;
     }
@@ -222,7 +220,7 @@ njs_vm_compile(njs_vm_t *vm, u_char **start, u_char *end)
     global_items = (vm->global_scope != NULL) ? vm->global_scope->items : 0;
 
     ret = njs_parser_init(vm, &parser, vm->global_scope, &vm->options.file,
-                          *start, end, 0);
+                          *start, end);
     if (njs_slow_path(ret != NJS_OK)) {
         return NJS_ERROR;
     }
@@ -233,7 +231,7 @@ njs_vm_compile(njs_vm_t *vm, u_char **start, u_char *end)
     }
 
     if (njs_slow_path(vm->options.ast)) {
-        NJS_CHB_MP_INIT(&chain, vm);
+        NJS_CHB_MP_INIT(&chain, njs_vm_memory_pool(vm));
         ret = njs_parser_serialize_ast(parser.node, &chain);
         if (njs_slow_path(ret == NJS_ERROR)) {
             return ret;
@@ -312,7 +310,6 @@ njs_vm_compile_module(njs_vm_t *vm, njs_str_t *name, u_char **start,
     u_char *end)
 {
     njs_int_t              ret;
-    njs_arr_t              *arr;
     njs_mod_t              *module;
     njs_parser_t           parser;
     njs_vm_code_t          *code;
@@ -330,7 +327,7 @@ njs_vm_compile_module(njs_vm_t *vm, njs_str_t *name, u_char **start,
         return NULL;
     }
 
-    ret = njs_parser_init(vm, &parser, NULL, name, *start, end, 1);
+    ret = njs_parser_init(vm, &parser, NULL, name, *start, end);
     if (njs_slow_path(ret != NJS_OK)) {
         return NULL;
     }
@@ -368,10 +365,6 @@ njs_vm_compile_module(njs_vm_t *vm, njs_str_t *name, u_char **start,
     lambda->start = generator.code_start;
     lambda->nlocal = scope->items;
 
-    arr = scope->declarations;
-    lambda->declarations = (arr != NULL) ? arr->start : NULL;
-    lambda->ndeclarations = (arr != NULL) ? arr->items : 0;
-
     module->function.u.lambda = lambda;
 
     return module;
@@ -419,6 +412,11 @@ njs_vm_clone(njs_vm_t *vm, njs_external_ptr_t external)
     nvm->trace.data = nvm;
     nvm->external = external;
 
+    nvm->shared_atom_count = vm->atom_id_generator;
+
+    njs_lvlhsh_init(&nvm->atom_hash);
+    nvm->atom_hash_current = &nvm->atom_hash;
+
     ret = njs_vm_runtime_init(nvm);
     if (njs_slow_path(ret != NJS_OK)) {
         goto fail;
@@ -495,7 +493,7 @@ njs_vm_runtime_init(njs_vm_t *vm)
     }
 
     njs_lvlhsh_init(&vm->values_hash);
-    njs_lvlhsh_init(&vm->keywords_hash);
+
     njs_lvlhsh_init(&vm->modules_hash);
 
     njs_rbtree_init(&vm->global_symbols, njs_symbol_rbtree_cmp);
@@ -874,12 +872,13 @@ njs_vm_value(njs_vm_t *vm, const njs_str_t *path, njs_value_t *retval)
             return NJS_ERROR;
         }
 
-        ret = njs_string_create(vm, &key, start, size);
+        ret = njs_atom_string_create(vm, &key, start, size);
         if (njs_slow_path(ret != NJS_OK)) {
             return NJS_ERROR;
         }
 
-        ret = njs_value_property(vm, &value, &key, njs_value_arg(retval));
+        ret = njs_value_property(vm, &value, key.atom_id,
+                                 njs_value_arg(retval));
         if (njs_slow_path(ret == NJS_ERROR)) {
             return ret;
         }
@@ -900,19 +899,20 @@ static njs_int_t
 njs_vm_bind2(njs_vm_t *vm, const njs_str_t *var_name, njs_object_prop_t *prop,
     njs_bool_t shared)
 {
-    njs_int_t           ret;
-    njs_object_t        *global;
-    njs_lvlhsh_t        *hash;
-    njs_lvlhsh_query_t  lhq;
+    njs_int_t            ret;
+    njs_value_t          prop_name;
+    njs_object_t         *global;
+    njs_flathsh_t        *hash;
+    njs_object_prop_t    *obj_prop;
+    njs_flathsh_query_t  lhq;
 
-    ret = njs_string_create(vm, &prop->name, var_name->start, var_name->length);
+    ret = njs_atom_string_create(vm, &prop_name, var_name->start,
+                                 var_name->length);
     if (njs_slow_path(ret != NJS_OK)) {
         return NJS_ERROR;
     }
 
-    lhq.value = prop;
-    lhq.key = *var_name;
-    lhq.key_hash = njs_djb_hash(lhq.key.start, lhq.key.length);
+    lhq.key_hash = prop_name.atom_id;
     lhq.replace = 1;
     lhq.pool = vm->mem_pool;
     lhq.proto = &njs_object_hash_proto;
@@ -920,12 +920,20 @@ njs_vm_bind2(njs_vm_t *vm, const njs_str_t *var_name, njs_object_prop_t *prop,
     global = &vm->global_object;
     hash = shared ? &global->shared_hash : &global->hash;
 
-    ret = njs_lvlhsh_insert(hash, &lhq);
+    ret = njs_flathsh_unique_insert(hash, &lhq);
     if (njs_slow_path(ret != NJS_OK)) {
         njs_internal_error(vm, "lvlhsh insert failed");
         return ret;
     }
 
+    obj_prop = lhq.value;
+
+    obj_prop->type = prop->type;
+    obj_prop->enumerable = prop->enumerable;
+    obj_prop->configurable = prop->configurable;
+    obj_prop->writable = prop->writable;
+    obj_prop->u.value = prop->u.value;
+
     return NJS_OK;
 }
 
@@ -934,14 +942,13 @@ njs_int_t
 njs_vm_bind(njs_vm_t *vm, const njs_str_t *var_name, const njs_value_t *value,
     njs_bool_t shared)
 {
-    njs_object_prop_t   *prop;
+    njs_object_prop_t  prop;
 
-    prop = njs_object_prop_alloc(vm, &njs_value_undefined, value, 1);
-    if (njs_slow_path(prop == NULL)) {
-        return NJS_ERROR;
-    }
+    njs_object_prop_init(&prop, NJS_PROPERTY,
+                         NJS_OBJECT_PROP_VALUE_ECW);
+    *njs_prop_value(&prop) = *value;
 
-    return njs_vm_bind2(vm, var_name, prop, shared);
+    return njs_vm_bind2(vm, var_name, &prop, shared);
 }
 
 
@@ -950,29 +957,25 @@ njs_vm_bind_handler(njs_vm_t *vm, const njs_str_t *var_name,
     njs_prop_handler_t handler, uint16_t magic16, uint32_t magic32,
     njs_bool_t shared)
 {
-    njs_object_prop_t  *prop;
+    njs_object_prop_t  prop;
 
-    prop = njs_object_prop_alloc(vm, &njs_string_empty,
-                                 &njs_value_invalid, 1);
-    if (njs_slow_path(prop == NULL)) {
-        return NJS_ERROR;
-    }
+    njs_object_prop_init(&prop, NJS_PROPERTY_HANDLER,
+                         NJS_OBJECT_PROP_VALUE_ECW);
 
-    prop->type = NJS_PROPERTY_HANDLER;
-    prop->u.value.type = NJS_INVALID;
-    prop->u.value.data.truth = 1;
-    njs_prop_magic16(prop) = magic16;
-    njs_prop_magic32(prop) = magic32;
-    njs_prop_handler(prop) = handler;
+    prop.u.value.type = NJS_INVALID;
+    prop.u.value.data.truth = 1;
+    njs_prop_magic16(&prop) = magic16;
+    njs_prop_magic32(&prop) = magic32;
+    njs_prop_handler(&prop) = handler;
 
-    return njs_vm_bind2(vm, var_name, prop, shared);
+    return njs_vm_bind2(vm, var_name, &prop, shared);
 }
 
 
 void
-njs_value_string_get(njs_value_t *value, njs_str_t *dst)
+njs_value_string_get(njs_vm_t *vm, njs_value_t *value, njs_str_t *dst)
 {
-    njs_string_get(value, dst);
+    njs_string_get(vm, value, dst);
 }
 
 
@@ -1050,13 +1053,21 @@ njs_vm_prop_magic32(njs_object_prop_t *prop)
 
 
 njs_int_t
-njs_vm_prop_name(njs_vm_t *vm, njs_object_prop_t *prop, njs_str_t *dst)
+njs_vm_prop_name(njs_vm_t *vm, uint32_t atom_id, njs_str_t *dst)
 {
-    if (njs_slow_path(!njs_is_string(&prop->name))) {
+    njs_int_t    ret;
+    njs_value_t  prop_name;
+
+    ret = njs_atom_to_value(vm, &prop_name, atom_id);
+    if (ret != NJS_OK) {
         return NJS_ERROR;
     }
 
-    njs_string_get(&prop->name, dst);
+    if (njs_slow_path(!njs_is_string(&prop_name))) {
+        return NJS_ERROR;
+    }
+
+    njs_string_get(vm, &prop_name, dst);
 
     return NJS_OK;
 }
@@ -1090,7 +1101,7 @@ njs_vm_value_string(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
                       && njs_number(src) == 0
                       && signbit(njs_number(src))))
     {
-        njs_string_get(&njs_string_minus_zero, dst);
+        *dst = njs_str_value("-0");
         return NJS_OK;
     }
 
@@ -1134,22 +1145,18 @@ njs_value_t *
 njs_vm_value_enumerate(njs_vm_t *vm, njs_value_t *value, uint32_t flags,
     njs_value_t *retval)
 {
-    njs_int_t                ret;
-    njs_value_t              *val;
-    njs_array_t              *keys;
-    njs_rbtree_t             *variables;
-    njs_rbtree_node_t        *rb_node;
-    njs_variable_node_t      *node;
-    const njs_lexer_entry_t  *lex_entry;
-
-    static const njs_str_t  njs_this_str = njs_str("this");
+    njs_value_t          *val;
+    njs_array_t          *keys;
+    njs_rbtree_t         *variables;
+    njs_rbtree_node_t    *rb_node;
+    njs_variable_node_t  *node;
 
     keys = njs_value_enumerate(vm, value, flags);
     if (njs_slow_path(keys == NULL)) {
         return NULL;
     }
 
-    if (!njs_values_same(value, &vm->global_value)
+    if (!njs_values_same(vm, value, &vm->global_value)
         || vm->global_scope == NULL)
     {
         goto done;
@@ -1163,12 +1170,7 @@ njs_vm_value_enumerate(njs_vm_t *vm, njs_value_t *value, uint32_t flags,
     while (njs_rbtree_is_there_successor(variables, rb_node)) {
         node = (njs_variable_node_t *) rb_node;
 
-        lex_entry = njs_lexer_entry(node->variable->unique_id);
-        if (njs_slow_path(lex_entry == NULL)) {
-            return NULL;
-        }
-
-        if (njs_strstr_eq(&lex_entry->name, &njs_this_str)) {
+        if (node->variable->atom_id == NJS_ATOM_STRING_this) {
             rb_node = njs_rbtree_node_successor(variables, rb_node);
             continue;
         }
@@ -1178,11 +1180,7 @@ njs_vm_value_enumerate(njs_vm_t *vm, njs_value_t *value, uint32_t flags,
             return NULL;
         }
 
-        ret = njs_string_create(vm, val, lex_entry->name.start,
-                                lex_entry->name.length);
-        if (njs_slow_path(ret != NJS_OK)) {
-            return NULL;
-        }
+        njs_atom_to_value(vm, val, node->variable->atom_id);
 
         rb_node = njs_rbtree_node_successor(variables, rb_node);
     }
@@ -1215,12 +1213,12 @@ njs_vm_value_own_enumerate(njs_vm_t *vm, njs_value_t *value, uint32_t flags,
 njs_int_t
 njs_vm_object_alloc(njs_vm_t *vm, njs_value_t *retval, ...)
 {
-    va_list             args;
-    njs_int_t           ret;
-    njs_value_t         *name, *value;
-    njs_object_t        *object;
-    njs_object_prop_t   *prop;
-    njs_lvlhsh_query_t  lhq;
+    va_list              args;
+    njs_int_t            ret;
+    njs_value_t          *name, *value;
+    njs_object_t         *object;
+    njs_object_prop_t    *prop;
+    njs_flathsh_query_t  lhq;
 
     object = njs_object_alloc(vm);
     if (njs_slow_path(object == NULL)) {
@@ -1248,25 +1246,31 @@ njs_vm_object_alloc(njs_vm_t *vm, njs_value_t *retval, ...)
             goto done;
         }
 
+        if (name->atom_id == NJS_ATOM_STRING_unknown) {
+            ret = njs_atom_atomize_key(vm, name);
+            if (ret != NJS_OK) {
+                goto done;
+            }
+        }
+
+        lhq.key_hash = name->atom_id;
         lhq.replace = 0;
         lhq.pool = vm->mem_pool;
         lhq.proto = &njs_object_hash_proto;
 
-        njs_string_get(name, &lhq.key);
-        lhq.key_hash = njs_djb_hash(lhq.key.start, lhq.key.length);
-
-        prop = njs_object_prop_alloc(vm, name, value, 1);
-        if (njs_slow_path(prop == NULL)) {
-            goto done;
-        }
-
-        lhq.value = prop;
-
-        ret = njs_lvlhsh_insert(&object->hash, &lhq);
+        ret = njs_flathsh_unique_insert(&object->hash, &lhq);
         if (njs_slow_path(ret != NJS_OK)) {
             njs_internal_error(vm, NULL);
             goto done;
         }
+
+        prop = lhq.value;
+
+        prop->type = NJS_PROPERTY;
+        prop->enumerable = 1;
+        prop->configurable = 1;
+        prop->writable = 1;
+        prop->u.value = *value;
     }
 
     ret = NJS_OK;
@@ -1357,7 +1361,7 @@ njs_vm_object_prop(njs_vm_t *vm, njs_value_t *value, const njs_str_t *prop,
         return NULL;
     }
 
-    ret = njs_value_property(vm, value, &key, njs_value_arg(retval));
+    ret = njs_value_property_val(vm, value, &key, njs_value_arg(retval));
     if (njs_slow_path(ret != NJS_OK)) {
         return NULL;
     }
@@ -1378,12 +1382,13 @@ njs_vm_object_prop_set(njs_vm_t *vm, njs_value_t *value, const njs_str_t *prop,
         return NJS_ERROR;
     }
 
-    ret = njs_vm_value_string_create(vm, &key, prop->start, prop->length);
+    ret = njs_atom_string_create(vm, &key, prop->start, prop->length);
     if (njs_slow_path(ret != NJS_OK)) {
         return NJS_ERROR;
     }
 
-    ret = njs_value_property_set(vm, value, &key, njs_value_arg(setval));
+    ret = njs_value_property_set(vm, value, key.atom_id,
+                                 njs_value_arg(setval));
     if (njs_slow_path(ret != NJS_OK)) {
         return NJS_ERROR;
     }
@@ -1473,8 +1478,6 @@ njs_vm_date_alloc(njs_vm_t *vm, njs_value_t *retval, double time)
 njs_int_t
 njs_vm_value_to_string(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 {
-    u_char       *start;
-    size_t       size;
     njs_int_t    ret;
     njs_value_t  value, stack;
 
@@ -1484,7 +1487,7 @@ njs_vm_value_to_string(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 
     if (njs_is_error(src)) {
         if (njs_is_memory_error(vm, src)) {
-            njs_string_get(&njs_string_memory_error, dst);
+            *dst = njs_str_value("MemoryError");
             return NJS_OK;
         }
 
@@ -1503,24 +1506,7 @@ njs_vm_value_to_string(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
     ret = njs_value_to_string(vm, &value, &value);
 
     if (njs_fast_path(ret == NJS_OK)) {
-        size = value.short_string.size;
-
-        if (size != NJS_STRING_LONG) {
-            start = njs_mp_alloc(vm->mem_pool, size);
-            if (njs_slow_path(start == NULL)) {
-                njs_memory_error(vm);
-                return NJS_ERROR;
-            }
-
-            memcpy(start, value.short_string.start, size);
-
-        } else {
-            size = value.long_string.size;
-            start = value.long_string.data->start;
-        }
-
-        dst->length = size;
-        dst->start = start;
+        njs_string_get(vm, &value, dst);
     }
 
     return ret;
@@ -1535,24 +1521,12 @@ njs_vm_value_to_string(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 const char *
 njs_vm_value_to_c_string(njs_vm_t *vm, njs_value_t *value)
 {
-    u_char  *p, *data, *start;
+    u_char  *p, *data;
     size_t  size;
 
     njs_assert(njs_is_string(value));
 
-    if (value->short_string.size != NJS_STRING_LONG) {
-        start = value->short_string.start;
-        size = value->short_string.size;
-
-        if (size < NJS_STRING_SHORT) {
-            start[size] = '\0';
-            return (const char *) start;
-        }
-
-    } else {
-        start = value->long_string.data->start;
-        size = value->long_string.size;
-    }
+    size = value->string.data->size;
 
     data = njs_mp_alloc(vm->mem_pool, size + njs_length("\0"));
     if (njs_slow_path(data == NULL)) {
@@ -1560,7 +1534,7 @@ njs_vm_value_to_c_string(njs_vm_t *vm, njs_value_t *value)
         return NULL;
     }
 
-    p = njs_cpymem(data, start, size);
+    p = njs_cpymem(data, value->string.data->start, size);
     *p++ = '\0';
 
     return (const char *) data;
@@ -1595,8 +1569,7 @@ njs_value_to_string(njs_vm_t *vm, njs_value_t *dst, njs_value_t *value)
 njs_int_t
 njs_vm_value_to_bytes(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 {
-    u_char              *start;
-    size_t              size, length, offset;
+    size_t              length, offset;
     njs_int_t           ret;
     njs_value_t         value;
     njs_typed_array_t   *array;
@@ -1647,24 +1620,8 @@ njs_vm_value_to_bytes(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
             return NJS_ERROR;
         }
 
-        size = value.short_string.size;
-
-        if (size != NJS_STRING_LONG) {
-            start = njs_mp_alloc(vm->mem_pool, size);
-            if (njs_slow_path(start == NULL)) {
-                njs_memory_error(vm);
-                return NJS_ERROR;
-            }
-
-            memcpy(start, value.short_string.start, size);
-
-        } else {
-            size = value.long_string.size;
-            start = value.long_string.data->start;
-        }
-
-        dst->length = size;
-        dst->start = start;
+        dst->length = value.string.data->size;
+        dst->start = value.string.data->start;
     }
 
     return ret;
@@ -1672,9 +1629,10 @@ njs_vm_value_to_bytes(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 
 
 njs_int_t
-njs_vm_string_compare(const njs_value_t *v1, const njs_value_t *v2)
+njs_vm_string_compare(njs_vm_t *vm, const njs_value_t *v1,
+    const njs_value_t *v2)
 {
-    return njs_string_cmp(v1, v2);
+    return njs_string_cmp(vm, v1, v2);
 }
 
 
