@@ -4,6 +4,14 @@ function compareObjects(ref, obj) {
     }
 
     for (const key in ref) {
+        if (!Object.prototype.hasOwnProperty.call(ref, key)) {
+            continue;
+        }
+
+        if (!Object.prototype.hasOwnProperty.call(obj, key)) {
+            return false;
+        }
+
         if (!compareObjects(ref[key], obj[key])) {
             return false;
         }
@@ -13,5 +21,5 @@ function compareObjects(ref, obj) {
 }
 
 function isObject(object) {
-    return object != null && typeof object === 'object';
+    return object !== null && typeof object === 'object';
 }
