@@ -4,7 +4,7 @@ flags: []
 paths: [test/js/module/]
 ---*/
 
-var a = 42;
+globalThis.a = 42;
 import m from 'export_global_a.js';
 
 assert.sameValue(m.f(), 42);
